#include "catagory.h"
#include "../stub/catagorymgr.h"
#include "../stub/ifapp.h"
#include <qfile.h>
#include <qdatastream.h>
#include <qvaluelist.h>

#include <stdlib.h>

Catagory::Catagory()
{
    count = 0;
    head = NULL;
    modified = false;
}

Catagory::~Catagory()
{
    clear();
}

void Catagory::clear()
{
    struct catagoryinfo *i = head;
    struct catagoryinfo *tmp;
    while(i){
        tmp = i->next;
        free(i);
        i = tmp;
    }
    count = 0;
    head = NULL;
    modified = false;
}

bool Catagory::readDb()
{
    qWarning("Reading catagory database");
    bool useObseleteList = false;
    QValueList<int>obseleteList;
    clear();

    // check for obselete items in catagory mapping
    QString fileStr(pathStr + "/.catagory-mapping");
    if(!QFile::exists(fileStr)){
        qWarning("No Pixie catagory mapping in folder");
        return(true);
    }
    QFile f(fileStr);
    if(!f.open(IO_ReadOnly)){
        // TODO: error dialog
        qWarning("Unable to open catagory mapping for folder!");
        return(false);
    }
    QDataStream stream;
    stream.setDevice(&f);

    QString **catarray = kifapp()->catagoryManager()->array();
    QString tmpStr;
    unsigned char tmp;
    unsigned int id;

    while(!stream.atEnd()){
        stream >> tmpStr;
        stream >> tmp;
        id = tmp; // gcc don't like unsigned char indexes
        if(!catarray[id] || *catarray[id] != tmpStr){
            qWarning("Obselete catagory index found");
            useObseleteList = true;
            obseleteList.append(id);
        }
    }
    f.close();

    // load in catagory database
    fileStr = pathStr + "/.catagories";
    if(!QFile::exists(fileStr)){
        qWarning("No Pixie catagories in folder");
        return(false);
    }
    f.setName(fileStr);
    if(!f.open(IO_ReadOnly)){
        // TODO: error dialog
        qWarning("Unable to open catagories for folder!");
        return(false);
    }
    stream.setDevice(&f);

    struct catagoryinfo *info;
    int i;
    while(!stream.atEnd()){
        info = (struct catagoryinfo *)malloc(sizeof(struct catagoryinfo));
        info->next = head;
        info->prev = NULL;
        if(head)
            head->prev = info;
        head = info;
        stream >> info->inode;
        stream >> info->numcatagories;
        if(useObseleteList){
            for(i=0, id=0; i < info->numcatagories; ++i){
                stream >> info->catagories[id];
                if(obseleteList.findIndex(info->catagories[id]) == -1)
                    ++id;
            }
            if(id == 0){
                if(info->next){
                    info->next->prev = NULL;
                    head = info->next;
                }
                else
                    head = NULL;
                free(info);
            }
            else{
                info->numcatagories = id;
                ++count;
            }
        }
        else{
            for(i=0; i < info->numcatagories; ++i)
                stream >> info->catagories[i];
            ++count;
        }
    }
    f.close();
    qWarning("Catagory database load complete");
    return(true);
}

bool Catagory::writeDb()
{
    if(count == 0){ // no database, remove any files
        bool result = true;
        if(QFile::exists(pathStr + "/.catagories")){
            if(unlink(QString(pathStr+"/.catagories").ascii()) == -1){
                // TODO: Message Box
                qWarning("Unable to remove catagory file!");
                result = false;
            }
        }
        if(QFile::exists(pathStr + "/.catagory-mapping")){
            if(unlink(QString(pathStr+"/.catagory-mapping").ascii()) == -1){
                // TODO: Message Box
                qWarning("Unable to remove catagory mapping file!");
                result = false;
            }
        }
        return(result);
    }
    bool catagories[256];
    unsigned int i;
    QString **catarray = kifapp()->catagoryManager()->array();

    for(i=0; i < 256; ++i)
        catagories[i] = false;

    // write catagory database
    QString fileStr(pathStr + "/.catagories");
    QFile f(fileStr);
    if(!f.open(IO_WriteOnly)){
        // TODO: error dialog
        qWarning("Unable to write catagories for folder!");
        return(false);
    }
    QDataStream stream;
    stream.setDevice(&f);

    struct catagoryinfo *info = head;
    while(info){
        stream << info->inode;
        stream << info->numcatagories;
        for(i=0; i < info->numcatagories; ++i){
            stream << info->catagories[i];
            catagories[info->catagories[i]] = true; // hehe, works w/ gcc >:)
        }
        info = info->next;
    }
    f.close();

    // write catagory mapping
    fileStr = pathStr + "/.catagory-mapping";
    f.setName(fileStr);
    if(!f.open(IO_WriteOnly)){
        // TODO: error dialog
        qWarning("Unable to write catagory mapping for folder!");
        return(false);
    }
    stream.setDevice(&f);
    unsigned char tmp;
    for(i=0; i < 256; ++i){
        if(catagories[i]){
            stream << *catarray[i];
            tmp = i;
            stream << tmp;
        }
    }
    f.close();
    modified = false;
    return(true);
}

bool Catagory::setPath(const QString &path)
{
    if(QFile::exists(path)) // FIXME
        return(readDb());
    else
        return(false);
}

catagoryinfo* Catagory::info(struct stat *file)
{
    if(!count)
        return(NULL);
    struct catagoryinfo *info = head;
    while(info){
        if(info->inode == (unsigned int)file->st_ino)
            return(info);
        info = info->next;
    }
    return(NULL);
}

bool Catagory::addCatagory(struct stat *file, unsigned int id)
{
    // see if it exists
    struct catagoryinfo *info = head;
    while(info){
        if(info->inode == (unsigned int)file->st_ino){
            if(info->numcatagories > 7){
                // TODO Message box
                qWarning("Too many catagories!");
                return(false);
            }
            else{
                info->catagories[info->numcatagories] = id;
                ++info->numcatagories;
                modified = true;
                return(true);
            }
        }
    }
    info = (struct catagoryinfo *)malloc(sizeof(struct catagoryinfo));
    info->next = head;
    info->prev = NULL;
    if(head)
        head->prev = info;
    head = info;
    ++count;
    modified = true;
    return(true);
}

bool Catagory::removeCatagory(struct stat *file, unsigned int id)
{
    struct catagoryinfo *info = head;
    unsigned int i, j;
    while(info){
        if(info->inode == (unsigned int)file->st_ino){
            if(info->numcatagories-1 <= 0){
                qWarning("All catagories or item removed, deleteing item");
                if(info->prev)
                    info->prev->next = info->next;
                if(info->next)
                    info->next->prev = info->prev;
                free(info);
                --count;
                modified = true;
                return(true);
            }
            else{
                qWarning("Removing catagory for item");
                // justify
                unsigned char tmp[8];
                for(i=0, j=0; i < info->numcatagories; ++i){
                    if(info->catagories[i] != id){
                        tmp[j] = info->catagories[i];
                        ++j;
                    }
                }
                for(i=0; i < info->numcatagories-1; ++i)
                    info->catagories[i] = tmp[i];
                --info->numcatagories;
                modified = true;
                return(true);
            }
        }
    }
    return(false);
}

void Catagory::sync()
{
    if(!modified){
        qWarning("Catagory database not modified");
        return;
    }
    else
        writeDb();
}





