#include "catagorydlg.h"
#include "../stub/ifapp.h"
#include "../stub/catagorymgr.h"
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qinputdialog.h>
#include <qlayout.h>
#include <kbuttonbox.h>
#include <klocale.h>
#include <kseparator.h>
#include <kmessagebox.h>


CatagoryDialog::CatagoryDialog(QWidget *parent, const char *name)
    : QDialog(parent, name, true)
{
    QGridLayout *layout = new QGridLayout(this, 1, 1, 5);
    listBox = new QListBox(this);
    listBox->setSelectionMode(QListBox::Extended);
    listBox->setMinimumSize(300, 100);
    connect(listBox, SIGNAL(selectionChanged()), this,
            SLOT(slotSelectionChanged()));
    layout->addMultiCellWidget(listBox, 1, 3, 0, 0);
    QPushButton *newBtn = new QPushButton(i18n("New..."), this);
    connect(newBtn, SIGNAL(clicked()), this, SLOT(slotAdd()));
    layout->addWidget(newBtn, 1, 1);
    removeBtn = new QPushButton(i18n("Remove"), this);
    removeBtn->setEnabled(false);
    connect(removeBtn, SIGNAL(clicked()), this, SLOT(slotRemove()));
    layout->addWidget(removeBtn, 2, 1);
    KSeparator *sep = new KSeparator(Qt::Horizontal, this);
    layout->addMultiCellWidget(sep, 4, 4, 0, 1);

    KButtonBox *bbox = new KButtonBox(this);
    bbox->addStretch(1);
    connect(bbox->addButton(i18n("OK")), SIGNAL(clicked()), this,
            SLOT(accept()));
    connect(bbox->addButton(i18n("Cancel")), SIGNAL(clicked()), this,
            SLOT(reject()));
    layout->addMultiCellWidget(bbox, 5, 5, 0, 1);

    layout->setRowStretch(3, 1);
    layout->setColStretch(0, 1);

    modified = false;
    // load initial list
    QString **array = kifapp()->catagoryManager()->array();
    QStringList strList;
    int i;
    for(i=1; i < 256; ++i){
        if(array[i] != NULL)
            strList.append(*array[i]);
    }
    if(!strList.isEmpty()){
        strList.sort();
        listBox->insertStringList(strList);
    }

    setCaption("Pixie File Catagories");
}

void CatagoryDialog::slotAdd()
{
    bool ok;
    QString str = QInputDialog::getText(i18n("New Catagory"),
                                        i18n("Please enter a name for the new catagory:"),
                                        QLineEdit::Normal,
                                        QString::null, &ok, this);
    if(ok){
        if(listBox->findItem(str, Qt::CaseSensitive | Qt::BeginsWith) != 0){
            KMessageBox::sorry(this, i18n("Catagory already exists!"));
            return;
        }
        else{
            listBox->insertItem(str);
            listBox->sort();
            int id = listBox->index(listBox->findItem(str, Qt::CaseSensitive |
                                                      Qt::BeginsWith));
            if(!listBox->itemVisible(id))
                listBox->setTopItem(id);
            modified = true;
        }
    }
}

void CatagoryDialog::slotRemove()
{
    // we need this funky stringlist because you can't just loop and remove
    // multiselected items, since removing items changes the selection
    QStringList strList;
    int i;
    for(i=0; i < listBox->count(); ++i){
        if(!listBox->item(i)->isSelected())
            strList.append(listBox->text(i));
    }
    listBox->clear();
    listBox->insertStringList(strList);
    modified = true;
}

void CatagoryDialog::slotSelectionChanged()
{
    int i;
    for(i=0; i < listBox->count(); ++i){
        if(listBox->item(i)->isSelected()){
            removeBtn->setEnabled(true);
            return;
        }
    }
    removeBtn->setEnabled(false);
}

void CatagoryDialog::sync()
{
    if(!modified)
        return;
    QStringList strList;
    int i;
    for(i=0; i < listBox->count(); ++i){
        strList.append(listBox->text(i));
    }
    kifapp()->catagoryManager()->insertStringList(strList);
    kifapp()->catagoryManager()->sync();
}



