#ifndef __PIXIE_DIRTREE_H
#define __PIXIE_DIRTREE_H

#include <qlistview.h>
#include <qpixmap.h>
#include <qdragobject.h>
#include <kdirwatch.h>

class PixieDirItem;

class PixieBaseItem : public QListViewItem
{
public:
    PixieBaseItem(QListView *parent, const QString &file);
    PixieBaseItem(PixieDirItem *parent, const QString &file);
    QString fullPath();
    QString text(int col) const;
    bool isDir();
protected:
    PixieDirItem *parentDir;
    QString fileStr;
    bool dirFlag;
};

class PixieDirItem : public PixieBaseItem
{
public:
    PixieDirItem(QListView *parent, const QString &file);
    PixieDirItem(PixieDirItem *parent, const QString &file);
    void setOpen(bool open);
    bool canOpen(){return(hasAccess);}
protected:
    bool hasAccess;
};

class PixieFileItem : public PixieBaseItem
{
public:
    PixieFileItem(PixieDirItem *parent, const QString &file);
};


class PixieDirTree : public QListView
{
    friend class PixieDirItem;
    Q_OBJECT
public:
    PixieDirTree(bool dirOnly, QWidget *parent=0, const char *name=0);
    ~PixieDirTree();
    void setPath(const QString &path);
    bool isDirOnly(){return(noFiles);}
protected slots:
    void slotClicked(QListViewItem *i);
    void slotDirChanged(const QString &path);
    void slotDirOpened(QListViewItem *);
    void slotDirClosed(QListViewItem *);
    void slotRightClick(QListViewItem *i, const QPoint &pos, int);
protected:
    void dragEnterEvent(QDragEnterEvent *ev);
    void dragLeaveEvent(QDragLeaveEvent *ev);
    void dragMoveEvent(QDragMoveEvent *ev);
    void dropEvent(QDropEvent *ev);
    QListViewItem *lastOpen;
    QListViewItem *dndItem;
    QListViewItem *prevSelectedItem;
    QPixmap fPix, fOpenPix, fNoPermPix;
    KDirWatch *dirWatch;
    bool noFiles;
    QPixmap *folderPix;

};

#endif

