#include "imagepreview.h"
#include "../stub/ifapp.h"
#include <qpainter.h>

KIFImagePreview::KIFImagePreview(QWidget *parent, const char *name)
    : QWidget(parent, name)
{
    setBackgroundMode(NoBackground);
}

void KIFImagePreview::slotSetFile(const QString &file)
{
    qWarning("In KIFImagePreview::slotSetFile");
    scaleImage.reset();
    if(!kifLoadImage(image, file)){
        fileStr = "";
    }
    else{
        //adjustAlpha(image);
        fileStr = file;
        resizeWithAspect();
        QApplication::syncX();
        repaint();
    }
}
void KIFImagePreview::resizeWithAspect()
{
    if(image.isNull())
        return;
    if(pix.width() != width() || pix.height() != height()){
        //QRect deskRect = QApplication::desktop()->rect();
        QSize wSize = size();
        scaleImage = image;
        QSize r = QSize(scaleImage.width(), scaleImage.height());
        // may need to do this twice
        if(r.width() > wSize.width() || r.height() > wSize.height()){
            while(r.width() > wSize.width() || r.height() > wSize.height()){
                if(r.width() > wSize.width()){
                    float percent = ((float)wSize.width())/r.width();
                    r.setWidth((int)(r.width()*percent));
                    r.setHeight((int)(r.height()*percent));
                    qWarning("Scaling width");
                }
                if(r.height() > wSize.height()){
                    float percent = ((float)wSize.height())/r.height();
                    r.setWidth((int)(r.width()*percent));
                    r.setHeight((int)(r.height()*percent));
                    qWarning("Scaling height");
                }
            }
        }
        scaleImage = scaleImage.smoothScale(r.width(), r.height());
        pix.convertFromImage(scaleImage);
    }
}

void KIFImagePreview::resizeEvent(QResizeEvent* )
{
    resizeWithAspect();
}

void KIFImagePreview::paintEvent(QPaintEvent *ev)
{
    QPainter p;
    p.begin(this);
    if(pix.isNull() || pix.mask())
        p.fillRect(ev->rect(), Qt::white);
    if(pix.isNull()){
        return;
    }

    QRect r = ev->rect();
    p.drawPixmap(r.topLeft(), pix, r);
    QRect pixRect(0, 0, pix.width(), pix.height());
    pixRect = pixRect.intersect(r);
    if(pixRect.right() < r.right()){
        p.fillRect(pixRect.right()+1, r.y(), r.width()-pixRect.width(),
                   r.height(), Qt::white);
    }
    if(pixRect.bottom() < r.bottom()){
        p.fillRect(r.x(), pixRect.bottom()+1, r.width(),
                   r.height()-pixRect.height(), Qt::white);
    }
    p.end();
}

void KIFImagePreview::closeEvent(QCloseEvent *ev)
{
    clear();
    ev->accept();
}




