#include "printdialog.h"
 
#include <qlayout.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qprinter.h>
#include <qpaintdevicemetrics.h>
#include <qlabel.h>
#include <qvalidator.h>
#include <qlineedit.h>
#include <qpushbutton.h>
//#include <knuminput.h>
#include <kbuttonbox.h>
#include <klocale.h>
#include <kconfig.h>
#include <kglobal.h>

// conversions from koGlobal.h
#ifndef CONVERT_FUNCS_DEFINED
#define CONVERT_FUNCS_DEFINED
inline double convertMMToInch(double mm){return(mm*0.039370147);}
inline double convertInchToMM(double inch){return(inch*25.399956);}
#endif
inline double convertMMToInch(const QString& mm){return(mm.toDouble()*0.039370147);}
inline double convertInchToMM(const QString& inch){return(inch.toDouble()*25.399956);}


#define US_MEASURE 0
#define METRIC_MEASURE 1
 
KIFPrintDialog::KIFPrintDialog(KPrinter &p, QWidget *parent, const char *name)
    : QDialog(parent, name, true)
{
    doSave = false;
    p.setFullPage(true);
    QPaintDeviceMetrics metrics(&p);
    KConfig *config = KGlobal::config();
    config->setGroup("Print");

    setCaption(i18n("Page Layout"));
    QVBoxLayout *layout = new QVBoxLayout(this, 4);
    QGroupBox *mBox = new QGroupBox(i18n("Margins"), this);
    QGridLayout *mLayout = new QGridLayout(mBox, 1, 1, 4);
    mLayout->addRowSpacing(0, 15);
    metricBox = new QComboBox(mBox);
    metricBox->insertItem(i18n("Inches"), US_MEASURE);
    metricBox->insertItem(i18n("Millimeters"), METRIC_MEASURE);
    metricBox->setCurrentItem(config->readNumEntry("MetricType", US_MEASURE));
    connect(metricBox, SIGNAL(activated(int)), this,
            SLOT(slotMetricsClicked(int)));
    mLayout->addMultiCellWidget(metricBox, 1, 1, 0, 2);

    double defHMargin, defVMargin;
    QSize margin = p.margins();
    defHMargin = ((double)margin.width())/metrics.logicalDpiX();
    defVMargin = ((double)margin.height())/metrics.logicalDpiY();
    if(metricBox->currentItem() == METRIC_MEASURE){
        defHMargin = convertInchToMM(defHMargin);
        defVMargin = convertInchToMM(defVMargin);
    }

    validator = new QDoubleValidator(this);
    validator->setDecimals(2);
    QLabel *lbl = new QLabel(i18n("Left:"), mBox);
    mLayout->addWidget(lbl, 2, 0);
    lEdit = new
        QLineEdit(QString::number(config->readDoubleNumEntry("LMargin", defHMargin)),
                  mBox);
    lEdit->setValidator(validator);
    mLayout->addWidget(lEdit, 2, 1);
    lbl = new QLabel(i18n("Right:"), mBox);
    mLayout->addWidget(lbl, 2, 3);
    rEdit = new
        QLineEdit(QString::number(config->readDoubleNumEntry("RMargin", defHMargin)),
                  mBox);
    rEdit->setValidator(validator);
    mLayout->addWidget(rEdit, 2, 4);

    lbl = new QLabel(i18n("Top:"), mBox);
    mLayout->addWidget(lbl, 3, 0);
    tEdit = new
        QLineEdit(QString::number(config->readDoubleNumEntry("TMargin", defVMargin)),
                  mBox);
    tEdit->setValidator(validator);
    mLayout->addWidget(tEdit, 3, 1);
    lbl = new QLabel(i18n("Bottom:"), mBox);
    mLayout->addWidget(lbl, 3, 3);
    bEdit = new
        QLineEdit(QString::number(config->readDoubleNumEntry("BMargin", defVMargin)),
                  mBox);
    bEdit->setValidator(validator);
    mLayout->addWidget(bEdit, 3, 4);

    mLayout->addColSpacing(2, 15);
    mLayout->setColStretch(5, 100);
    qWarning("HMargin: %f, VMargin: %f", defHMargin, defVMargin);
    layout->addWidget(mBox);

    scaleLarge = new QCheckBox(i18n("Scale large images to page size."), this);
    scaleLarge->setChecked(config->readBoolEntry("Scale", true));
    connect(scaleLarge, SIGNAL(clicked()), this, SLOT(slotOptionClicked()));
    layout->addWidget(scaleLarge);
    zoomSmall = new QCheckBox(i18n("Zoom small images to page size."), this);
    zoomSmall->setChecked(config->readBoolEntry("Zoom", false));
    connect(scaleLarge, SIGNAL(clicked()), this, SLOT(slotOptionClicked()));
    layout->addWidget(zoomSmall);
    manualDither = new QCheckBox(i18n("Manually dither highcolor pictures."),
                                 this);
    manualDither->setChecked(config->readBoolEntry("Dither", false));
    connect(scaleLarge, SIGNAL(clicked()), this, SLOT(slotOptionClicked()));
    layout->addWidget(manualDither);


    KButtonBox *bbox = new KButtonBox(this);
    bbox->addStretch(1);
   
    connect(bbox->addButton(i18n("OK")), SIGNAL(clicked()), this,
            SLOT(accept()));
    connect(bbox->addButton(i18n("Cancel")), SIGNAL(clicked()), this,
            SLOT(reject()));
    layout->addWidget(bbox);         
}

void KIFPrintDialog::slotOptionClicked()
{
    doSave = true;
    /*
    KConfig *config = KGlobal::config();
    config->setGroup("Print");
    config->writeEntry("Scale", scaleLarge->isChecked());
    config->writeEntry("Zoom", zoomSmall->isChecked());
    config->writeEntry("Dither", manualDither->isChecked());
    config->sync();*/
}

void KIFPrintDialog::slotMetricsClicked(int id)
{
    /*
    KConfig *config = KGlobal::config();
    config->setGroup("Print");
    config->writeEntry("MetricType", id);
    config->sync();
    */
    doSave = true;
    if(id == METRIC_MEASURE){
        lEdit->setText(QString::number(convertInchToMM(lEdit->text())));
        rEdit->setText(QString::number(convertInchToMM(rEdit->text())));
        tEdit->setText(QString::number(convertInchToMM(tEdit->text())));
        bEdit->setText(QString::number(convertInchToMM(bEdit->text())));

    }
    else{
        lEdit->setText(QString::number(convertMMToInch(lEdit->text())));
        rEdit->setText(QString::number(convertMMToInch(rEdit->text())));
        tEdit->setText(QString::number(convertMMToInch(tEdit->text())));
        bEdit->setText(QString::number(convertMMToInch(bEdit->text())));
    }

}

void KIFPrintDialog::accept()
{
    qWarning("In slotAccept");
    KConfig *config = KGlobal::config();
    config->setGroup("Print");
    config->writeEntry("Scale", scaleLarge->isChecked());
    config->writeEntry("Zoom", zoomSmall->isChecked());
    config->writeEntry("Dither", manualDither->isChecked());
    config->writeEntry("MetricType", metricBox->currentItem());

    config->writeEntry("LMargin", lEdit->text().toDouble());
    config->writeEntry("RMargin", rEdit->text().toDouble());
    config->writeEntry("TMargin", tEdit->text().toDouble());
    config->writeEntry("BMargin", bEdit->text().toDouble());
    config->sync();
    QDialog::accept();
}

#include "printdialog.moc"

