#include "quality.h"
#include "../stub/ifapp.h"
#include <qlabel.h>
#include <qimage.h>
#include <qvbuttongroup.h>
#include <qvbox.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <kbuttonbox.h>
#include <klocale.h>
#include <kconfig.h>
#include <kglobal.h>

bool saveImage(QImage *img, const QString &filename, const char *format)
{
    if(qstricmp(format, "PNG") == 0 || qstricmp(format, "JPEG") == 0){
        int quality;
        KIFQualityDialog dlg(false, format);
        if(dlg.exec() == QDialog::Rejected){
            return(false);
        }
        quality = dlg.quality();
        if(dlg.psudeoClass() && img->depth() > 8){
            QImage newImg = img->convertDepth(8, QPixmap::ColorOnly |
                                              QPixmap::DiffuseDither |
                                              QPixmap::ThresholdAlphaDither |
                                              QPixmap::AvoidDither);
            return(kifSaveImage(newImg, filename, format, quality));
        }
        else{
            return(kifSaveImage(*img, filename, format, quality));
        }
    }
    else{
        return(kifSaveImage(*img, filename, format));
    }
}



KIFQualityDialog::KIFQualityDialog(bool isBatch, const char *format,
                                   QWidget *parent, const char *name)
    : QDialog(parent, name, true)
{
    QVBoxLayout *layout = new QVBoxLayout(this);
    QLabel *lbl;
    if(isBatch)
        lbl = new QLabel(i18n("Please select the quality level for the images saved\n\
by the batch effect. This controls the compression\n\
level. Higher quality means less compression, but\n\
smoother edges and better color fills."), this);
    else
        lbl = new QLabel(i18n("Please select the quality level for the saved image.\n\
This controls the compression level. Higher quality\n\
means less compression, but smoother edges and better\n\
color fills."), this);
    layout->addWidget(lbl);
    layout->addSpacing(16);

    QVBox *qBox = new QVBox(this);
    qBox->setFrameStyle(QFrame::Box | QFrame::Sunken);
    qBox->setSpacing(15);
    if(isBatch || qstricmp(format, "PNG") == 0){
        qEdit = new KIntNumInput(6, qBox);
        qEdit->setRange(1, 10);
    }
    else{
        qEdit = new KIntNumInput(75, qBox);
        qEdit->setRange(50, 100);
    }
    if(isBatch)
        qEdit->setValue(7);
    layout->addWidget(qBox);
    layout->addSpacing(16);

    QVButtonGroup *colorGrp = new QVButtonGroup(i18n("Color palette"), this);
    directBtn = new QRadioButton(i18n("High quality (as many colors as needed)"), colorGrp);
    psudeoBtn = new QRadioButton(i18n("Low quality (256 color dithered)"), colorGrp);
    directBtn->setChecked(true);
    layout->addWidget(colorGrp);

    layout->addStretch();

    KButtonBox *bbox = new KButtonBox(this);
    bbox->addStretch(1);
    connect(bbox->addButton(i18n("Cancel")), SIGNAL(clicked()), this,
            SLOT(reject()));
    connect(bbox->addButton(i18n("OK")), SIGNAL(clicked()), this,
            SLOT(accept()));
    layout->addWidget(bbox);
    if(isBatch)
        setCaption(i18n("Image Quality Selection"));
    else
        setCaption(i18n("Image Quality for ")+ QString(format));
}




