#include "rightclick.h"
#include "thumbiconview.h"
#include "uimanager.h"
#include "fileop.h"
#include "batch.h"
#include "aniplayer.h"
#include "quality.h"

#include "../stub/ifapp.h"
#include "../stub/catagorymgr.h"

#include <qpopupmenu.h>
#include <qclipboard.h>
#include <qinputdialog.h>
#include <qpopupmenu.h>
#include <kmessagebox.h>
#include <kpopupmenu.h>
#include <kpropsdlg.h>
#include <kiconloader.h>
#include <klocale.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <utime.h>

#include <X11/X.h>
#include <X11/Xlib.h>


void iconRightClickDelete(const QString &currentDir, KIFThumbView *view)
{
    QStringList fileList;
    CustomIconViewItem *i = view->firstItem();
    while (i){
        if(i->isSelected())
            fileList.append(i->text());
        i = i->nextItem();
    }
    if(fileList.isEmpty())
        return;
    if(KMessageBox::warningContinueCancelList(view,
                                              i18n("Do you really want to delete these files?"),
                                              fileList,
                                              i18n("Delete Confirmation"),
                                              i18n("Delete")) ==
       KMessageBox::Cancel)
        return;
    QString fileStr;
    QFileInfo fi;
    for(QStringList::Iterator it = fileList.begin(); it != fileList.end(); ++it) {
        fileStr = currentDir + "/" + (*it);
        qWarning("Deleting %s", fileStr.latin1());
        fi.setFile(fileStr);
        if(fi.isDir()){
            qWarning("In folder removal");
            // recursive folder delete
            QStringList dirList;
            if(!KIFFileTransfer::recursiveParseDir(fi.absFilePath(),
                                                   dirList, true, false,
                                                   true, false)){
                // TODO Error dialog
                KMessageBox::sorry(view, i18n("Folder removal error."));
                return;
            }
            if(KMessageBox::warningContinueCancelList(view,
                                                      i18n("Do you really want to all these files?"),
                                                      dirList,
                                                      i18n("Folder Delete Confirmation"),
                                                      i18n("Delete")) ==
               KMessageBox::Cancel)
                return;
            for(QStringList::Iterator dit = dirList.begin(); dit != dirList.end(); ++dit){
                qWarning("removing %s", (*dit).latin1());
                fi.setFile(*dit);
                if(fi.isDir()){
                    if(rmdir((*dit).ascii()) == -1){
                        qWarning("rmdir error");
                        KMessageBox::sorry(view, i18n("Could not remove folder ") + (*dit).latin1());
                        return;
                    }
                }
                else{
                    if(unlink((*dit).ascii()) == -1){
                        qWarning("unlink error");
                        KMessageBox::sorry(view, i18n("Could not remove file ") + (*dit).latin1());
                        return;
                    }
                }
            }
        }
        else{
            if(unlink(fileStr.ascii()) == -1){
                KMessageBox::sorry(view, i18n("Could not remove file ") + fileStr.latin1());
                return;
            }
            if(QFile::exists(currentDir + "/.pics/huge/" + (*it)))
                unlink(QString(currentDir + "/.pics/huge/" + (*it)).ascii());
            if(QFile::exists(currentDir + "/.pics/large/" + (*it)))
                unlink(QString(currentDir + "/.pics/large/" + (*it)).ascii());
            if(QFile::exists(currentDir + "/.pics/med/" + (*it)))
                unlink(QString(currentDir + "/.pics/med/" + (*it)).ascii());
            if(QFile::exists(currentDir + "/.pics/small/" + (*it)))
                unlink(QString(currentDir + "/.pics/small/" + (*it)).ascii());
        }
    }
}

void iconRightClickMenu(const QString &currentDir, UIManager *mgr,
                        KIFThumbView *view, KIFThumbViewItem *item,
                        const QPoint &pos)
{
    bool supportCatagories = false;
    KURL url("file:"+item->path());
    bool isDir = S_ISDIR(((KIFThumbViewItem *)item)->statusInfo()->st_mode);
    bool isImage =  !isDir && KMimeType::findByURL(url, true, true)->name().left(6)
        == "image/";
    QStringList nameList, pathList;

    QPopupMenu *batchMnu = NULL;
    if(isImage){
        // various batch effects
        batchMnu = new QPopupMenu;

        QPopupMenu *transformMnu = new QPopupMenu;
        transformMnu->insertItem(i18n("Scale to Max Size..."), 100);
        transformMnu->insertItem(i18n("Flip Horizontal"), 101);
        transformMnu->insertItem(i18n("Flip Vertical"), 102);
        transformMnu->insertItem(i18n("Rotate 90 Degrees"), 103);
        transformMnu->insertItem(i18n("Rotate 180 Degrees"), 104);
        transformMnu->insertItem(i18n("Rotate 270 Degrees "), 105);
        batchMnu->insertItem(i18n("Transform"), transformMnu);

        QPopupMenu *enhanceMnu = new QPopupMenu;
        enhanceMnu->insertItem(i18n("Spiff"), 125);
        enhanceMnu->insertItem(i18n("Dull"), 126);
        enhanceMnu->insertItem(i18n("Equalize"), 127);
        enhanceMnu->insertItem(i18n("Normalize"), 128);
        enhanceMnu->insertItem(i18n("Invert"), 129);
        enhanceMnu->insertItem(i18n("Grayscale"), 130);
        batchMnu->insertItem(i18n("Enhance"), enhanceMnu);

        QPopupMenu *effectsMnu = new QPopupMenu;
        effectsMnu->insertItem(i18n("Add Text..."), 150);
        effectsMnu->insertItem(i18n("Add Border..."), 151);
        effectsMnu->insertSeparator();
        effectsMnu->insertItem(i18n("Add Noise..."), 152);
        effectsMnu->insertItem(i18n("Despeckle..."), 153);
        effectsMnu->insertItem(i18n("Emboss..."), 154);
        effectsMnu->insertItem(i18n("Sharpen..."), 155);
        effectsMnu->insertItem(i18n("Blur..."), 156);
        effectsMnu->insertItem(i18n("Threshold..."), 157);
        effectsMnu->insertItem(i18n("Edge Detect..."), 158);
        effectsMnu->insertItem(i18n("Spread..."), 159);
        effectsMnu->insertItem(i18n("Shade..."), 160);
        batchMnu->insertItem(i18n("Normal Effects"), effectsMnu);

        QPopupMenu *specialMnu = new QPopupMenu;
        specialMnu->insertItem(i18n("Solarize..."), 175);
        specialMnu->insertItem(i18n("Swirl..."), 176);
        specialMnu->insertItem(i18n("Implode..."), 177);
        specialMnu->insertItem(i18n("Wave..."), 178);
        specialMnu->insertItem(i18n("Oil Paint..."), 179);
        specialMnu->insertItem(i18n("Charcol Draw..."), 180);
        batchMnu->insertItem(i18n("Special Effects"), specialMnu);
    }

    QPopupMenu *convMnu = NULL;
    if(isImage){
        convMnu = new QPopupMenu;
        QStrList iotypes = kifImageOutputFormats();
        const char *str;
        int i;
        for(str = iotypes.first(), i=300; str != NULL; str = iotypes.next(), ++i){
            convMnu->insertItem(str, i);
        }
    }

    KPopupMenu *mnu = new KPopupMenu;
    mnu->insertTitle(BarIcon("filenew", KIcon::SizeSmall),i18n("File Options"));
    if(isImage){
        //mnu->insertItem(BarIcon("pencil", KIcon::SizeSmall), i18n("Edit Image..."), 2);
        mnu->insertItem(i18n("Convert to..."), convMnu, 3);

        mnu->insertSeparator();
        mnu->insertItem(BarIcon("editcopy", KIcon::SizeSmall), i18n("Copy files"), 5);
        mnu->insertItem(BarIcon("editpaste", KIcon::SizeSmall), i18n("Paste"), 6);
        mnu->insertSeparator();
        mnu->insertItem(BarIcon("filenew", KIcon::SizeSmall),
                        i18n("Copy selection to FileList"), 8);
        mnu->insertItem(BarIcon("filenew", KIcon::SizeSmall),
                        i18n("Copy all images to FileList"), 9);

        mnu->insertSeparator();
    }
    mnu->insertItem(BarIcon("editcopy", KIcon::SizeSmall), i18n("Copy file location"), 11);
    mnu->insertItem(BarIcon("editcopy", KIcon::SizeSmall), i18n("Copy filename only"), 12);
    mnu->insertSeparator();
    mnu->insertItem(i18n("Rename"), 14);
    mnu->insertItem(BarIcon("edittrash", KIcon::SizeSmall), i18n("Delete"), 15);
    mnu->insertItem(BarIcon("wizard", KIcon::SizeSmall), i18n("Properties"), 16);
    mnu->insertItem(i18n("Play Animation (Fullscreen only)"), 17);
    if(isImage){
        mnu->insertSeparator();
        mnu->insertItem(i18n("Batch effects"), batchMnu, 19);
    }
    mnu->insertSeparator();
    if(supportCatagories){
        /*
        qWarning("Creating catagory menus");
        QPopupMenu *catMnu = new QPopupMenu;
        QPopupMenu *addMnu = new QPopupMenu;
        QPopupMenu *delMnu = new QPopupMenu;
        int c = 0;
        bool multipleSelected = false;
        QStringList catList(*kifapp()->catagoryManager()->catagoryList());
        QStringList itemList;

        CustomIconViewItem *i = view->firstItem();
        while(i && !multipleSelected){ // check for multiple selection
            if(i->isSelected()){
                ++c;
                if(c > 1)
                    multipleSelected = true;
            }
            i = i->nextItem();
        }
        if(!multipleSelected)
            itemList = itemCatagories((KIFThumbViewItem*)item);
        QStringList::Iterator it;
        for(it=catList.begin(), c=1000; it != catList.end(); ++it, ++c){
            if(multipleSelected){
                addMnu->insertItem((*it), c);
                delMnu->insertItem((*it), c+1000);
            }
            else{
                if(itemList.findIndex((*it)) == -1)
                    addMnu->insertItem((*it), c);
                else
                    delMnu->insertItem((*it), c+1000);
            }
        }
        catMnu->insertItem(i18n("Add to catagory"), addMnu);
        catMnu->insertItem(i18n("Remove from catagory"), delMnu);
        mnu->insertItem(i18n("Catagories..."), catMnu, 21);
        */
    }
    else{
        mnu->insertItem(i18n("Catagories..."), 21); // disable for now
        mnu->setItemEnabled(21, false);
    }
    mnu->insertItem(i18n("Make newest file(s)"), 22);
    mnu->insertItem(i18n("Make oldest file(s)"), 23);
    if(!isDir){
        mnu->insertTitle(BarIcon("mini-ray.png", KIcon::SizeSmall),
                         i18n("Copy to HotList (Shift to Move)"));

        // add hotlist items
        KConfig *config = KGlobal::config();
        config->setGroup("Thumbnails");
        nameList = config->readListEntry("Names");
        pathList = config->readListEntry("Paths");

        nameList.prepend(i18n("Home"));
        pathList.prepend(QDir::homeDirPath());
        int i = 25;
        for(QStringList::Iterator it = nameList.begin();
            it != nameList.end(); ++it, ++i){
            mnu->insertItem(BarIcon("folder", KIcon::SizeSmall),
                            (*it), i);
        }
    }

    // FIXME
    // disable unfinished or unsupported items
    // mnu->setItemEnabled(17, fi->extension(false).lower() == "gif");

    // run menu
    int id = mnu->exec(pos);
    qWarning("id: %d", id);
    if(batchMnu)
        delete batchMnu;
    if(convMnu)
        delete convMnu;
    delete mnu;
    if(id == -1)
        return;
    else if(id == 2){
        //emit editImage();
    }
    else if(id == 5){
        // Copy files
        view->slotCopy();
    }
    else if (id == 6){
        // Paste
        view->slotPaste();
    }
    else if(id == 8){
        // Copy selection to filelist
        CustomIconViewItem *i = view->firstItem();
        QString fileStr;
        bool isImage;
        while(i){
            if(i->isSelected()){
                fileStr = currentDir + "/" + i->text();
                KURL url("file:" + fileStr);
                isImage =  KMimeType::findByURL(url, true, true)->name().left(6) ==
                    "image/";
                if(isImage)
                    mgr->slotAddURL(fileStr);

            }
            i = i->nextItem();
        }
    }
    else if(id == 9){
        // Copy all images to the filelist
        CustomIconViewItem *i = view->firstItem();
        QString fileStr;
        bool isImage;
        while(i){
            fileStr = currentDir + "/" + i->text();
            KURL url("file:" + fileStr);
            isImage =  KMimeType::findByURL(url, true, true)->name().left(6) ==
                "image/";
            if(isImage)
                mgr->slotAddURL(fileStr);
            i = i->nextItem();
        }
    }
    else if(id == 11){
        // Copy Filename Location
        QFileInfo fi(item->path());
        QApplication::clipboard()->setText(fi.absFilePath());
    }
    else if(id == 12){
        // Copy Filename Only
        QFileInfo fi(item->path());
        QApplication::clipboard()->setText(fi.fileName());
    }
    else if(id == 14){
        // Rename
        bool result = false;
        QString str = QInputDialog::getText(i18n("Rename file"),
                                            i18n("Enter new filename:"),
                                            QLineEdit::Normal, item->text(),
                                            &result, view);
        if(!result || str.isEmpty())
            return;
        qWarning("Rename for %s to %s", item->path().latin1(),
                 str.latin1());
        KIFFileTransfer::transferFile(item->path(),
                                      currentDir+"/"+str, QDropEvent::Move);
    }
    else if(id == 15){
        // Delete
        iconRightClickDelete(currentDir, view);
    }
    else if(id == 16){
        // Properties
        KFileItem fileitem(-1, -1, KURL(QString("file:")+currentDir+"/"+
                                    item->text()));
        (void) new KPropertiesDialog(&fileitem, view);
    }
    else if(id == 17){
        // Play animation
        (void) new KIFAniPlayer(currentDir+"/"+item->text());
    }
    else if(id == 22){
        // Make newest file
        CustomIconViewItem *i = view->firstItem();
        QString fileStr, thumbStr;

        struct utimbuf timeBuf;
        struct stat status;
        time_t modtime;
        QStringList newestList; // preserve order for multiple selections
        while(i){
            if(i->isSelected())
                newestList.append(i->text());
            i = i->nextItem();
        }

        modtime = view->newest()+newestList.count()+1;
        for(QStringList::Iterator it = newestList.begin(); it != newestList.end(); ++it) {
            fileStr = currentDir + "/" + (*it);
            if(stat(fileStr.ascii(), &status) == -1){
                if(KMessageBox::warningContinueCancel(view,
                                                      i18n("Could not access ")+(*it),
                                                      i18n("File Error!"))
                   == KMessageBox::Cancel){
                    mgr->slotUpdateView();
                    return;
                }
            }
            else{
                timeBuf.actime = status.st_atime;
                timeBuf.modtime = modtime;
                if(utime(fileStr.ascii(), &timeBuf) == -1){
                    if(KMessageBox::warningContinueCancel(view,
                                                          i18n("Could not update timestamp for ")+(*it),
                                                          i18n("File Error!"))
                       == KMessageBox::Cancel){
                        mgr->slotUpdateView();
                        return;
                    }
                }
                else{
                    // handle thumbnails, too, so they don't get marked as old
                    // don't worry about errors if they don't exist
                    thumbStr = currentDir + "/.pics/huge/" + (*it);
                    if(stat(thumbStr.ascii(), &status) == 0)
                        utime(thumbStr.ascii(), &timeBuf);
                    thumbStr = currentDir + "/.pics/large/" + (*it);
                    if(stat(thumbStr.ascii(), &status) == 0)
                        utime(thumbStr.ascii(), &timeBuf);
                    thumbStr = currentDir + "/.pics/med/" + (*it);
                    if(stat(thumbStr.ascii(), &status) == 0)
                        utime(thumbStr.ascii(), &timeBuf);
                    thumbStr = currentDir + "/.pics/small/" + (*it);
                    if(stat(thumbStr.ascii(), &status) == 0)
                        utime(thumbStr.ascii(), &timeBuf);
                }
            }
            --modtime;
        }
        mgr->slotUpdateView();
    }
    else if(id == 23){
        // Make oldest file
        CustomIconViewItem *i = view->firstItem();
        QString fileStr;
        struct utimbuf timeBuf;
        struct stat status;
        time_t modtime = view->oldest()-1;
        while(i){
            if(i->isSelected()){
                fileStr = currentDir + "/" + i->text();
                if(stat(fileStr.ascii(), &status) == -1){
                    if(KMessageBox::warningContinueCancel(view,
                                                          i18n("Could not access ")+i->text(),
                                                          i18n("File Error!"))
                       == KMessageBox::Cancel){
                        mgr->slotUpdateView();
                    return;
                    }
                }
                else{
                    timeBuf.actime = status.st_atime;
                    timeBuf.modtime = modtime;
                    if(utime(fileStr.ascii(), &timeBuf) == -1){
                        if(KMessageBox::warningContinueCancel(view,
                                                              i18n("Could not update timestamp for ")+i->text(),
                                                              i18n("File Error!"))
                           == KMessageBox::Cancel){
                            mgr->slotUpdateView();
                            return;
                        }

                    }
                    --modtime;
                }
            }
            i = i->nextItem();
        }
        mgr->slotUpdateView();
    }
    else if(id >= 25 && id < 100){
        // Hotlistbox Operations
        id = id-25;
        qWarning("HotListBox drop for %s", pathList[id].latin1());
        QStringList fileList;
        CustomIconViewItem *i = view->firstItem();
        while (i){
            if(i->isSelected())
                fileList.append(i->text());
            i = i->nextItem();
        }
        if(fileList.isEmpty())
            return;
        Window root;
        Window child;
        int root_x, root_y, win_x, win_y;
        uint keybstate;
        XQueryPointer(qt_xdisplay(), qt_xrootwin(), &root, &child,
                      &root_x, &root_y, &win_x, &win_y, &keybstate );
        if(keybstate & ShiftMask)
            KIFFileTransfer::transferFiles(fileList, pathList[id],
                                           QDropEvent::Move);
        else
            KIFFileTransfer::transferFiles(fileList, pathList[id],
                                           QDropEvent::Copy);
    }
    else if(id >= 100 && id < 300){
        // Batch Operations
        QStringList fileList;
        CustomIconViewItem *i = view->firstItem();
        bool reload;
        while (i){
            if(i->isSelected())
                fileList.append(i->text());
            i = i->nextItem();
        }
        if(fileList.isEmpty())
            return;
        view->watch()->stopDirScan(currentDir);
        // transform menu
        if(id == 100){
            reload = KIFBatchEffect::effect(Scale, view, fileList, mgr);
        }
        if(id == 101){
            reload = KIFBatchEffect::effect(FlipH, view, fileList, mgr);
        }
        if(id == 102){
            reload = KIFBatchEffect::effect(FlipV, view, fileList, mgr);
        }
        if(id == 103){
            reload = KIFBatchEffect::effect(Rot90, view, fileList, mgr);
        }
        if(id == 104){
            reload = KIFBatchEffect::effect(Rot180, view, fileList, mgr);
        }
        if(id == 105){
            reload = KIFBatchEffect::effect(Rot270, view, fileList, mgr);
        }
        // enhance menu
        if(id == 125){
            reload = KIFBatchEffect::effect(IncContrast, view, fileList, mgr);
        }
        if(id == 126){
            reload = KIFBatchEffect::effect(DecContrast, view, fileList, mgr);
        }
        if(id == 127){
            reload = KIFBatchEffect::effect(Equalize, view, fileList, mgr);
        }
        if(id == 128){
            reload = KIFBatchEffect::effect(Normalize, view, fileList, mgr);
        }
        if(id == 129){
            reload = KIFBatchEffect::effect(Invert, view, fileList, mgr);
        }
        if(id == 130){
            reload = KIFBatchEffect::effect(Grayscale, view, fileList, mgr);
        }
        // normal effect menu
        if(id == 150){
            reload = KIFBatchEffect::effect(AddText, view, fileList, mgr);
        }
        if(id == 151){
            reload = KIFBatchEffect::effect(Border, view, fileList, mgr);
        }
        if(id == 152){
            reload = KIFBatchEffect::effect(AddNoise, view, fileList, mgr);
        }
        if(id == 153){
            reload = KIFBatchEffect::effect(Despeckle, view, fileList, mgr);
        }
        if(id == 154){
            reload = KIFBatchEffect::effect(Emboss, view, fileList, mgr);
        }
        if(id == 155){
            reload = KIFBatchEffect::effect(Sharpen, view, fileList, mgr);
        }
        if(id == 156){
            reload = KIFBatchEffect::effect(Blur, view, fileList, mgr);
        }
        if(id == 157){
            reload = KIFBatchEffect::effect(Threshold, view, fileList, mgr);
        }
        if(id == 158){
            reload = KIFBatchEffect::effect(Edge, view, fileList, mgr);
        }
        if(id == 159){
            reload = KIFBatchEffect::effect(Spread, view, fileList, mgr);
        }
        if(id == 160){
            reload = KIFBatchEffect::effect(Shade, view, fileList, mgr);
        }
        // special effect menu
        if(id == 175){
            reload = KIFBatchEffect::effect(Solarize, view, fileList, mgr);
        }
        if(id == 176){
            reload = KIFBatchEffect::effect(Swirl, view, fileList, mgr);
        }
        if(id == 177){
            reload = KIFBatchEffect::effect(Implode, view, fileList, mgr);
        }
        if(id == 178){
            reload = KIFBatchEffect::effect(Wave, view, fileList, mgr);
        }
        if(id == 179){
            reload = KIFBatchEffect::effect(OilPaint, view, fileList, mgr);
        }
        if(id == 180){
            reload = KIFBatchEffect::effect(Charcoal, view, fileList, mgr);
        }
        view->watch()->restartDirScan(currentDir);
        if(reload){
            mgr->slotUpdateView();
            mgr->resetImageBuffer();
        }
    }
    else if(id >= 300 && id < 1000){
        // convert;
        QStringList fileList;
        CustomIconViewItem *i = view->firstItem();
        while (i){
            if(i->isSelected())
                fileList.append(i->text());
            i = i->nextItem();
        }
        if(fileList.isEmpty())
            return;

        QStrList iotypes = kifImageOutputFormats();
        const char *format = iotypes.at(id-300);
        qWarning("Convert to format: %s", format);
        bool useQuality = false;
        int quality;
        bool psudeoClass, result;
        if(qstricmp(format, "PNG") == 0 || qstricmp(format, "JPEG") == 0){
            KIFQualityDialog dlg(false, format);
            if(dlg.exec() == QDialog::Rejected){
                return;
            }
            quality = dlg.quality();
            psudeoClass = dlg.psudeoClass();
            useQuality = true;
        }
        QString srcStr;
        QString destStr;
        QFileInfo fi;
        QImage img;
        for(QStringList::Iterator it = fileList.begin(); it != fileList.end(); ++it) {
            srcStr = currentDir + "/" + (*it);
            fi.setFile(srcStr);
            destStr = currentDir + "/" + fi.baseName() + "." +
                kifImageTypeToExtension(format);
            qWarning("Converting %s to %s", srcStr.latin1(), destStr.latin1());
            if(qstrcmp(kifImageType(srcStr).ascii(), format) == 0){
                qWarning("%s already in requested format, skipping.",
                         srcStr.latin1());
            }
            else{
                if(!kifLoadImage(img, srcStr)){
                    KMessageBox::sorry(view, i18n("Cannot load image :") + srcStr,
                                       i18n("Pixie Error!"));
                    return;
                }
                if(useQuality){
                    if(psudeoClass && img.depth() > 8){
                        QImage newImg = img.convertDepth(8, QPixmap::ColorOnly |
                                                         QPixmap::DiffuseDither |
                                                         QPixmap::ThresholdAlphaDither |
                                                         QPixmap::AvoidDither);
                        result = kifSaveImage(newImg, destStr, format, quality);
                    }
                    else{
                        result = kifSaveImage(img, destStr, format, quality);
                    }
                }
                else
                    result = kifSaveImage(img, destStr, format);

                if(!result){
                    KMessageBox::sorry(view, i18n("Cannot save image :") + destStr,
                                       i18n("Pixie Error!"));
                    return;
                }
            }
        }
    }
    else if(id >= 1000 && id < 2000){
        // add catagory
        /*
        QStringList catList(*kifapp()->catagoryManager()->catagoryList());
        int catId = kifapp()->catagoryManager()->id(catList[id-1000]);
        CustomIconViewItem *i = view->firstItem();
        while(i){ // check for multiple selection
            if(i->isSelected())
                view->addCatagory((KIFThumbViewItem *)i, catId);
            i = i->nextItem();
        }
        view->saveCatagories();*/

    }
    else{
        // remove catagory
        /*
        QStringList catList(*kifapp()->catagoryManager()->catagoryList());
        int catId = kifapp()->catagoryManager()->id(catList[id-1000]);
        CustomIconViewItem *i = view->firstItem();
        while(i){ // check for multiple selection
            if(i->isSelected())
                view->removeCatagory((KIFThumbViewItem *)i, catId);
            i = i->nextItem();
        }
        view->saveCatagories();*/
    }
}

void viewRightClickMenu(const QString &currentDir, UIManager *mgr,
                        KIFThumbView *view, const QPoint &pos)
{
    KPopupMenu *mnu = new KPopupMenu;
    mnu->insertTitle(BarIcon("filenew", KIcon::SizeSmall),i18n("Folder Options"));
    mnu->insertItem(BarIcon("folder", KIcon::SizeSmall), i18n("Create a new folder..."), 1);
    mnu->insertSeparator();
    mnu->insertItem(BarIcon("up", KIcon::SizeSmall), i18n("Up"), 3);
    mnu->insertItem(BarIcon("back", KIcon::SizeSmall), i18n("Back"), 4);
    mnu->insertItem(BarIcon("forward", KIcon::SizeSmall), i18n("Forward"), 5);
    mnu->insertItem(BarIcon("reload", KIcon::SizeSmall), i18n("Reload"), 6);
    mnu->insertSeparator();
    mnu->insertItem(BarIcon("editcopy", KIcon::SizeSmall), i18n("Copy folder location"), 8);
    mnu->insertItem(BarIcon("editcopy", KIcon::SizeSmall), i18n("Copy all images to FileList"), 9);
    mnu->insertItem(BarIcon("editpaste", KIcon::SizeSmall), i18n("Paste"), 10);
    mnu->insertSeparator();
    mnu->insertItem(BarIcon("wizard", KIcon::SizeSmall), i18n("Properties"), 12);
    mnu->setItemEnabled(3, mgr->isUpDirEnabled());
    mnu->setItemEnabled(4, mgr->isBackDirEnabled());
    mnu->setItemEnabled(5, mgr->isForwardDirEnabled());

    int id = mnu->exec(pos);
    delete mnu;
    if(id == 1){
        bool okay;
        QString folderStr = QInputDialog::getText(i18n("Folder Name"),
                                                  i18n("Please enter a name for the new folder."),
                                                  QLineEdit::Normal, QString::null,
                                                  &okay,
                                                  view);
        if(okay){
            if(mkdir((currentDir + "/" + folderStr).ascii(), 0777) == -1){
                KMessageBox::sorry(view, i18n("Could not create folder!"));
            }
        }
    }
    else if(id == 3)
        mgr->slotUpDir();
    else if(id == 4)
        mgr->slotBackDir();
    else if(id == 5)
        mgr->slotForwardDir();
    else if(id == 6)
        mgr->slotReload();
    else if(id == 8){
        QApplication::clipboard()->setText(currentDir);
    }
    else if(id == 9){
        CustomIconViewItem *i = view->firstItem();
        QString fileStr;
        bool isImage;
        while (i){
            fileStr = currentDir + "/" + i->text();
            KURL url("file:" + fileStr);
            isImage =  KMimeType::findByURL(url, true, true)->name().left(6) ==
                "image/";
            if(isImage)
                mgr->slotAddURL(fileStr);
            i = i->nextItem();
        }
    }
    else if(id == 10){
        view->slotPaste();
    }
    else if(id == 12){
        // Properties
        KFileItem fileitem(-1, -1, KURL(QString("file:")+currentDir+"/"));
        (void) new KPropertiesDialog(&fileitem, view);
    }
}

