#include "thumbiconview.h"
#include "uimanager.h"
#include "fileop.h"
#include "rightclick.h"
#include "../stub/ifapp.h"
#include "../stub/catagorymgr.h"
#include <qfile.h>
#include <qdatastream.h>
#include <qimage.h>
#include <qdragobject.h>
#include <qdrawutil.h>
#include <qpainter.h>
#include <qclipboard.h>
#include <klocale.h>
#include <qintdict.h>
#include <kmessagebox.h>
#include <kpixmapeffect.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kmimetype.h>
#include <kstandarddirs.h>

#include <kio/previewjob.h>
#include <kmdcodec.h>

#include <unistd.h>
#include <fnmatch.h>
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <string.h>
#include <time.h>
#include <utime.h>
#include <pwd.h>
#include <grp.h>

#include "pixiebtn.xpm"

#include "pics/photo-huge.xpm"
#include "pics/photo-large.xpm"
#include "pics/photo-mid.xpm"
#include "pics/photo-small.xpm"


bool isImageExtension(const char *str)
{
    if(!str)
        return(false);
    if(qstricmp(str, "png") == 0 || qstricmp(str, "xpm") == 0 ||
       qstricmp(str, "tiff") == 0 || qstricmp(str, "tif") == 0 ||
       qstricmp(str, "jpg") == 0 || qstricmp(str, "jpeg") == 0 ||
       qstricmp(str, "gif") == 0 || qstricmp(str, "xbm") == 0 ||
       qstricmp(str, "bmp") == 0 || qstricmp(str, "xcf") == 0 ||
       qstricmp(str, "pcx") == 0 || qstricmp(str, "xwd") == 0 ||
       qstricmp(str, "pcd") == 0 || qstricmp(str, "tga") == 0 ||
       qstricmp(str, "miff") == 0 || qstricmp(str, "pict") == 0)
        return(true);
    return(false);
}

const char* extension(char *str)
{
    char *ptr = str;
    char *last;
    while(*ptr != '\0')
        ++ptr;
    if(ptr == str)
        return(NULL);
    last = ptr;
    while(*ptr != '.' && ptr != str)
        --ptr;
    if(*ptr == '.' && ptr != last)
        return(++ptr);
    else
        return(NULL);
}


void KIFToolTip::maybeTip(const QPoint &pos)
{
    KIFThumbViewItem *item =
        (KIFThumbViewItem*)view->findItem(view->viewportToContents(pos));
    if(!item)
        return;
    QString tipStr = item->text() + '\n';
    QString sizeStr;
    struct stat *status = item->statusInfo();
    if(!S_ISDIR(status->st_mode)){
        float size = status->st_size;
        if(size >= 1024){
            size /= 1024.0;
            if(size >= 1024){
                size /= 1024.0;
                sizeStr += i18n("Size: ")+QString::number(size, 'f', 2)+"M\n";
            }
            else{
                sizeStr += i18n("Size: ")+QString::number(size, 'f', 2)+"K\n";

            }
        }
        else{
            sizeStr += i18n("Size: ")+QString::number((int)size)+"B\n";
        }
        tipStr += sizeStr;
    }
    struct passwd *pentry = getpwuid(status->st_uid);
    if(pentry)
        tipStr += i18n("Owner: ") + QString(pentry->pw_name) + ", ";
    struct group *gentry = getgrgid(status->st_gid);
    if(gentry)
        tipStr += i18n("Group: ") + QString(gentry->gr_name) + '\n';

    QString pathStr = item->path();
    tipStr += i18n("Your rights: ");
    bool anyRights = false;
    if(access(QFile::encodeName(pathStr), R_OK) == 0){
        tipStr += i18n("Read ");
        anyRights = true;
    }
    if(access(QFile::encodeName(pathStr), W_OK) == 0){
        tipStr += i18n("Write ");
        anyRights = true;
    }
    if(access(QFile::encodeName(pathStr), X_OK) == 0){
        tipStr += i18n("Execute ");
        anyRights = true;
    }
    if(!anyRights)
        tipStr += i18n("None");
    tipStr += '\n';

    QDateTime t;
    t.setTime_t(status->st_atime);
    tipStr += i18n("Last accessed: ") + t.toString() + '\n';
    t.setTime_t(status->st_mtime);
    tipStr += i18n("Last modified: ") + t.toString() + '\n';


    tipStr += i18n("Catagories: ");
    QStringList itemList(view->itemCatagories(item));
    if(itemList.count() == 0)
        tipStr += i18n("None");
    else{
        QStringList::Iterator it;
        for(it = itemList.begin(); it != itemList.end(); ++it){
            tipStr += "\n\t";
            tipStr += (*it);
        }
    }

    //tip(item->rect(), tipStr);
    QRect r = item->pixmapRect(false);
    r.moveTopLeft(view->contentsToViewport(r.topLeft()));
    tip(r, tipStr);
}

QString KIFThumbView::calcKonqThumbPath(const QString &path, int iconSize)
{
    QString sizeStr;
    if(iconSize == 48)
        sizeStr = "small";
    else if(iconSize == 64)
        sizeStr = "med";
    else if(iconSize == 90)
        sizeStr = "large";
    else if(iconSize == 112)
        sizeStr = "xxl";

    KURL url;
    url.setPath(QDir::cleanDirPath(path));
    KMD5 md5(QFile::encodeName(url.url()));
    QCString hash = md5.hexDigest();
    QString thumbPath(KGlobal::dirs()->localkdedir() + "/share/thumbnails/" +
                      QString::fromLatin1(hash.data(), 4) + "/" +
                      QString::fromLatin1(hash.data()+4, 4) + "/" +
                      QString::fromLatin1(hash.data()+8) + "/" +
                      sizeStr + "/");
    return(thumbPath);
}

KIFThumbViewItem::KIFThumbViewItem(CustomIconView *parent, const QString &textStr,
                                   struct stat *statusBuffer)
    : CustomIconViewItem(parent, textStr)
{
    isThumbnailed = false;
    status = statusBuffer;
    textDirty = true;
}

void KIFThumbViewItem::calcTextWrapping()
{
    if(!textDirty)
        return;
    textDirty = false;
    int w = ((KIFThumbView*)iconView())->currentFill->width();
    QString str(text());
    QFontMetrics *fm =  ((KIFThumbView*)iconView())->fm;
    if(fm->width(str) <= w){
        tmpStr = str;
        return;
    }

    tmpStr = "";
    int i = 0;
    while(fm->width(tmpStr + str[i]) < w)
        tmpStr += str[i++];

    QString buffer;
    if(fm->width(str)-fm->width(tmpStr) < w){
        while(fm->width(buffer + str[i]) < w && i < str.length())
            buffer += str[i++];
    }
    else{
        while(fm->width(buffer + str[i]) < w && i < str.length())
            buffer += str[i++];
        buffer.remove(0, 3);
        buffer += "...";
    }
    tmpStr = tmpStr + "\n" + buffer;
}

void KIFThumbViewItem::paintItem(QPainter *p, const QColorGroup &)
{
    KIFThumbView *view = (KIFThumbView*)iconView();
    if(!isThumbnailed || view->inDirLoad){
        return;
    }
    KPixmap *fill = view->currentFill;
    QPixmap *pix = pixmap();
    QColorGroup cg(iconView()->colorGroup());

    if(textDirty)
        calcTextWrapping();

    if(view->tmpPixmap->width() != fill->width() ||
       view->tmpPixmap->height() != fill->height()+2+view->textHeight)
        view->tmpPixmap->resize(fill->width(),
                                fill->height()+2+view->textHeight);

    view->tmpPixmap->fill(cg.base().rgb());
    QPainter painter;
    painter.begin(view->tmpPixmap);
    painter.drawPixmap(0, 0, *view->currentFill);
    if(pix){
        int dx = (view->currentFill->width() - pix->width())/2;
        int dy = (view->currentFill->height() - pix->height())/2;
        painter.drawPixmap(dx, dy, *pix);

    }
    if(isSelected()){
        QBrush fillBrush(cg.highlight(), Qt::Dense4Pattern);
        painter.fillRect(0, 0, fill->width(), fill->height(), fillBrush);
        painter.fillRect(0, fill->height()+2, fill->width(),
                         view->textHeight, cg.highlight());
        painter.setPen(cg.highlightedText());
        painter.drawText(0, fill->height()+2, fill->width(),
                         view->textHeight, Qt::AlignHCenter | AlignTop, tmpStr);
    }
    else{
        painter.fillRect(0, fill->height()+2, fill->width(),
                         view->textHeight, cg.base());
        painter.drawText(0, fill->height()+2, fill->width(),
                         view->textHeight, Qt::AlignHCenter | AlignTop, tmpStr);
    }
    painter.end();
    p->drawPixmap(x(), y(), *view->tmpPixmap);
}

void KIFThumbViewItem::swap(KIFThumbViewItem *item)
{
    bool srcThumbnailed = item->isThumbnailed;
    bool srcMakeThumbs = item->makeThumbs;
    bool srcDirty = item->textDirty;
    struct stat *srcStatus = item->status;
    QString srcStr(item->tmpStr);
    QString srcText(item->text());

    QPixmap srcPixmap;
    if(item->pixmap()){
        srcPixmap = *item->pixmap();
    }

    item->isThumbnailed = isThumbnailed;
    item->makeThumbs = makeThumbs;
    item->status = status;
    item->tmpStr = tmpStr;
    item->textDirty = textDirty;
    if(pixmap())
        item->setPixmap(*pixmap());
    else
        item->setPixmap(QPixmap());
    item->setText(text());

    isThumbnailed = srcThumbnailed;
    makeThumbs = srcMakeThumbs;
    textDirty = srcDirty;
    status = srcStatus;
    tmpStr = srcStr;
    setText(srcText);
    setPixmap(srcPixmap);
}




KIFThumbViewItem::~KIFThumbViewItem()
{
    free(status);
}

QString KIFThumbViewItem::path()
{
    return(((KIFThumbView *)iconView())->currentDir + "/" + text());
}

void KIFThumbViewItem::blendMimetypeIcon(QImage &mainImage,
                                         QImage &iconImage)
{
    int x, y;
    int dx = mainImage.width()-iconImage.width();
    int dy = mainImage.height()-iconImage.height();
    int sx, sy;
    float srcPercent, destPercent;
    unsigned char alpha, r, g, b;
    unsigned int *srcData, *destData;

    for(y=dy, sy=0; sy < iconImage.height(); ++y, ++sy){
        srcData = (unsigned int *)iconImage.scanLine(sy);
        destData = (unsigned int *)mainImage.scanLine(y);
        for(x=dx, sx=0; sx < iconImage.width(); ++x, ++sx){
            alpha = qAlpha(srcData[sx]);
            alpha = alpha >> 2;
            if(alpha){
                srcPercent = ((float)alpha)/255.0;
                destPercent = 1.0-srcPercent;
                r = (int)((srcPercent*qRed(srcData[sx])) +
                          (destPercent*qRed(destData[x])));
                g = (int)((srcPercent*qGreen(srcData[sx])) +
                          (destPercent*qGreen(destData[x])));
                b = (int)((srcPercent*qBlue(srcData[sx])) +
                          (destPercent*qBlue(destData[x])));
                destData[x] = qRgba(r, g, b, 255);
            }
        }
    }
}

void KIFThumbViewItem::updateThumbnail(const QString &path,
                                       int iconSize, bool useFrame)

{
    if(isThumbnailed)
        return;

    KIFThumbView *view = (KIFThumbView*)iconView();
    QFileInfo fi(path+"/"+text());

    bool isImage = false;
    bool isText = false;
    bool isHTML = false;
    bool isPSPDF = false;
    bool isSound = false;
    KURL url("file:"+fi.absFilePath());
    QString mimeStr = KMimeType::findByURL(url, true, true)->name();

    isImage = mimeStr.left(6) == "image/";
    if(!isImage && view->mgr->previewHTML)
        isHTML = mimeStr == "text/html";
    if(!isImage && !isHTML && view->mgr->previewText)
        isText = mimeStr.left(5) == "text/";
    if(!isImage && !isText && !isHTML && view->mgr->previewPS)
        isPSPDF = mimeStr.right(3)  == "/ps" ||
            mimeStr.right(4) == "/pdf";
    if(!isImage && !isText && !isHTML && view->mgr->previewSound)
        isSound = mimeStr.left(6) == "audio/";

    QPixmap currentPix;

    bool hasPixieThumb = QFile::exists(view->pixieThumbPath + fi.fileName())&&
        QFileInfo(view->pixieThumbPath+fi.fileName()).lastModified() >= fi.lastModified();
    bool checkKonqThumb = !hasPixieThumb && (isImage || isText || isHTML ||
                                             isPSPDF || isSound);
    bool hasKonqThumb = checkKonqThumb && view->hasKonqThumbnails &&
        QFile::exists(view->konqThumbPath + fi.fileName()) &&
        QFileInfo(view->konqThumbPath + fi.fileName()).lastModified() >=
        fi.lastModified();

    if(hasPixieThumb || hasKonqThumb){
        if(!isText){
            if(view->mgr->internalImagePreview)
                currentPix.load(hasPixieThumb ? view->pixieThumbPath + fi.fileName() :
                                view->konqThumbPath + fi.fileName());
            else
                currentPix.load(hasKonqThumb ? view->konqThumbPath + fi.fileName() :
                                view->pixieThumbPath + fi.fileName());
        }
        else{
            // blend mimetype icon
            KIconLoader *ldr = KGlobal::iconLoader();
            QImage tmpImage;
            if(view->mgr->internalImagePreview)
                tmpImage.load(hasPixieThumb ? view->pixieThumbPath + fi.fileName() :
                              view->konqThumbPath + fi.fileName());
            else
                tmpImage.load(hasKonqThumb ? view->konqThumbPath + fi.fileName() :
                              view->pixieThumbPath + fi.fileName());
            if(tmpImage.depth() < 32)
                tmpImage = tmpImage.convertDepth(32);
            QString mimeIconStr = KMimeType::iconForURL(url);
            QImage *iconImage =  kifapp()->mimetypeImageDict()->
                find(mimeIconStr.ascii());
            if(!iconImage){
                QString tmpStr;
                if(iconSize >= 90)
                    tmpStr = ldr->iconPath(mimeIconStr, -64);
                else if(iconSize >= 64)
                    tmpStr = ldr->iconPath(mimeIconStr, -48);
                else
                    tmpStr = ldr->iconPath(mimeIconStr, -32);
                iconImage = new QImage(tmpStr);
                if(iconImage->isNull()){
                    qWarning("Unable to load mimetype image: %s",
                             tmpStr.latin1());
                    delete iconImage;
                    iconImage = NULL;
                }
                else{
                    if(iconImage->depth() < 32)
                        *iconImage = iconImage->convertDepth(32);
                    kifapp()->mimetypeImageDict()->insert(mimeIconStr.ascii(),
                                                          iconImage);
                }
            }
            if(iconImage)
                blendMimetypeIcon(tmpImage, *iconImage);
            currentPix.convertFromImage(tmpImage);
        }
    }
    if(currentPix.isNull()){
        if(iconSize >= 64){
            if(isImage){
                if(iconSize > 100){
                    if(view->hugeImage == NULL)
                        view->hugeImage = new QPixmap(photo_huge_xpm);
                    currentPix = *view->hugeImage;
                }
                else if(iconSize >= 90){
                    if(view->largeImage == NULL)
                        view->largeImage = new QPixmap(photo_large_xpm);
                    currentPix = *view->largeImage;
                }
                else{
                    if(view->medImage == NULL)
                        view->medImage = new QPixmap(photo_mid_xpm);
                    currentPix = *view->medImage;
                }

            }
            else
                currentPix = KMimeType::pixmapForURL(url, 0, KIcon::Desktop, 64);
        }
        else{
            if(isImage){
                if(view->smallImage == NULL)
                    view->smallImage = new QPixmap(photo_small_xpm);
                currentPix = *view->smallImage;
            }
            else{
                currentPix = KMimeType::pixmapForURL(url, 0, KIcon::Desktop, 48);
            }
        }
    }
    setPixmap(currentPix);
    isThumbnailed = true;
}

void KIFThumbViewItem::setPreview(const QPixmap &pix)
{
    setPixmap(pix);
    isThumbnailed = true;
}


void KIFThumbViewItem::makeThumbnail(const QString &path, int iconSize,
                                     const QString &thumbPath,
                                     bool saveThumbnail)
{
    QPixmap currentPix;
    bool isImage = false;
    bool isText = false;
    KIFThumbView *view = (KIFThumbView*)iconView();

    QFileInfo fi(path+"/"+text());
    KURL url("file:"+fi.absFilePath());
    QString mimeStr =  KMimeType::findByURL(url, true, true)->name();

    isImage = mimeStr.left(6) == "image/";
    if(!isImage && view->mgr->previewText)
        isText = mimeStr != "text/html" && mimeStr.left(5) == "text/";
    if(!isImage && !isText)
        return;

    // does thumbnail already exist and is up to date?
    if((QFile::exists(view->pixieThumbPath + fi.fileName())&&
        QFileInfo(view->pixieThumbPath+fi.fileName()).lastModified() >= fi.lastModified())
       || (view->hasKonqThumbnails &&
           QFile::exists(view->konqThumbPath + fi.fileName()) &&
           QFileInfo(view->konqThumbPath + fi.fileName()).lastModified() >=
           fi.lastModified())){
        return;
    }

    if(isImage){
        QImage image;
        kifLoadImage(image, path+"/"+text());
        QString descStr;
        descStr.sprintf("%d %d %d", image.width(), image.height(),
                        image.depth());
        if(image.width() > iconSize || image.height() > iconSize){
            if(image.width() > image.height()){
                float percent = (((float)iconSize)/image.width());
                int h = (int)(image.height()*percent);
                image = image.smoothScale(iconSize, h);
            }
            else{
                float percent = (((float)iconSize)/image.height());
                int w = (int)(image.width()*percent);
                image = image.smoothScale(w, iconSize);
            }
        }
        if(image.isNull())
            return;
        if(saveThumbnail){
            //image.save(thumbPath + text(), "PNG");
            QImageIO io(thumbPath+text(), "PNG");
            io.setImage(image);
            io.setQuality(65);
            io.write();
        }

        currentPix.convertFromImage(image);
    }
    else if(isText){
        char buffer[256];
        FILE *fp = fopen(QFile::encodeName(path+"/"+text()), "r");
        if(!fp)
            return;
        // make fuzzy
        currentPix.resize(iconSize*2, iconSize*2);
        QFont fnt(iconView()->font().family(), 17);
        QFontMetrics fm(fnt);
        int y;
        int spacing = fm.lineSpacing()+1;

        currentPix.fill(Qt::white.rgb());
        QPainter painter;
        painter.begin(&currentPix);
        painter.setPen(Qt::black);
        painter.setFont(fnt);
        fgets(buffer, 256, fp);

        for(y=2; y+spacing <= iconSize*2-2 && !feof(fp); y+=spacing){
            painter.drawText(2, y, iconSize*2-4, spacing, Qt::AlignLeft |
                             Qt::AlignTop, buffer);
            fgets(buffer, 256, fp);
        }
        painter.end();
        fclose(fp);
        QImage img(currentPix.convertToImage());
        img = img.smoothScale(iconSize, iconSize);
        currentPix.convertFromImage(img);
        painter.begin(&currentPix);
        painter.setPen(Qt::lightGray);
        painter.drawLine(0, 0, iconSize-1, 0);
        painter.drawLine(0, 0, 0, iconSize-1);
        painter.setPen(Qt::darkGray);
        painter.drawLine(0, iconSize-1, iconSize-1, iconSize-1);
        painter.drawLine(iconSize-1, 0, iconSize-1, iconSize-1);
        painter.end();
        if(saveThumbnail)
            currentPix.save(thumbPath + text(), "PNG");

        // blend the icon
        KIconLoader *ldr = KGlobal::iconLoader();
        QString mimeIconStr = KMimeType::iconForURL(url);
        QImage *iconImage =  kifapp()->mimetypeImageDict()->
            find(mimeIconStr.ascii());
        if(!iconImage){
            QString tmpStr;
            if(iconSize >= 90)
                tmpStr = ldr->iconPath(mimeIconStr, -64);
            else if(iconSize >= 64)
                tmpStr = ldr->iconPath(mimeIconStr, -48);
            else
                tmpStr = ldr->iconPath(mimeIconStr, -32);
            iconImage = new QImage(tmpStr);
            if(iconImage->isNull()){
                qWarning("Unable to load mimetype image: %s",
                         tmpStr.latin1());
                delete iconImage;
                iconImage = NULL;
            }
            else{
                if(iconImage->depth() < 32)
                    *iconImage = iconImage->convertDepth(32);
                kifapp()->mimetypeImageDict()->insert(mimeIconStr.ascii(),
                                                      iconImage);
            }
        }
        // another convert... we needed a painter to draw the text, an image
        // to scale the blur, a painter to draw the frame afterwards, and
        // now an image to blend the mimetype - ugh, thankfully it ain't
        // big ;-)
        if(iconImage){
            img = currentPix.convertToImage();
            if(img.depth() < 32)
                img = img.convertDepth(32);
            blendMimetypeIcon(img, *iconImage);
            currentPix.convertFromImage(img);
        }

    }
    else
        return;

    setPixmap(currentPix);
    isThumbnailed = true;
    QRect r(rect());
    if(QRect(view->contentsX(), view->contentsY(),
             view->visibleWidth(), view->visibleHeight()).intersects(r))
        view->repaintContents(r.x()-1, r.y()-1, r.width()+2, r.height()+2,
                              false);

}


KIFThumbView::KIFThumbView(UIManager *manager, QWidget *parent, const char *name)
    : CustomIconView(parent, name)
{
    mgr = manager;
    viewport()->setBackgroundMode(NoBackground);
    setGridX(95);
    setGridY(95);
    setAutoArrange(false);
    setSorting(false);
    setResizeMode(Adjust);
    setWordWrapIconText(true);
    setSelectionMode(CustomIconView::Extended);
    connect(this, SIGNAL(dropped(QDropEvent *, const QValueList<CustomIconDragItem>&)),
            this, SLOT(slotIconDrop(QDropEvent *, const QValueList<CustomIconDragItem>&)));
    connect(this, SIGNAL(itemRenamed(CustomIconViewItem *, const QString &)), this,
            SLOT(slotItemRenamed(CustomIconViewItem *, const QString &)));
    connect(this, SIGNAL(rightButtonPressed(CustomIconViewItem *, const QPoint &)),
            this, SLOT(slotRightButton(CustomIconViewItem *, const QPoint &)));
    iconOnly = false;
    inDirLoad = false;
    currentSize = Large;
    useFrame = true;
    inUpdate = false;
    inGenerateThumbs = false;
    disableContentsResize = false;

    thumbJob = NULL;

    oldestFile = newestFile = 0;

    fm = new QFontMetrics(fontMetrics());
    textHeight = fm->lineSpacing()*2;

    dirWatch = new KDirWatch;
    connect(dirWatch, SIGNAL(dirty(const QString &)), this,
            SLOT(slotDirChanged(const QString &)));
    dirWatch->startScan();
    // preview stuff
    autoPreview = true;
    hugeFill = new KPixmap;
    largeFill = new KPixmap;
    medFill = new KPixmap;
    smallFill = new KPixmap;
    tmpPixmap = new KPixmap;
    hugeImage = largeImage = medImage = smallImage = currentFill = NULL;


    // hack to fix some odd painting of pixmapped backgrounds
    // tooltips
    useToolTips = true;
    toolTips = new KIFToolTip(viewport(), this);

    catDict.setAutoDelete(true);
}

KIFThumbView::~KIFThumbView()
{
    qWarning("In thumb view destructor");
    stopProcessing = true;
    if(thumbJob)
        thumbJob->kill();

    delete fm;
    delete dirWatch;
    delete hugeFill;
    delete largeFill;
    delete medFill;
    delete smallFill;
    delete tmpPixmap;

    if(hugeImage)
        delete hugeImage;
    if(largeImage)
        delete largeImage;
    if(medImage)
        delete medImage;
    if(smallImage)
        delete smallImage;
}

void KIFThumbView::updateTiles()
{
    QImage img(pixiebtn_xpm);
    hugeFill->convertFromImage(img);
    largeFill->convertFromImage(img.smoothScale(94, 94));
    medFill->convertFromImage(img.smoothScale(68, 68));
    smallFill->convertFromImage(img.smoothScale(52, 52));
}

/*
void KIFThumbView::drawBackground(QPainter *p, const QRect &r)
{
    p->fillRect(r, Qt::white);
}*/

void KIFThumbView::setIconOnly(bool en)
{
    iconOnly = en;
}


void KIFThumbView::slotStopClicked()
{
    stopProcessing = true;
    if(inGenerateThumbs && thumbJob){
        qWarning("Killing KIO thumbnail generation");
        thumbJob->kill(false);
        thumbJob = NULL;
    }
    inGenerateThumbs = false;
}

void KIFThumbView::drawFill(KPixmap &pix)
{
    if(framePixmap.isNull()){
        if(QPixmap::defaultDepth() > 8){
            KPixmapEffect::gradient(pix, frameColor.light(130), frameColor.dark(130),
                                    KPixmapEffect::DiagonalGradient);
            QPainter p(&pix);
            p.setPen(frameColor.dark(150));
            p.drawRect(0, 0, pix.width(), pix.height());
            qDrawShadePanel(&p, 1, 1, pix.width()-2, pix.height()-2,
                            palette().active(), false, 1);
            p.end();
        }
        else{
            pix.fill(frameColor.rgb());
            QPainter p(&pix);
            qDrawShadePanel(&p, 0, 0, pix.width(), pix.height(),
                            palette().active(), false, 2);
            p.end();
        }

    }
    else{
        QImage img = framePixmap.convertToImage();
        img = img.smoothScale(pix.width(), pix.height());
        pix.convertFromImage(img);
    }
}

void KIFThumbView::slotDirChanged(const QString &)
{
    qWarning("In slotDirChanged");
    emit updateMe();
}

struct dirinfo
{
    char *name;
    struct stat status;
    struct dirinfo *next;
};

QIntDict<CatInfo> *compareDict;
bool placeImagesOnTop;

int compareNameAscending(const void *a, const void *b)
{
    const dirinfo **a_dir = (const dirinfo **)a;
    const dirinfo **b_dir = (const dirinfo **)b;
    if(S_ISDIR(a_dir[0]->status.st_mode))
        if(S_ISDIR(b_dir[0]->status.st_mode))
            return(strcasecmp(a_dir[0]->name, b_dir[0]->name));
        else
            return(-1);
    else if(S_ISDIR(b_dir[0]->status.st_mode))
        return(1);
    if(placeImagesOnTop){
        if(isImageExtension(extension(a_dir[0]->name))){
            if(isImageExtension(extension(b_dir[0]->name)))
                return(strcasecmp(a_dir[0]->name, b_dir[0]->name));
            else
                return(-1);
        }
        else if(isImageExtension(extension(b_dir[0]->name)))
            return(1);
    }
    return(strcasecmp(a_dir[0]->name, b_dir[0]->name));
}

int compareNameDescending(const void *a, const void *b)
{
    const dirinfo **a_dir = (const dirinfo **)a;
    const dirinfo **b_dir = (const dirinfo **)b;
    if(S_ISDIR(a_dir[0]->status.st_mode))
        if(S_ISDIR(b_dir[0]->status.st_mode))
            return(strcasecmp(a_dir[0]->name, b_dir[0]->name));
        else
            return(-1);
    else if(S_ISDIR(b_dir[0]->status.st_mode))
        return(1);
    if(placeImagesOnTop){
        if(isImageExtension(extension(a_dir[0]->name))){
            if(isImageExtension(extension(b_dir[0]->name)))
                return(strcasecmp(b_dir[0]->name, a_dir[0]->name));
            else
                return(-1);
        }
        else if(isImageExtension(extension(b_dir[0]->name)))
            return(1);
    }
    return(strcasecmp(b_dir[0]->name, a_dir[0]->name));
}

int compareSizeAscending(const void *a, const void *b)
{
    const dirinfo **a_dir = (const dirinfo **)a;
    const dirinfo **b_dir = (const dirinfo **)b;
    if(S_ISDIR(a_dir[0]->status.st_mode))
        if(S_ISDIR(b_dir[0]->status.st_mode))
            return(strcasecmp(a_dir[0]->name, b_dir[0]->name));
        else
            return(-1);
    else if(S_ISDIR(b_dir[0]->status.st_mode))
        return(1);
    if(placeImagesOnTop){
        if(isImageExtension(extension(a_dir[0]->name))){
            if(isImageExtension(extension(b_dir[0]->name)))
                return(b_dir[0]->status.st_size - a_dir[0]->status.st_size);
            else
                return(-1);
        }
        else if(isImageExtension(extension(b_dir[0]->name)))
            return(1);
    }
    return(b_dir[0]->status.st_size - a_dir[0]->status.st_size);
}

int compareSizeDescending(const void *a, const void *b)
{
    const dirinfo **a_dir = (const dirinfo **)a;
    const dirinfo **b_dir = (const dirinfo **)b;
    if(S_ISDIR(a_dir[0]->status.st_mode))
        if(S_ISDIR(b_dir[0]->status.st_mode))
            return(strcasecmp(a_dir[0]->name, b_dir[0]->name));
        else
            return(-1);
    else if(S_ISDIR(b_dir[0]->status.st_mode))
        return(1);
    if(placeImagesOnTop){
        if(isImageExtension(extension(a_dir[0]->name))){
            if(isImageExtension(extension(b_dir[0]->name)))
                return(a_dir[0]->status.st_size - b_dir[0]->status.st_size);
            else
                return(-1);
        }
        else if(isImageExtension(extension(b_dir[0]->name)))
            return(1);
    }
    return(a_dir[0]->status.st_size - b_dir[0]->status.st_size);
}

int compareDateAscending(const void *a, const void *b)
{
    const dirinfo **a_dir = (const dirinfo **)a;
    const dirinfo **b_dir = (const dirinfo **)b;
    if(S_ISDIR(a_dir[0]->status.st_mode))
        if(S_ISDIR(b_dir[0]->status.st_mode))
            return(strcasecmp(a_dir[0]->name, b_dir[0]->name));
        else
            return(-1);
    else if(S_ISDIR(b_dir[0]->status.st_mode))
        return(1);
    if(placeImagesOnTop){
        if(isImageExtension(extension(a_dir[0]->name))){
            if(isImageExtension(extension(b_dir[0]->name)))
                return(b_dir[0]->status.st_mtime - a_dir[0]->status.st_mtime);
            else
                return(-1);
        }
        else if(isImageExtension(extension(b_dir[0]->name)))
            return(1);
    }
    return(b_dir[0]->status.st_mtime - a_dir[0]->status.st_mtime);
}

int compareDateDescending(const void *a, const void *b)
{
    const dirinfo **a_dir = (const dirinfo **)a;
    const dirinfo **b_dir = (const dirinfo **)b;
    if(S_ISDIR(a_dir[0]->status.st_mode))
        if(S_ISDIR(b_dir[0]->status.st_mode))
            return(strcasecmp(a_dir[0]->name, b_dir[0]->name));
        else
            return(-1);
    else if(S_ISDIR(b_dir[0]->status.st_mode))
        return(1);
    if(placeImagesOnTop){
        if(isImageExtension(extension(a_dir[0]->name))){
            if(isImageExtension(extension(b_dir[0]->name)))
                return(a_dir[0]->status.st_mtime - b_dir[0]->status.st_mtime);
            else
                return(-1);
        }
        else if(isImageExtension(extension(b_dir[0]->name)))
            return(1);
    }
    return(a_dir[0]->status.st_mtime - b_dir[0]->status.st_mtime);
}

// TODO - place images on top for catagories
int compareNameAscendingCatagorized(const void *a, const void *b)
{
    const dirinfo **a_dir = (const dirinfo **)a;
    const dirinfo **b_dir = (const dirinfo **)b;
    if(S_ISDIR(a_dir[0]->status.st_mode))
        if(S_ISDIR(b_dir[0]->status.st_mode))
            return(strcasecmp(a_dir[0]->name, b_dir[0]->name));
        else
            return(-1);
    else if(S_ISDIR(b_dir[0]->status.st_mode))
        return(1);
    if(compareDict->count() != 0){
        CatInfo *acat = compareDict->find((unsigned long)
                                          a_dir[0]->status.st_ino);
        CatInfo *bcat = compareDict->find((unsigned long)
                                          b_dir[0]->status.st_ino);
        if(acat || bcat){
            if(!acat)
                return(1);
            else if(!bcat)
                return(-1);
            else if(acat->catagories[0] == bcat->catagories[0])
                return(strcasecmp(a_dir[0]->name, b_dir[0]->name));
            else{
                QString **catarray = kifapp()->catagoryManager()->array();
                return(QString::compare(*catarray[acat->catagories[0]],
                                        *catarray[bcat->catagories[0]]));
            }
        }
    }
    return(strcasecmp(a_dir[0]->name, b_dir[0]->name));
}

int compareNameDescendingCatagorized(const void *a, const void *b)
{
    const dirinfo **a_dir = (const dirinfo **)a;
    const dirinfo **b_dir = (const dirinfo **)b;
    if(S_ISDIR(a_dir[0]->status.st_mode))
        if(S_ISDIR(b_dir[0]->status.st_mode))
            return(strcasecmp(a_dir[0]->name, b_dir[0]->name));
        else
            return(-1);
    else if(S_ISDIR(b_dir[0]->status.st_mode))
        return(1);
    if(compareDict->count() != 0){
        CatInfo *acat = compareDict->find((unsigned long)
                                          a_dir[0]->status.st_ino);
        CatInfo *bcat = compareDict->find((unsigned long)
                                          b_dir[0]->status.st_ino);
        if(acat || bcat){
            if(!acat)
                return(1);
            else if(!bcat)
                return(-1);
            else if(acat->catagories[0] == bcat->catagories[0])
                return(strcasecmp(b_dir[0]->name, a_dir[0]->name));
            else{
                QString **catarray = kifapp()->catagoryManager()->array();
                return(QString::compare(*catarray[acat->catagories[0]],
                                        *catarray[bcat->catagories[0]]));
            }
        }
    }
    return(strcasecmp(b_dir[0]->name, a_dir[0]->name));
}

int compareSizeAscendingCatagorized(const void *a, const void *b)
{
    const dirinfo **a_dir = (const dirinfo **)a;
    const dirinfo **b_dir = (const dirinfo **)b;
    if(S_ISDIR(a_dir[0]->status.st_mode))
        if(S_ISDIR(b_dir[0]->status.st_mode))
            return(strcasecmp(a_dir[0]->name, b_dir[0]->name));
        else
            return(-1);
    else if(S_ISDIR(b_dir[0]->status.st_mode))
        return(1);
    if(compareDict->count() != 0){
        CatInfo *acat = compareDict->find((unsigned long)
                                          a_dir[0]->status.st_ino);
        CatInfo *bcat = compareDict->find((unsigned long)
                                          b_dir[0]->status.st_ino);
        if(acat || bcat){
            if(!acat)
                return(1);
            else if(!bcat)
                return(-1);
            else if(acat->catagories[0] == bcat->catagories[0])
                return(b_dir[0]->status.st_size - a_dir[0]->status.st_size);
            else{
                QString **catarray = kifapp()->catagoryManager()->array();
                return(QString::compare(*catarray[acat->catagories[0]],
                                        *catarray[bcat->catagories[0]]));
            }
        }
    }
    return(b_dir[0]->status.st_size - a_dir[0]->status.st_size);
}

int compareSizeDescendingCatagorized(const void *a, const void *b)
{
    const dirinfo **a_dir = (const dirinfo **)a;
    const dirinfo **b_dir = (const dirinfo **)b;
    if(S_ISDIR(a_dir[0]->status.st_mode))
        if(S_ISDIR(b_dir[0]->status.st_mode))
            return(strcasecmp(a_dir[0]->name, b_dir[0]->name));
        else
            return(-1);
    else if(S_ISDIR(b_dir[0]->status.st_mode))
        return(1);
    if(compareDict->count() != 0){
        CatInfo *acat = compareDict->find((unsigned long)
                                          a_dir[0]->status.st_ino);
        CatInfo *bcat = compareDict->find((unsigned long)
                                          b_dir[0]->status.st_ino);
        if(acat || bcat){
            if(!acat)
                return(1);
            else if(!bcat)
                return(-1);
            else if(acat->catagories[0] == bcat->catagories[0])
                return(a_dir[0]->status.st_size - b_dir[0]->status.st_size);
            else{
                QString **catarray = kifapp()->catagoryManager()->array();
                return(QString::compare(*catarray[acat->catagories[0]],
                                        *catarray[bcat->catagories[0]]));
            }
        }
    }
    return(a_dir[0]->status.st_size - b_dir[0]->status.st_size);
}

int compareDateAscendingCatagorized(const void *a, const void *b)
{
    const dirinfo **a_dir = (const dirinfo **)a;
    const dirinfo **b_dir = (const dirinfo **)b;
    if(S_ISDIR(a_dir[0]->status.st_mode))
        if(S_ISDIR(b_dir[0]->status.st_mode))
            return(strcasecmp(a_dir[0]->name, b_dir[0]->name));
        else
            return(-1);
    else if(S_ISDIR(b_dir[0]->status.st_mode))
        return(1);
    if(compareDict->count() != 0){
        CatInfo *acat = compareDict->find((unsigned long)
                                          a_dir[0]->status.st_ino);
        CatInfo *bcat = compareDict->find((unsigned long)
                                          b_dir[0]->status.st_ino);
        if(acat || bcat){
            if(!acat)
                return(1);
            else if(!bcat)
                return(-1);
            else if(acat->catagories[0] == bcat->catagories[0])
                return(b_dir[0]->status.st_mtime - a_dir[0]->status.st_mtime);
            else{
                QString **catarray = kifapp()->catagoryManager()->array();
                return(QString::compare(*catarray[acat->catagories[0]],
                                        *catarray[bcat->catagories[0]]));
            }
        }
    }
    return(b_dir[0]->status.st_mtime - a_dir[0]->status.st_mtime);
}

int compareDateDescendingCatagorized(const void *a, const void *b)
{
    const dirinfo **a_dir = (const dirinfo **)a;
    const dirinfo **b_dir = (const dirinfo **)b;
    if(S_ISDIR(a_dir[0]->status.st_mode))
        if(S_ISDIR(b_dir[0]->status.st_mode))
            return(strcasecmp(a_dir[0]->name, b_dir[0]->name));
        else
            return(-1);
    else if(S_ISDIR(b_dir[0]->status.st_mode))
        return(1);
    if(compareDict->count() != 0){
        CatInfo *acat = compareDict->find((unsigned long)
                                          a_dir[0]->status.st_ino);
        CatInfo *bcat = compareDict->find((unsigned long)
                                          b_dir[0]->status.st_ino);
        if(acat || bcat){
            if(!acat)
                return(1);
            else if(!bcat)
                return(-1);
            else if(acat->catagories[0] == bcat->catagories[0])
                return(a_dir[0]->status.st_mtime - b_dir[0]->status.st_mtime);
            else{
                QString **catarray = kifapp()->catagoryManager()->array();
                return(QString::compare(*catarray[acat->catagories[0]],
                                        *catarray[bcat->catagories[0]]));
            }
        }
    }
    return(a_dir[0]->status.st_mtime - b_dir[0]->status.st_mtime);
}

void KIFThumbView::update(const QString &dirStr, IconSize iSize,
                          int sortType, bool createThumbs,
                          bool dupFirst, bool autoThumbnails,
                          bool catagoriesFirst,
                          const QString &nameFilter, bool updateHistory)
{
    emit setStatusBarText(i18n("Loading Directory..."));
    kapp->processEvents();

    int oldYPosition = currentDir == dirStr ? contentsY() : 0;

    if(thumbJob && currentDir != dirStr){
        qWarning("Killing old thumbnail generator");
        thumbJob->kill();
        thumbJob = NULL;
        stopProcessing = true;
        emit enableStopButton(false);
        inGenerateThumbs = false;
    }

    inDirLoad = true;
    disableContentsResize = true;
    clear();

    bool hasNameFilter = !nameFilter.isEmpty();
    const char *nameFilterStr = nameFilter.latin1();

    if(updateHistory){
        int hIndex;
        hIndex = historyList.find(hasNameFilter ?
                                  QString(dirStr+"/"+nameFilter).ascii() :
                                  dirStr.ascii());
        if(hIndex != -1)
            historyList.remove(hIndex);
        historyList.prepend(hasNameFilter ?
                            QString(dirStr+"/"+nameFilter).ascii() :
                            dirStr.ascii());
        if(historyList.count() != 1)
            emit enableBackDir(true);
        emit enableForwardDir(false);
    }
    emit enableUpDir(QFile::exists(".."));

    QTime testTime;
    testTime.start();
    if(currentDir != dirStr){
        if(!currentDir.isNull())
            dirWatch->removeDir(currentDir);
        dirWatch->addDir(dirStr);
    }

    dirWatch->stopDirScan(dirStr);

    currentSize = iSize;
    persistant = createThumbs;
    autoPreview = autoThumbnails;
    stopProcessing = false;

    currentDir = dirStr;

    int iconSize;
    QString dirPrefix;
    switch(iSize){
    case Huge:
        iconSize = 112;
        currentFill = hugeFill;
        dirPrefix = "/xxl/";
        break;
    case Large:
        iconSize = 90;
        currentFill = largeFill;
        dirPrefix = "/large/";
        break;
    case Medium:
        iconSize = 64;
        currentFill = medFill;
        dirPrefix = "/med/";
        break;
    case Small:
    default:
        iconSize = 48;
        currentFill = smallFill;
        dirPrefix = "/small/";
        break;
    }
    setGridX(iconSize+6);
    setGridY(iconSize+6);


    konqThumbPath = calcKonqThumbPath(currentDir, iconSize);
    pixieThumbPath = currentDir + "/.pics" + dirPrefix;
    hasKonqThumbnails = QFile::exists(konqThumbPath);

    int current = 1;
    bool processEntry;
    int n, count;
    struct stat *tmpBuffer;
    const char *ext;

    struct dirinfo *listitem, *listhead = NULL;
    struct dirinfo **listarray;
    struct dirent *direntry;
    DIR *dirHandle;

    count = 0;
    oldestFile = newestFile = 0;

    // read all items into a singly linked list
    dirHandle = opendir(QFile::encodeName(dirStr));
    if(!dirHandle){
        qWarning("Invalid folder given to update!");
        // FIXME
        resizeContents(0, 0);
        viewport()->setUpdatesEnabled(true);
        return;
    }
    direntry = readdir(dirHandle);
    while(direntry){
        if(direntry->d_name[0] != '.'){
            processEntry = true;
            listitem = (struct dirinfo *)malloc(sizeof(struct dirinfo));
            if(stat(QFile::encodeName(dirStr+"/"+direntry->d_name), &listitem->status) == -1){
                qWarning("Unable to stat: %s", direntry->d_name);
                free(listitem);
                processEntry = false;
            }
            else{
                if(iconOnly){
                    if(S_ISDIR(listitem->status.st_mode))
                        ;
                    else{
                        ext = extension(direntry->d_name);
                        if(!isImageExtension(ext)){
                            free(listitem);
                            processEntry = false;
                        }
                    }
                }
                if(processEntry && hasNameFilter){
                    if(fnmatch(nameFilterStr, direntry->d_name, 0) != 0)
                        processEntry = false;
                }

            }
            if(processEntry){
                if(!count)
                    oldestFile = newestFile = listitem->status.st_mtime;
                else{
                    if(listitem->status.st_mtime < oldestFile)
                        oldestFile = listitem->status.st_mtime;
                    if(listitem->status.st_mtime > newestFile)
                        newestFile = listitem->status.st_mtime;
                }
                ++count;
                listitem->name = (char *)malloc(strlen(direntry->d_name)+1);
                strcpy(listitem->name, direntry->d_name);
                listitem->next = listhead;
                listhead = listitem;
            }
        }
        direntry = readdir(dirHandle);
    }
    closedir(dirHandle);
    loadCatagories();

    // create an index to all the items
    listarray = (struct dirinfo **)malloc((sizeof(struct dirinfo *))*count);
    listitem = listhead;
    for(n=0; n < count; ++n){
        listarray[n] = listitem;
        listitem = listitem->next;
    }

    // sort
    placeImagesOnTop = mgr->imagesOnTop;
    if(catagoriesFirst && catDict.count() != 0){
        compareDict = &catDict;
        switch(sortType){
        case 0:
            qsort(listarray, count, sizeof(struct dirinfo *),
                  compareNameAscendingCatagorized);
            break;
        case 2:
            qsort(listarray, count, sizeof(struct dirinfo *),
                  compareDateAscendingCatagorized);
            break;
        case 3:
            qsort(listarray, count, sizeof(struct dirinfo *),
                  compareDateDescendingCatagorized);
            break;
        case 4:
        case 6:
            qsort(listarray, count, sizeof(struct dirinfo *),
                  compareSizeAscendingCatagorized);
            break;
        case 5:
            qsort(listarray, count, sizeof(struct dirinfo *),
                  compareSizeDescendingCatagorized);
            break;
        default:
            qsort(listarray, count, sizeof(struct dirinfo *),
                  compareNameDescendingCatagorized);
            break;
        }
    }
    else{
        switch(sortType){
        case 0:
            qsort(listarray, count, sizeof(struct dirinfo *),
                  compareNameAscending);
            break;
        case 2:
            qsort(listarray, count, sizeof(struct dirinfo *),
                  compareDateAscending);
            break;
        case 3:
            qsort(listarray, count, sizeof(struct dirinfo *),
                  compareDateDescending);
            break;
        case 4:
        case 6:
            qsort(listarray, count, sizeof(struct dirinfo *),
                  compareSizeAscending);
            break;
        case 5:
            qsort(listarray, count, sizeof(struct dirinfo *),
                  compareSizeDescending);
            break;
        default:
            qsort(listarray, count, sizeof(struct dirinfo *),
                  compareNameDescending);
            break;
        }
    }


    // insert items
    viewport()->setUpdatesEnabled(false);
    for(n=0; n < count && !stopProcessing; ++n){
        KIFThumbViewItem *i = NULL;
        listitem = listarray[n];
        tmpBuffer = (struct stat *)malloc(sizeof(struct stat));
        memcpy(tmpBuffer, &listitem->status, sizeof(struct stat));
        i = new KIFThumbViewItem(this, listitem->name, tmpBuffer);
        if(S_ISDIR(listitem->status.st_mode)){
            i->setDropEnabled(true);
            i->setDragEnabled(true);
            //i->setRenameEnabled(true);
        }
        else{
            i->setDropEnabled(false);
            i->setDragEnabled(true);
            //i->setRenameEnabled(true);
        }
        ++current;
        free(listitem->name);
        free(listitem);
    }
    free(listarray);

    disableContentsResize = false;
    //if(dupFirst)
    //    delete fileList;

    CustomIconView::slotUpdate();

    inDirLoad = false; // renable repaint

    if(oldYPosition){
        if(oldYPosition + visibleHeight() < contentsHeight())
            setContentsPos(contentsX(), oldYPosition);
        else
            setContentsPos(contentsX(), contentsHeight()-visibleHeight()-1);
    }
    updateContents();


    if(!CustomIconView::count()){
        resizeContents(0, 0);
        viewport()->setUpdatesEnabled(true);
    }

    stopProcessing = true;
    setFocus();
    //if(firstItem())
    //    setSelected(firstItem(), true);
    dirWatch->restartDirScan(dirStr);
    if(autoPreview){
        emit setStatusBarText(i18n("Automaking new previews, use View menu to disable."));
        slotGenerateThumbnails();
    }
    emit setStatusBarText(i18n("Directory Loaded"));
}

void KIFThumbView::slotUpdate()
{
    ;
}

void KIFThumbView::slotGenerateThumbnails()
{
    if(thumbJob || inGenerateThumbs){ // TODO - dialog
        KMessageBox::sorry(this, i18n("Already making thumbnail previews!"),
                           i18n("Thumbnail Error"));
        return;
    }
    dirWatch->stopDirScan(currentDir);
    stopProcessing = false;
    emit enableStopButton(true);
    inGenerateThumbs = true;

    // check and/or create dirs
    int iconSize;
    QString thumbStr;
    switch(currentSize){
    case KIFThumbView::Huge:
        thumbStr = "xxl/";
        iconSize = 112;
        break;
    case KIFThumbView::Large:
        thumbStr = "large/";
        iconSize = 90;
        break;
    case KIFThumbView::Medium:
        thumbStr = "med/";
        iconSize = 64;
        break;
    case KIFThumbView::Small:
    default:
        thumbStr = "small/";
        iconSize = 48;
        break;
    }

    QString path(currentDir);
    // Pixie
    struct stat fileInfo;
    bool canSavePixieThumb = true;
    if(stat(QFile::encodeName(path+"/.pics"), &fileInfo) == -1){
        if(mkdir(QFile::encodeName(path+"/.pics"), 0777) == -1){
            canSavePixieThumb = false;
        }
    }
    if(canSavePixieThumb &&
       stat(QFile::encodeName(path+"/.pics/"+thumbStr), &fileInfo) == -1){
        if(mkdir(QFile::encodeName(path+"/.pics/"+thumbStr), 0777) == -1){
            canSavePixieThumb = false;
        }
    }
    if(access(QFile::encodeName(path+"/.pics/"+thumbStr), W_OK) != 0)
        canSavePixieThumb = false;

    // Konq - make dirs even if we use internal previews if HTML, PS, or
    // sound is previewd
    bool canSaveKonqThumb = true;
    if(!mgr->internalImagePreview || !canSavePixieThumb ||  mgr->previewHTML
       || mgr->previewPS || mgr->previewSound){
        KURL url;
        url.setPath(QDir::cleanDirPath(currentDir));
        KMD5 md5(QFile::encodeName(url.url()));
        QCString hash = md5.hexDigest();
        QString testKonqPath = KGlobal::dirs()->localkdedir() +
            "/share/thumbnails/";
        if(stat(QFile::encodeName(testKonqPath), &fileInfo) == -1){
            if(mkdir(QFile::encodeName(testKonqPath), 0777) == -1){
                canSaveKonqThumb = false;
            }
        }
        testKonqPath += QString::fromLatin1(hash.data(), 4) + "/";
        if(canSaveKonqThumb &&
           stat(QFile::encodeName(testKonqPath), &fileInfo) == -1){
            if(mkdir(QFile::encodeName(testKonqPath), 0777) == -1){
                canSaveKonqThumb = false;
            }
        }
        testKonqPath += QString::fromLatin1(hash.data()+4, 4) + "/";
        if(canSaveKonqThumb &&
           stat(QFile::encodeName(testKonqPath), &fileInfo) == -1){
            if(mkdir(QFile::encodeName(testKonqPath), 0777) == -1){
                canSaveKonqThumb = false;
            }
        }
        testKonqPath += QString::fromLatin1(hash.data()+8) + "/";
        if(canSaveKonqThumb &&
           stat(QFile::encodeName(testKonqPath), &fileInfo) == -1){
            if(mkdir(QFile::encodeName(testKonqPath), 0777) == -1){
                canSaveKonqThumb = false;
            }
        }
        testKonqPath += thumbStr;
        if(canSaveKonqThumb &&
           stat(QFile::encodeName(testKonqPath), &fileInfo) == -1){
            if(mkdir(QFile::encodeName(testKonqPath), 0777) == -1){
                canSaveKonqThumb = false;
            }
        }
        if(access(QFile::encodeName(testKonqPath), W_OK) != 0)
            canSaveKonqThumb = false;
    }


    bool saveThumb;
    if(mgr->internalImagePreview && canSavePixieThumb){
        thumbStr = pixieThumbPath;
        saveThumb = true;
    }
    else{
        thumbStr = konqThumbPath;
        saveThumb = canSaveKonqThumb;
    }

    if(!saveThumb){
        qWarning("Could not save thumbnails!"); // shouldn't happen
    }

    // make internal image & text previews, and a list of KIO slave ones
    KIFThumbViewItem *current = (KIFThumbViewItem *)firstItem();
    KFileItemList kioslaveList;
    KFileItem *fileItem;
    KURL url;
    QString mimeStr;
    QString pathStr;

    int total = count();
    int percent = 0;
    int currentItem = 1;

    while(current && !stopProcessing){
        pathStr = QDir::cleanDirPath(currentDir + "/" + current->text());
        url.setPath(pathStr);
        mimeStr = KMimeType::findByURL(url, true, true)->name();
        if(mimeStr.left(6) == "image/"){
            current->makeThumbnail(path, iconSize, thumbStr, saveThumb);
            ++currentItem;
            if((int)(((float)currentItem/total)*100) >=  percent+10){
                percent =  (int)(((float)currentItem/total)*100);
                emit updateProgress(percent);
                kifapp()->processEvents();
            }
        }
        else if(mgr->previewHTML && mimeStr == "text/html"){
            fileItem = new KFileItem(url, mimeStr, KFileItem::Unknown);
            kioslaveList.append(fileItem);
        }
        else if(mgr->previewText && mimeStr.left(5) == "text/"){
            current->makeThumbnail(path, iconSize, thumbStr, saveThumb);
            ++currentItem;
            if((int)(((float)currentItem/total)*100) >=  percent+10){
                percent =  (int)(((float)currentItem/total)*100);
                emit updateProgress(percent);
                kifapp()->processEvents();
            }
        }
        else if(mgr->previewPS && mimeStr.right(3)  == "/ps" ||  mimeStr.right(4) == "/pdf"){
            fileItem = new KFileItem(url, mimeStr, KFileItem::Unknown);
            kioslaveList.append(fileItem);
        }
        else if(mgr->previewSound && mimeStr.left(6) == "audio/"){
            fileItem = new KFileItem(url, mimeStr, KFileItem::Unknown);
            kioslaveList.append(fileItem);
        }
        else
            ++currentItem;
        current = (KIFThumbViewItem *)current->nextItem();
    }

    qWarning("KIO Thumbnail previews: %d", kioslaveList.count());
    if(kioslaveList.count()){
        kioTotalIcons = total;
        kioCurrentIcon = currentItem;
        thumbJob = new KIO::PreviewJob(kioslaveList, iconSize, iconSize, 0, 0,
                                       true, true, NULL, true);
        connect(thumbJob, SIGNAL(gotPreview(const KFileItem *,
                                            const QPixmap &)),
                this, SLOT(slotThumbJobPreview(const KFileItem *,
                                               const QPixmap &)));
        connect(thumbJob, SIGNAL(failed(const KFileItem *)), this,
                SLOT(slotThumbJobFailed(const KFileItem *)));
        connect(thumbJob, SIGNAL(result(KIO::Job *)), this,
                SLOT(slotThumbJobResult(KIO::Job *)));
        connect(thumbJob, SIGNAL(percent(KIO::Job *, unsigned long)), this,
                SLOT(slotThumbJobProgress(KIO::Job *, unsigned long)));

    }
    else{
        thumbJob = NULL;
        kioTotalIcons = kioCurrentIcon = 0; // shouldn't matter
        emit updateProgress(100);
        kifapp()->processEvents();
        stopProcessing = true;
        emit enableStopButton(false);
        dirWatch->restartDirScan(currentDir);
        inGenerateThumbs = false;
    }

}

void KIFThumbView::insertInGrid(CustomIconViewItem *)
{
    ;
}

void KIFThumbView::insertItem(CustomIconViewItem *i, CustomIconViewItem *after)
{
    CustomIconView::insertItem(i, after);
}


void KIFThumbView::drawContents(QPainter *p, int x, int y, int w, int h)
{
    if(inUpdate || inDirLoad)
        return;
    inUpdate = true;
    QRect r(x, y, w, h);

    int iconSize;
    switch(currentSize){
    case KIFThumbView::Huge:
        iconSize = 112;
        break;
    case KIFThumbView::Large:
        iconSize = 90;
        break;
    case KIFThumbView::Medium:
        iconSize = 64;
        break;
    case KIFThumbView::Small:
    default:
        iconSize = 48;
        break;
    }

    updateRubber(p);
    QRect viewportRect(contentsToViewport(r.topLeft()), r.size());
    QRegion bgRegion(viewportRect);

    // do bg rects first
    KIFThumbViewItem *current = (KIFThumbViewItem *)findFirstVisibleItem(r);
    KIFThumbViewItem *tmp = current;
    KIFThumbViewItem *last = (KIFThumbViewItem *)findLastVisibleItem(r);
    while(current){
        bgRegion -= QRect(contentsToViewport(current->rect().topLeft()),
                          current->size());
        if(current == last)
            current = NULL;
        else
            current = (KIFThumbViewItem *)current->nextItem();
    }
    p->save();
    p->setClipRegion(bgRegion);
    p->fillRect(r, colorGroup().base());
    p->restore();
    current = tmp;
    while(current){
        current->updateThumbnail(currentPath(), iconSize, useFrame);
        current->paintItem(p, colorGroup());
        if(current == last)
            current = NULL;
        else
            current = (KIFThumbViewItem *)current->nextItem();
    }


    updateRubber(p);
    inUpdate = false;
}

void KIFThumbView::viewportPaintEvent(QPaintEvent *ev)
{

    if(inDirLoad || inUpdate)
        return;
    CustomIconView::viewportPaintEvent(ev);
}

void KIFThumbView::resizeContents(int w, int h)
{
    if(disableContentsResize){
        cachedResize.setWidth(w);
        cachedResize.setHeight(h);
    }
    else{
        CustomIconView::resizeContents(w, h);
    }
}


void KIFThumbView::showEvent(QShowEvent *ev)
{
    arrangeItemsInGrid(false);
    QScrollView::show();
}

QDragObject* KIFThumbView::dragObject()
{
    QStrList uriList;
    KIFThumbViewItem *i = (KIFThumbViewItem *)firstItem();
    KIFThumbViewItem *firstSelItem = NULL; // for pixmap
    while(i){
        if(i->isSelected() && !i->parentDir()){
            uriList.append(QUriDrag::localFileToUri(currentDir + "/" +
                                                    i->text()));
            if(!firstSelItem)
                firstSelItem = i;
        }
        i = (KIFThumbViewItem *)i->nextItem();
    }

    QUriDrag *uri = new QUriDrag(uriList, this);
    if(uriList.count() == 1)
        uri->setPixmap(*firstSelItem->pixmap());
    else{
        uri->setPixmap(KGlobal::iconLoader()->loadIcon("kmultiple",
                                                       KIcon::NoGroup,
                                                       KIcon::SizeMedium));
    }
    return(uri);
}

void KIFThumbView::slotIconDrop(QDropEvent *ev,
                                const QValueList<CustomIconDragItem> &)
{
    QStringList fileList;
    if(!QUriDrag::decodeLocalFiles(ev, fileList)){
        qWarning("Pixie: Can't decode drop.");
        return;
    }

    if(ev->source() == this){
        qWarning("Pixie: Dropped thumbnails onto their own directory.");
        return;
    }
    if(!fileList.count())
        return;
    
    int op;
    QPopupMenu opPopup;
    opPopup.insertItem(i18n("&Copy Here"), 1);
    opPopup.insertItem(i18n("&Move Here"), 2);
    opPopup.insertItem(i18n("&Link Here"), 3);
    QPoint pos = contentsToViewport(ev->pos());
    op = opPopup.exec(viewport()->mapToGlobal(pos));
    switch(op){
    case 1:
        ev->setAction(QDropEvent::Copy);
        break;
    case 2:
        ev->setAction(QDropEvent::Move);
        break;
    case 3:
        ev->setAction(QDropEvent::Link);
        break;
    default:
        return;
    }

    KIFThumbViewItem *item = (KIFThumbViewItem *)findItem(ev->pos());
    QString destStr(currentDir);
    if(item && item->dropEnabled()){
        if(item->parentDir())
            destStr += "/../";
        else
            destStr += "/" + item->text() + "/";
    }
    KIFFileTransfer::transferFiles(fileList, destStr, ev->action());
}

void KIFThumbView::slotItemRenamed(CustomIconViewItem *item, const QString &str)
{
    KIFThumbViewItem *i = (KIFThumbViewItem *)item;
    qWarning("Rename for %s to %s", i->path().latin1(),
             str.latin1());
    // must use right click now because Qt3 sets item text before calling
    // us :P
}

void KIFThumbView::keyPressEvent(QKeyEvent *ev)
{
    int key = ev->key();
    if(key == Key_Enter || key == Key_Return){
        if(currentItem())
            emit doubleClicked(currentItem());
    }
    else
        CustomIconView::keyPressEvent(ev);
}

void KIFThumbView::slotRightButton(CustomIconViewItem *item, const QPoint &pos)
{
    if(!item){
        iconViewRightButton(pos);
        return;
    }
    iconRightClickMenu(currentDir, mgr, this, (KIFThumbViewItem*)item, pos);

}

void KIFThumbView::iconViewRightButton(const QPoint &pos)
{
    viewRightClickMenu(currentDir, mgr, this, pos);
}

void KIFThumbView::slotPaste()
{
    QClipboard *cb = QApplication::clipboard();
    QStringList fileList;
    if(QUriDrag::decodeLocalFiles(cb->data(), fileList)){
        if(!fileList.count())
            return;
        KIFFileTransfer::transferFiles(fileList, currentDir,
                                       QDropEvent::Copy);
    }
    else
        KMessageBox::sorry(this, i18n("The clipboard is empty."),
                           i18n("Pixie Clipboard Error!"));
}

void KIFThumbView::slotCopy()
{
    QStrList fileList;
    CustomIconViewItem *i = firstItem();
    while (i){
        if(i->isSelected())
            fileList.append(QFile::encodeName(currentDir + "/" + i->text()));
        i = i->nextItem();
    }
    if(fileList.isEmpty())
        return;
    QUriDrag *obj = new QUriDrag(fileList);
    QClipboard *cb = QApplication::clipboard();
    cb->setData(obj);
}

bool KIFThumbView::loadCatagories()
{
    qWarning("Reading catagory database");
    catDict.clear();
    catError = false;
    if(kifapp()->catagoryManager()->catagoryList()->count() == 0){
        supportCatagories = false;
        return(true);
    }

    bool useObseleteList = false;
    QValueList<int>obseleteList;
    supportCatagories = true;

    // check for obselete items in catagory mapping
    QString fileStr(currentDir + "/.catagory-mapping-");
    fileStr += getpwuid(getuid())->pw_name;


    if(!QFile::exists(fileStr)){
        qWarning("No Pixie catagory mapping in folder");
        return(true);
    }
    QFile f(fileStr);
    if(!f.open(IO_ReadOnly)){
        // TODO: error dialog
        qWarning("Unable to open catagory mapping for folder!");
        catError = true;
        return(false);
    }
    QDataStream stream;
    stream.setDevice(&f);

    QString **catarray = kifapp()->catagoryManager()->array();
    QString tmpStr;
    unsigned char tmp;
    unsigned int id;

    while(!stream.atEnd()){
        stream >> tmpStr;
        stream >> tmp;
        id = tmp; // gcc don't like unsigned char indexes
        if(!catarray[id] || *catarray[id] != tmpStr){
            qWarning("Obselete catagory index found");
            useObseleteList = true;
            obseleteList.append(id);
        }
    }
    f.close();

    // load in catagory database
    fileStr = currentDir + "/.catagories-";
    fileStr += getpwuid(getuid())->pw_name;

    if(!QFile::exists(fileStr)){
        qWarning("No Pixie catagories in folder");
        catError = true;
        return(false);
    }
    f.setName(fileStr);
    if(!f.open(IO_ReadOnly)){
        // TODO: error dialog
        qWarning("Unable to open catagories for folder!");
        catError = true;
        return(false);
    }
    stream.setDevice(&f);

    unsigned long inode;
    long modtime;
    unsigned char numitems;
    CatInfo *data;
    int i;

    stream >> modtime;
    while(!stream.atEnd()){
        stream >> inode;
        stream >> numitems;
        data = new CatInfo;
        for(i=0; i < 8; ++i)
            data->catagories[i] = 0;
        if(useObseleteList){
            for(i=0, id=0; i < numitems; ++i){
                stream >> tmp;
                if(obseleteList.findIndex(tmp) == -1){
                    data->catagories[id] = tmp;
                    ++id;
                }
            }
            if(id == 0)
                delete data;
            else
                catDict.insert(inode, data);
        }
        else{
            for(i=0; i < numitems; ++i)
                stream >> data->catagories[i];
            catDict.insert(inode, data);
        }
    }
    f.close();
    qWarning("Catagory database load complete");
    return(true);
}

bool KIFThumbView::saveCatagories()
{
    qWarning("Saving catagory database");
    dirWatch->stopDirScan(currentDir);
    if(currentDir.isEmpty()){
        qWarning("Invalid path");
        return(true);
    }
    else if(!catDict.count()){ // no database, remove any files
        qWarning("No db needed");
        bool result = true;
        QString fileStr(currentDir + "/.catagories-");
        fileStr += getpwuid(getuid())->pw_name;
        if(QFile::exists(fileStr)){
            if(unlink(QFile::encodeName(fileStr)) == -1){
                // TODO: Message Box
                qWarning("Unable to remove catagory file!");
                result = false;
            }
        }
        fileStr = QString(currentDir + "/.catagory-mapping-");
        fileStr += getpwuid(getuid())->pw_name;
        if(QFile::exists(fileStr)){
            if(unlink(QFile::encodeName(fileStr)) == -1){
                // TODO: Message Box
                qWarning("Unable to remove catagory mapping file!");
                result = false;
            }
        }
        dirWatch->restartDirScan(currentDir);
        return(result);
    }
    bool usedcatagories[256];
    unsigned int i, j;
    unsigned char tmp;
    QString **catarray = kifapp()->catagoryManager()->array();

    for(i=0; i < 256; ++i)
        usedcatagories[i] = false;

    // write catagory database
    QString fileStr(currentDir + "/.catagories-");
    fileStr += getpwuid(getuid())->pw_name;
    QFile f(fileStr);
    if(!f.open(IO_WriteOnly)){
        // TODO: error dialog
        qWarning("Unable to write catagories for folder!");
        dirWatch->restartDirScan(currentDir);
        return(false);
    }
    QDataStream stream;
    stream.setDevice(&f);

    QIntDictIterator<CatInfo> it(catDict);
    unsigned char *data;
    unsigned long inode;
    long modtime = 0;

    stream << modtime;
    for(; it.current(); ++it){
        inode = (unsigned long)it.currentKey();
        stream << inode;
        data = it.current()->catagories;
        for(i=0; i < 8 && data[i] != 0; ++i)
            ;
        tmp = i;
        stream << tmp;
        qWarning("Catagories: %d", tmp);
        for(i=0; i < 8 && data[i] != 0; ++i){
            stream << data[i];
            j = data[i];
            qWarning("Writing catagory: %d\n", data[i]);
            usedcatagories[j] = true;
        }
    }
    f.close();


    qWarning("Writing catagory mapping");
    // write catagory mapping
    fileStr = currentDir + "/.catagory-mapping-";
    fileStr += getpwuid(getuid())->pw_name;
    f.setName(fileStr);
    if(!f.open(IO_WriteOnly)){
        // TODO: error dialog
        qWarning("Unable to write catagory mapping for folder!");
        catError = true;
        dirWatch->restartDirScan(currentDir);
        return(false);
    }
    stream.setDevice(&f);
    for(i=0; i < 256; ++i){
        if(usedcatagories[i]){
            stream << *catarray[i];
            tmp = i;
            qWarning("Writing catagory: %s, index: %d",
                     catarray[i]->latin1(), tmp);
            stream << tmp;
        }
    }
    f.close();
    qWarning("Catagory database saved");
    dirWatch->restartDirScan(currentDir);
    return(true);
}

void KIFThumbView::addCatagory(KIFThumbViewItem *item, int id)
{
    CatInfo *info;
    unsigned long inode = (unsigned long)item->statusInfo()->st_ino;
    int i;

    info = catDict.find(inode);
    if(!info){
        qWarning("Adding new catagory %d for %s",
                 id, item->text().latin1());
        info = new CatInfo;
        info->catagories[0] = id;
        for(i=1; i < 8; ++i)
            info->catagories[i] = 0;
        catDict.insert(inode, info);
    }
    else{
        unsigned char *data = info->catagories;
        for(i=0; i < 8 && data[i] != 0 && data[i] != id; ++i)
            ;
        if(i == 8){
            // TODO: messagebox
            qWarning("Maximum allowed catagories!");
            return;
        }
        else if(data[i] == id){
            qWarning("Id already set!");
            return;
        }
        else{
            data[i] = id;
        }
    }
}

void KIFThumbView::removeCatagory(KIFThumbViewItem *item, int id)
{
    qWarning("In removeCatagory");
    CatInfo *info;
    unsigned long inode = (unsigned long)item->statusInfo()->st_ino;
    int i;

    info = catDict.find(inode);
    if(!info){
        // TODO messagebox
        qWarning("Tried to remove catagory that wasn't set");
        return;
    }
    unsigned char *data = info->catagories;
    for(i=0; i < 8 && data[i] != id; ++i)
        ;
    if(i < 8 && data[i] == id){
        // justify left
        while(i < 7){
            data[i] = data[i+1];
            ++i;
        }
        data[7] = 0;
    }
    else
        qWarning("Id's are mismatched: id: %d, i: %d, data[i]: %d!",
                 id, i, data[i]);
    if(data[0] == 0){
        qWarning("No more catagories, removing entry");
        catDict.remove(inode);
    }
    qWarning("Leaving removeCatagory");
}

QStringList KIFThumbView::itemCatagories(KIFThumbViewItem *item)
{
    QStringList list;
    if(!item)
        return(list);
    CatInfo *info = catDict.find((unsigned long)item->statusInfo()->st_ino);
    QString **catarray = kifapp()->catagoryManager()->array();
    int i;

    if(info){
        for(i=0; i < 8 && info->catagories[i]; ++i){
            int j = info->catagories[i];
            if(catarray[j] != 0)
                list.append(*catarray[j]);
            else
                qWarning("Invalid catagory index %d, value: %d", i, j);
        }
    }
    list.sort();
    return(list);
}

void KIFThumbView::slotThumbJobResult(KIO::Job *job)
{
    if(job == thumbJob){
        qWarning("KIO job finished");
        thumbJob = NULL;
        emit updateProgress(100);
        kioTotalIcons = kioCurrentIcon = 0; // shouldn't matter
        stopProcessing = true;
        emit enableStopButton(false);
        dirWatch->restartDirScan(currentDir);
        inGenerateThumbs = false;
    }
}

void KIFThumbView::slotThumbJobProgress(KIO::Job *, unsigned long)
{
    ;
}

void KIFThumbView::slotThumbJobPreview(const KFileItem *item,
                                       const QPixmap &preview)
{
    int percent;
    ++kioCurrentIcon;
    percent = (int)(((float)kioCurrentIcon/kioTotalIcons)*100);
    if(percent == 100)
        percent = 99; // Don't do 100% until done
    emit updateProgress(percent);
    QString previewItemStr(item->url().fileName());
    qWarning("Got preview for %s", previewItemStr.latin1());
    KIFThumbViewItem *i = (KIFThumbViewItem*)findItem(previewItemStr,
                                                      Qt::ExactMatch);
    if(i)
        i->setPreview(preview);
}

void KIFThumbView::slotThumbJobFailed(const KFileItem *item)
{
    int percent;
    qWarning("Could not make preview for %s",
             item->url().fileName().latin1());
    ++kioCurrentIcon;
    percent = (int)(((float)kioCurrentIcon/kioTotalIcons)*100);
    emit updateProgress(percent);
}

void KIFThumbView::convertToPixieThumbs()
{
    KIFThumbViewItem *current = (KIFThumbViewItem *)firstItem();
    KIFThumbViewItem *last = (KIFThumbViewItem *)lastItem();

    if(!hasKonqThumbnails){
        KMessageBox::sorry(this, i18n("No Konqueror previews in this folder!"),
                           i18n("Thumbnail Error"));
        return;
    }

    dirWatch->stopDirScan(currentDir);

    QString thumbStr;
    switch(currentSize){
    case KIFThumbView::Huge:
        thumbStr = "xxl/";
        break;
    case KIFThumbView::Large:
        thumbStr = "large/";
        break;
    case KIFThumbView::Medium:
        thumbStr = "med/";
        break;
    case KIFThumbView::Small:
    default:
        thumbStr = "small/";
        break;
    }
    struct stat fileInfo;
    bool canSavePixieThumb = true;
    if(stat(QFile::encodeName(currentDir+"/.pics"), &fileInfo) == -1){
        if(mkdir(QFile::encodeName(currentDir+"/.pics"), 0777) == -1){
            canSavePixieThumb = false;
        }
    }
    if(canSavePixieThumb &&
       stat(QFile::encodeName(currentDir+"/.pics/"+thumbStr), &fileInfo) == -1){
        if(mkdir(QFile::encodeName(currentDir+"/.pics/"+thumbStr), 0777) == -1){
            canSavePixieThumb = false;
        }
    }
    if(access(QFile::encodeName(currentDir+"/.pics/"+thumbStr), W_OK) != 0)
        canSavePixieThumb = false;

    if(!canSavePixieThumb){
        KMessageBox::sorry(this, i18n("Cannot save PixiePlus previews for this folder!"),
                           i18n("Thumbnail Error"));
        dirWatch->restartDirScan(currentDir);
        return;
    }

    emit setStatusBarText(i18n("Converting thumbnails..."));
    kifapp()->processEvents();
    QFileInfo fi;
    while(current){
        if(QFile::exists(konqThumbPath + current->text())){
            fi.setFile(konqThumbPath + current->text());
            KURL url("file:"+fi.absFilePath());
            QString mimeStr =  KMimeType::findByURL(url, true, true)->name();
            // move only images and text
            if(mimeStr.left(6) == "image/" || (mimeStr.left(5) == "text/" &&
                                               mimeStr != "text/html")){

                if(QFile::exists(pixieThumbPath + current->text())){
                    qWarning("%s has both thumbnails", current->text().latin1());
                    if(QFileInfo(konqThumbPath+current->text()).lastModified() >
                       QFileInfo(pixieThumbPath+current->text()).lastModified())
                        KIFFileTransfer::move(konqThumbPath + current->text(),
                                              pixieThumbPath, false);

                }
                else
                    KIFFileTransfer::move(konqThumbPath + current->text(),
                                          pixieThumbPath, false);
            }
            else
                qWarning("Ignoring non image or text file %s",
                         current->text().latin1());
        }
        if(current == last)
            current = NULL;
        else
            current = (KIFThumbViewItem *)current->nextItem();

    }
    dirWatch->restartDirScan(currentDir);
    emit setStatusBarText(i18n("Done converting thumbnails"));
}

void KIFThumbView::convertToKonqThumbs()
{
    KIFThumbViewItem *current = (KIFThumbViewItem *)firstItem();
    KIFThumbViewItem *last = (KIFThumbViewItem *)lastItem();

    QString thumbStr;
    switch(currentSize){
    case KIFThumbView::Huge:
        thumbStr = "xxl/";
        break;
    case KIFThumbView::Large:
        thumbStr = "large/";
        break;
    case KIFThumbView::Medium:
        thumbStr = "med/";
        break;
    case KIFThumbView::Small:
    default:
        thumbStr = "small/";
        break;
    }

    QString pixiePath = currentDir+"/.pics/"+thumbStr;
    if(!QFile::exists(pixiePath)){
        KMessageBox::sorry(this, i18n("No PixiePlus previews in this folder!"),
                           i18n("Thumbnail Error"));
        return;
    }

    dirWatch->stopDirScan(currentDir);

    // test or create konq thumbnail dir
    bool canSaveKonqThumb = true;
    struct stat fileInfo;
    KURL url;
    url.setPath(QDir::cleanDirPath(currentDir));
    KMD5 md5(QFile::encodeName(url.url()));
    QCString hash = md5.hexDigest();
    QString testKonqPath = KGlobal::dirs()->localkdedir() +
        "/share/thumbnails/";
    if(stat(QFile::encodeName(testKonqPath), &fileInfo) == -1){
        if(mkdir(QFile::encodeName(testKonqPath), 0777) == -1){
            canSaveKonqThumb = false;
        }
    }
    testKonqPath += QString::fromLatin1(hash.data(), 4) + "/";
    if(canSaveKonqThumb &&
       stat(QFile::encodeName(testKonqPath), &fileInfo) == -1){
        if(mkdir(QFile::encodeName(testKonqPath), 0777) == -1){
            canSaveKonqThumb = false;
        }
    }
    testKonqPath += QString::fromLatin1(hash.data()+4, 4) + "/";
    if(canSaveKonqThumb &&
       stat(QFile::encodeName(testKonqPath), &fileInfo) == -1){
        if(mkdir(QFile::encodeName(testKonqPath), 0777) == -1){
            canSaveKonqThumb = false;
        }
    }
    testKonqPath += QString::fromLatin1(hash.data()+8) + "/";
    if(canSaveKonqThumb &&
       stat(QFile::encodeName(testKonqPath), &fileInfo) == -1){
        if(mkdir(QFile::encodeName(testKonqPath), 0777) == -1){
            canSaveKonqThumb = false;
        }
    }
    testKonqPath += thumbStr;
    if(canSaveKonqThumb &&
       stat(QFile::encodeName(testKonqPath), &fileInfo) == -1){
        if(mkdir(QFile::encodeName(testKonqPath), 0777) == -1){
            canSaveKonqThumb = false;
        }
    }
    if(access(QFile::encodeName(testKonqPath), W_OK) != 0)
        canSaveKonqThumb = false;
    if(!canSaveKonqThumb){
        KMessageBox::sorry(this, i18n("Cannot save Konqueror previews for this folder!"),
                           i18n("Thumbnail Error"));
        dirWatch->restartDirScan(currentDir);
        return;
    }

    // move them
    emit setStatusBarText(i18n("Converting thumbnails..."));
    kifapp()->processEvents();
    while(current){
        if(QFile::exists(pixiePath + current->text())){
            if(QFile::exists(testKonqPath + current->text())){
                qWarning("%s has both thumbnails", current->text().latin1());
                if(QFileInfo(pixiePath+current->text()).lastModified() >
                   QFileInfo(testKonqPath+current->text()).lastModified())
                    KIFFileTransfer::move(pixiePath + current->text(),
                                          testKonqPath, false);

            }
            else
                KIFFileTransfer::move(pixiePath + current->text(),
                                      testKonqPath, false);
        }
        if(current == last)
            current = NULL;
        else
            current = (KIFThumbViewItem *)current->nextItem();
    }
    dirWatch->restartDirScan(currentDir);
    emit setStatusBarText(i18n("Done converting thumbnails"));

}

void KIFThumbViewItem::dropped(QDropEvent *ev,
                               const QValueList<CustomIconDragItem> &)
{
    if(!dropEnabled())
        return;

    QStringList fileList;
    if(!QUriDrag::decodeLocalFiles(ev, fileList)){
        qWarning("Pixie: Can't decode drop.");
        return;
    }

    if(!fileList.count())
        return;
    
    int op;
    QPopupMenu opPopup;
    opPopup.insertItem(i18n("&Copy Here"), 1);
    opPopup.insertItem(i18n("&Move Here"), 2);
    opPopup.insertItem(i18n("&Link Here"), 3);

    QPoint pos = iconView()->contentsToViewport(ev->pos());
    op = opPopup.exec(iconView()->viewport()->mapToGlobal(pos));
    switch(op){
    case 1:
        ev->setAction(QDropEvent::Copy);
        break;
    case 2:
        ev->setAction(QDropEvent::Move);
        break;
    case 3:
        ev->setAction(QDropEvent::Link);
        break;
    default:
        return;
    }

    QString destStr(((KIFThumbView *)iconView())->currentPath());
    if(parentDir())
        destStr += "/../";
    else
        destStr += "/" + text() + "/";

    // TESTING!
    KIFFileTransfer::transferFiles(fileList, destStr, ev->action());
}

bool KIFThumbViewItem::acceptDrop(const QMimeSource *e) const
{
    return(dropEnabled() && QUriDrag::canDecode(e));

}



