#ifndef __THUMB_ICONVIEW_H
#define __THUMB_ICONVIEW_H

#include "customiconview.h"
#include <qtooltip.h>
#include <qstrlist.h>
#include <qdir.h>
#include <qintdict.h>
#include <kpixmap.h>
#include <kdirwatch.h>
#include <kio/job.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

class KFileItem;

class KIFThumbViewItem;
class KIFThumbView;
class UIManager;

bool isImageExtension(const char *);

typedef struct __catinfo{
    unsigned char catagories[8];
}CatInfo;

class KIFToolTip : public QToolTip
{
public:
    KIFToolTip(QWidget *parent, KIFThumbView *iconView)
        : QToolTip(parent){view = iconView;}
    void maybeTip(const QPoint &pos);
private:
    KIFThumbView *view;
};

class KIFThumbView : public CustomIconView
{
    friend class KIFThumbViewItem;
    friend class CustomIconViewItem;
    friend class KIFBatchEffect;
    Q_OBJECT
public:
    enum IconSize{Small=0, Medium, Large, Huge};
    KIFThumbView(UIManager *manager, QWidget *parent=0, const char *name=0);
    ~KIFThumbView();
    // dupFirst valid only on size sort mode!
    void update(const QString &dir, IconSize iSize, int sortType,
                bool createThumbs = false, bool dupFirst = false,
                bool autoThumbnails = false, bool catagoriesFirst = false,
                const QString &nameFilter = QString::null,
                bool updateHistory = true);
    const QString currentPath() {return(currentDir);}
    void setIconOnly(bool en);
    void setTileColor(const QColor &c){frameColor = c; framePixmap.resize(0,0); updateTiles();}
    void setTilePixmap(const QPixmap &pix){framePixmap = pix; updateTiles();}
    void setTileEnabled(bool on){useFrame = on;}
    void setAutoPreview(bool on){autoPreview = on;}
    QStrList *history(){return(&historyList);}
    void addCatagory(KIFThumbViewItem *i, int id);
    void removeCatagory(KIFThumbViewItem *i, int id);
    int oldest(){return(oldestFile);}
    int newest(){return(newestFile);}
    KDirWatch* watch(){return(dirWatch);}
    QStringList itemCatagories(KIFThumbViewItem *i);
    QString calcKonqThumbPath(const QString &path, int iconSize);
    void convertToKonqThumbs();
    void convertToPixieThumbs();
    void resizeContents(int w, int h);
public slots:
    void slotStopClicked();
    void slotUpdate();
    void slotGenerateThumbnails();
    void slotSelectAll(){selectAll(true);}
    void slotUnselectAll(){clearSelection();}
    void slotCopy();
    void slotPaste();
signals:
    void updateProgress(int val);
    void updateMe();
    void enableStopButton(bool on);
    void setStatusBarText(const QString &text);
    void editImage();
    // back, forward, and up dir signals
    void enableBackDir(bool on);
    void enableForwardDir(bool on);
    void enableUpDir(bool on);
protected slots:
    void slotIconDrop(QDropEvent *ev, const QValueList<CustomIconDragItem> &);
    void slotDirChanged(const QString &path);
    void slotItemRenamed(CustomIconViewItem *item, const QString &str);
    void slotRightButton(CustomIconViewItem *item, const QPoint &pos);

    void slotThumbJobResult(KIO::Job *job);
    void slotThumbJobProgress(KIO::Job *job, unsigned long percent);
    void slotThumbJobPreview(const KFileItem *item, const QPixmap &preview);
    void slotThumbJobFailed(const KFileItem *item);
protected:
    void drawContents(QPainter *p,  int x, int y, int w, int h);
    void viewportPaintEvent(QPaintEvent *ev);
    void insertInGrid(CustomIconViewItem *i);
    void insertItem(CustomIconViewItem *i, CustomIconViewItem *after);
    void paintEvent(QPaintEvent *ev){if(!inDirLoad)CustomIconView::paintEvent(ev);}
    void showEvent(QShowEvent *ev);
    void updateTiles();
    void keyPressEvent(QKeyEvent *ev);
    void iconViewRightButton(const QPoint &pos);
    //virtual void drawBackground(QPainter *p, const QRect &r);
    QDragObject* dragObject();
    void drawFill(KPixmap &pix);

    bool loadCatagories();
    bool saveCatagories();

    bool iconOnly, stopProcessing;
    QString currentDir;
    QBrush wallBrush;
    bool persistant, useFrame;
    IconSize currentSize;
    KDirWatch *dirWatch;
    QColor frameColor;
    QPixmap framePixmap;
    bool inUpdate;
    bool inDirLoad;
    bool disableContentsResize;
    QStrList historyList;
    // preview stuff
    bool autoPreview;
    bool hasKonqThumbnails;
    QString konqThumbPath, pixieThumbPath;
    // fills
    KPixmap *hugeFill, *largeFill, *medFill, *smallFill;
    KPixmap *currentFill, *tmpPixmap;
    // image icons
    QPixmap *hugeImage, *largeImage, *medImage, *smallImage;
    // tooltip
    bool useToolTips;
    KIFToolTip *toolTips;

    UIManager *mgr;

    QFontMetrics *fm;
    int textHeight;
    QSize cachedResize;

    time_t oldestFile, newestFile;

    // catagories
    QIntDict<CatInfo>catDict;
    bool catError;
    bool supportCatagories;

    GC clearGC;

    KIO::Job *thumbJob;
    int kioTotalIcons, kioCurrentIcon;
    bool inGenerateThumbs;
};

class KIFThumbViewItem : public CustomIconViewItem
{
    friend class KIFThumbView;
public:
    KIFThumbViewItem(CustomIconView *parent, const QString &textStr,
                     struct stat *statusBuffer);
    ~KIFThumbViewItem();
    bool parentDir(){return(false);}
    bool acceptDrop(const QMimeSource *) const;
    bool thumbCreated(){return(isThumbnailed);}
    struct stat* statusInfo(){return(status);}
    QString path();
    void setThumbnailStatus(bool created){isThumbnailed = created;}
    void updateThumbnail(const QString &path, int iconSize, bool useFrame);
    void makeThumbnail(const QString &path, int iconSize,
                       const QString &thumbPath, bool saveThumbnail);
    QPixmap* pixmap()const{return(isThumbnailed ? CustomIconViewItem::pixmap() :
                                  NULL);}
    void setPreview(const QPixmap &pix);
    void swap(KIFThumbViewItem *item);
protected:
    void blendMimetypeIcon(QImage &mainImage, QImage &iconImage);
    void paintItem(QPainter *p, const QColorGroup &g);
    void dropped(QDropEvent *ev, const QValueList<CustomIconDragItem> &);
    void calcTextWrapping();
    bool isThumbnailed;
    bool makeThumbs;
    struct stat *status;
    QString tmpStr;
    bool textDirty;

};




#endif

