#include "uimanager.h"
#include "sizedialog.h"
#include "slidedialog.h"
#include "slideshow.h"
#include "screengrab.h"
#include "dirtree.h"
#include "hotlistbox.h"
#include "printdialog.h"
#include "imagepreview.h"
#include "catagorydlg.h"
#include "../qimageutils/qimageutil.h"
#include "htmlexportwizard.h"
#include "htmlexport.h"
#include "imagecompare.h"
#include "../stub/imagebuffer.h"
#include "../stub/ifapp.h"
#include "../stub/dndlistbox.h"
#include "../stub/imagelistitem.h"
#include "../stub/scaledtoplevel.h"
#include "../stub/scrolltoplevel.h"
#include "../stub/fullscreen.h"
#include "../stub/filelist.h"
#include "../stub/about.h"
#include <qlabel.h>
#include <qtabwidget.h>
#include <qinputdialog.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qtoolbutton.h>
#include <qstatusbar.h>
#include <qtextstream.h>
#include <qclipboard.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>

#include <klocale.h>
//#include <kimageeffect.h>
#include <kprogress.h>
#include <kprinter.h>
#include <ktoolbar.h>
#include <kglobal.h>
#include <kstddirs.h>
#include <kiconloader.h>
#include <kmenubar.h>
#include <kfiledialog.h>
#include <krun.h>
#include <kmimetype.h>
#include <kmessagebox.h>
#include <kstdaccel.h>
#include <kstatusbar.h>
#include <kcolordialog.h>
//#include <kwinmodule.h>

#include <ltdl.h>

#include <sys/stat.h>
#include <sys/types.h>

// plugin stub
extern "C"{
    void init(KIFFileList *list)
    {
        UIManager *w = new UIManager(list);
        //kapp->setMainWidget(w);
        w->show();
    }
}


UIManager::UIManager(KIFFileList *list, const char *)
    : KDockMainWindow()
{
    connect(kifapp(), SIGNAL(hideAllWindows()), this,
            SLOT(slotHideAll()));
    connect(kifapp(), SIGNAL(showAllWindows()), this,
            SLOT(slotShowAll()));


    loadAllSettings();
    pathStr = QDir::current().absPath();

    KStatusBar *sBar = statusBar(); // Statusbar
    progress = new KProgress(100, sBar);
    progress->setMinimumWidth(80);
    sBar->addWidget(progress);
    statusLbl = new QLabel(sBar);
    statusLbl->setText(i18n("Pixie (C)QPL Daniel M. Duley"));
    sBar->addWidget(statusLbl, true);

    KDockWidget *mainDock = createDockWidget(i18n("Thumbnails"), QPixmap());

    mainWidget = new KIFThumbView(this, mainDock);
    mainWidget->setTileColor(QColor(136, 174, 255));
    connect(mainWidget, SIGNAL(clicked(CustomIconViewItem *)), this,
            SLOT(slotThumbClicked(CustomIconViewItem *)));
    connect(mainWidget, SIGNAL(doubleClicked(CustomIconViewItem *)), this,
            SLOT(slotThumbDblClicked(CustomIconViewItem *)));
    connect(mainWidget, SIGNAL(returnPressed(CustomIconViewItem *)), this,
            SLOT(slotThumbDblClicked(CustomIconViewItem *)));
    connect(mainWidget, SIGNAL(enableStopButton(bool)), this,
            SLOT(slotEnableStopBtn(bool)));
    connect(mainWidget, SIGNAL(updateProgress(int)), progress,
            SLOT(setValue(int)));
    connect(mainWidget, SIGNAL(setStatusBarText(const QString &)), this,
            SLOT(slotSetStatus(const QString &)));
    connect(mainWidget, SIGNAL(updateMe()), this,
            SLOT(slotUpdateView()));
    connect(mainWidget, SIGNAL(editImage()), this,
            SLOT(slotLoadEditor()));
    mainWidget->setIconOnly(imageOnly);
    mainDock->setWidget(mainWidget);
    setView(mainDock);
    setMainDockWidget(mainDock);
    KDockWidget *treeDock = createDockWidget(i18n("Files"), QPixmap(), NULL,
                                             i18n("Directory tree/loaded files"));
    treeTabWidget = new QTabWidget(treeDock);

    dirTree = new PixieDirTree(true, treeTabWidget);
    //treeTabWidget->addTab(dirTree, SmallIcon("folder.png"), i18n("Folders"));
    treeTabWidget->addTab(dirTree, i18n("Folders"));
    connect(dirTree, SIGNAL(clicked(QListViewItem *)), this,
            SLOT(slotTreeClicked(QListViewItem *)));
    connect(dirTree, SIGNAL(expanded(QListViewItem *)), this,
            SLOT(slotTreeOpened(QListViewItem *)));

    fileList = new KIFFileList(treeTabWidget);
    if(list)
       *fileList = *list;
    connect(fileList, SIGNAL(editImage(const QString &)), this,
            SLOT(slotLoadEditorFile(const QString &)));
    connect(fileList, SIGNAL(save()), this,
            SLOT(slotSaveFileList()));

    treeTabWidget->addTab(fileList, i18n("Files"));
    connect(fileList, SIGNAL(selected(QListBoxItem *)), this,
            SLOT(slotURLSelected(QListBoxItem *)));

    KIFHotListBox *hotList = new KIFHotListBox(this, treeTabWidget);
    connect(hotList, SIGNAL(hotListClicked(const QString &)), this,
            SLOT(slotHotListClicked(const QString &)));
    treeTabWidget->addTab(hotList, i18n("HotList"));

    KConfig *config = KGlobal::config();
    config->setGroup("UISettings");
    treeTabWidget->setCurrentPage(config->readNumEntry("TreeTab", 0));
    treeDock->setWidget(treeTabWidget);
    treeDock->manualDock(mainDock, KDockWidget::DockLeft, 40);


    imageBuffer = new KIFImage(this);
    KDockWidget *imageDock = createDockWidget(i18n("Image preview"), QPixmap(), NULL,
                                              i18n("Image Preview"));
    imagePreview = new KIFImagePreview(imageDock);

    scaledTopLevel = NULL;
    scrollTopLevel = NULL;
    fullScreenTopLevel = NULL;
    if(viewMode == 0){
        scaledTopLevel = new KIFScaledTopLevel(imageBuffer, NULL);
        connect(scaledTopLevel, SIGNAL(selectionChanged(const QRect &)),
                imageBuffer, SLOT(slotSelectionChanged(const QRect &)));
        connect(scaledTopLevel, SIGNAL(prevList()), fileList,
                SLOT(slotPrevInList()));
        connect(scaledTopLevel, SIGNAL(nextList()), fileList,
                SLOT(slotNextInList()));
        connect(scaledTopLevel, SIGNAL(prevImage()), this,
                SLOT(slotPrevInDir()));
        connect(scaledTopLevel, SIGNAL(nextImage()), this,
                SLOT(slotNextInDir()));
    }
    else if(viewMode == 1){
        scrollTopLevel = new KIFScrollTopLevel(imageBuffer, NULL);
        connect(scrollTopLevel, SIGNAL(prevList()), fileList,
                SLOT(slotPrevInList()));
        connect(scrollTopLevel, SIGNAL(nextList()), fileList,
                SLOT(slotNextInList()));
        connect(scrollTopLevel, SIGNAL(prevImage()), this,
                SLOT(slotPrevInDir()));
        connect(scrollTopLevel, SIGNAL(nextImage()), this,
                SLOT(slotNextInDir()));
    }
    else if(viewMode == 2){
        fullScreenTopLevel = new KIFFullScreen(imageBuffer, false, NULL);
        connect(fullScreenTopLevel, SIGNAL(prevList()), fileList,
                SLOT(slotPrevInList()));
        connect(fullScreenTopLevel, SIGNAL(nextList()), fileList,
                SLOT(slotNextInList()));
        connect(fullScreenTopLevel, SIGNAL(prevImage()), this,
                SLOT(slotPrevInDir()));
        connect(fullScreenTopLevel, SIGNAL(nextImage()), this,
                SLOT(slotNextInDir()));
    }
    else if(viewMode == 3){
        fullScreenTopLevel = new KIFFullScreen(imageBuffer, true, NULL);
        connect(fullScreenTopLevel, SIGNAL(prevList()), fileList,
                SLOT(slotPrevInList()));
        connect(fullScreenTopLevel, SIGNAL(nextList()), fileList,
                SLOT(slotNextInList()));
        connect(fullScreenTopLevel, SIGNAL(prevImage()), this,
                SLOT(slotPrevInDir()));
        connect(fullScreenTopLevel, SIGNAL(nextImage()), this,
                SLOT(slotNextInDir()));
    }

    imageDock->setWidget(imagePreview);
    imageDock->manualDock(treeDock, KDockWidget::DockBottom, 40);

    // TODO - load only if no file specified
    imagePreview->slotSetFile(KGlobal::dirs()->findResource("appdata",
                                                            "pixielogo.png"));
    //keys = new QAccel(this);
    createMenu(this);
    createToolbar(this);


    connect(mainWidget, SIGNAL(enableBackDir(bool)), this,
            SLOT(slotEnableBackDir(bool)));
    connect(mainWidget, SIGNAL(enableForwardDir(bool)), this,
            SLOT(slotEnableForwardDir(bool)));
    connect(mainWidget, SIGNAL(enableUpDir(bool)), this,
            SLOT(slotEnableUpDir(bool)));


    resize(525,350);
    readDockConfig(KGlobal::config(), "DockSetup");
    show();
    kapp->processEvents();
    mainWidget->update(pathStr, iconSize, sorting, false, dupFirst,
                       autoPreview, catagoriesFirst);
    dirTree->setPath(pathStr);
    setCaption(i18n("Pixie Image/Photo Editing Browser"));

    backDirBtn->setEnabled(false);
    forwardDirBtn->setEnabled(false);


    // TODO - do commandline enable
    //editMnu->setItemEnabled(editImageMnuId, false);

}

UIManager::~UIManager()
{
    writeDockConfig(KGlobal::config(), "DockSetup");

    KConfig *config = KGlobal::config();
    config->setGroup("UISettings");
    config->writeEntry("TreeTab", treeTabWidget->currentPageIndex());
    config->sync();
    if(scaledTopLevel)
        delete scaledTopLevel;
    else if(scrollTopLevel)
        delete scrollTopLevel;
    else if(fullScreenTopLevel)
        delete fullScreenTopLevel;

}

bool UIManager::imageShown()
{
    if((scaledTopLevel && scaledTopLevel->isVisible()) ||
       (scrollTopLevel && scrollTopLevel->isVisible()) ||
       (fullScreenTopLevel && fullScreenTopLevel->isVisible()))
        return(true);
    return(false);
}

void UIManager::loadAllSettings()
{
    KConfig *config  = KGlobal::config();
    config->setGroup("UISettings");
    int sortVal = config->readNumEntry("Sorting", 0);
    if(sortVal == 6){
        sorting = QDir::Size;
        dupFirst = true;
    }
    else{
        sorting = (QDir::SortSpec)sortVal;
        dupFirst = false;
    }

    catagoriesFirst = config->readBoolEntry("CatagoryFirst", false);

    iconSize = (KIFThumbView::IconSize)config->readNumEntry("IconSize", 2);

    viewMode = config->readNumEntry("ViewMode", 2);

    imageOnly = config->readBoolEntry("ImageOnly", false);

    imagesOnTop = config->readBoolEntry("ImagesOnTop", true);

    autoPreview = config->readBoolEntry("AutoPreview", false);

    internalImagePreview = config->readBoolEntry("InternalPreview", true);
    previewText = config->readBoolEntry("PreviewText", false);
    previewHTML = config->readBoolEntry("PreviewHTML", false);
    previewPS = config->readBoolEntry("PreviewPS", false);
    previewSound = config->readBoolEntry("PreviewSound", false);
}

void UIManager::slotTreeClicked(QListViewItem *i)
{
    if(!i)
        return;
    PixieBaseItem *item = (PixieBaseItem*)i;
    if(item->isDir()){
        PixieDirItem *dir = (PixieDirItem*)i;
        if(dir->canOpen()){
            pathStr = item->fullPath();
            pathEdit->setEditText(pathStr);
            pathEdit->addToHistory(pathStr);
            slotUpdateView();
        }
    }
    else
        imageBuffer->slotSetFile(item->fullPath());
}

void UIManager::slotTreeOpened(QListViewItem *i)
{
    /*
    PixieBaseItem *item = (PixieBaseItem*)i;
    if(item->isDir()){
        pathStr = item->fullPath();
        pathEdit->setEditText(pathStr);
        slotUpdateView();
    }*/
}

void UIManager::slotHotListClicked(const QString &path)
{
    QDir d(path);
    if(d.exists()){
        mainWidget->viewport()->erase();
        pathStr = d.absPath(); // get rid of crap in the URL
        pathEdit->setEditText(pathStr);
        pathEdit->addToHistory(pathStr);
        dirTree->setPath(pathStr);
        mainWidget->update(pathStr, iconSize, sorting, false, dupFirst,
                           autoPreview, catagoriesFirst);
    }
}

void UIManager::resetImageBuffer()
{
    imagePreview->resetFilename();
}

void UIManager::slotURLSelected(QListBoxItem *i)
{
    qWarning("URL selected: %s", ((KIFImageListItem *)i)->fileName().latin1());
    if(imagePreview->filename() == ((KIFImageListItem *)i)->fileName()){
        imageBuffer->setImage(imagePreview->currentImage());
    }
    else{
        QApplication::setOverrideCursor(kifapp()->interactiveBusyCursor1());
        imageBuffer->slotSetFile(((KIFImageListItem *)i)->fileName());
        QApplication::restoreOverrideCursor();
    }

    if(scaledTopLevel){
        if(!scaledTopLevel->isVisible())
            scaledTopLevel->show();
        scaledTopLevel->setCaption(i->text());
    }
    else if(scrollTopLevel){
        if(!scrollTopLevel->isVisible())
            scrollTopLevel->show();
        scrollTopLevel->setCaption(i->text());
    }
    else if(fullScreenTopLevel){
        if(!fullScreenTopLevel->isVisible())
            fullScreenTopLevel->show();
    }


    //fileTb->setItemEnabled(3, i->prev() != 0);
    //fileTb->setItemEnabled(4, i->next() != 0);
}

void UIManager::slotAddAndSetURL(const QString &path)
{
    if(QFile::exists(path)){
        fileList->slotAppendAndSet(path);
        QListBoxItem *i = fileList->findItem(path);
        slotURLSelected(i);
    }
    else{

    }
}

void UIManager::slotAddURL(const QString &path)
{
    if(QFile::exists(path)){
        fileList->slotAppend(path);
    }
}

// remove
void UIManager::slotSetTopItem()
{
    if(fileList->count() > 0){
        fileList->setCurrentItem(0);
        imageBuffer->slotSetFile(((KIFImageListItem *)fileList->item(0))->
                                  fileName());
    }
}

// remove
int UIManager::findURLIndex(const QString &url)
{
    unsigned int i;
    for(i=0; i < fileList->count(); ++i){
        if(fileList->text(i) == url)
            return(i);
    }
    return(-1);
}

void UIManager::slotOpen()
{
    qWarning("In slotOpen");
    QString str = KFileDialog::getOpenFileName(QDir::currentDirPath(),
                                               kifImagePattern(false), this,
                                               i18n("Open image"));
    if(!str.isEmpty())
        slotAddAndSetURL(str);
}

#ifndef CONVERT_FUNCS_DEFINED
#define CONVERT_FUNCS_DEFINED
inline double convertMMToInch(double mm){return(mm*0.039370147);}
inline double convertInchToMM(double inch){return(inch*25.399956);}
#endif
#define US_MEASURE 0
#define METRIC_MEASURE 1

void UIManager::slotPrint()
{
    KPrinter printer;
    if(printer.setup(this)){
        // get page layout options
        bool dither, zoomSmall, scaleLarge;
        int metricType;
        KConfig *config = KGlobal::config();
        QSize defMargin = printer.margins();
        QPaintDeviceMetrics metrics(&printer);

        config->setGroup("Print");
        dither = config->readBoolEntry("Dither", false);
        scaleLarge = config->readBoolEntry("Scale", true);
        zoomSmall = config->readBoolEntry("Zoom", false);
        metricType = config->readBoolEntry("MetricType", 0);

        // do the margins
        double l, r, t, b;
        double defH, defV;
        defH = ((double)defMargin.width())/metrics.logicalDpiX();
        defV = ((double)defMargin.height())/metrics.logicalDpiY();
        if(metricType == METRIC_MEASURE){
            defH = convertInchToMM(defH);
            defV = convertInchToMM(defV);
        }
        l = config->readDoubleNumEntry("LMargin", defH);
        r = config->readDoubleNumEntry("RMargin", defH);
        t = config->readDoubleNumEntry("TMargin", defV);
        b = config->readDoubleNumEntry("BMargin", defV);
        if(metricType == METRIC_MEASURE){
            l = convertMMToInch(l);
            r = convertMMToInch(r);
            t = convertMMToInch(t);
            b = convertMMToInch(b);
        }
        l = l*metrics.logicalDpiX();
        r = l*metrics.logicalDpiX();
        t = l*metrics.logicalDpiY();
        b = l*metrics.logicalDpiY();
        int x, y, w, h;
        x = (int)l;
        y = (int)t;
        w = (int)(metrics.width()-l-r);
        h = (int)(metrics.height()-t-b);

        // okay, margins done - x, y, w, h are the coords in dots
        QPixmap pix;
        pix.convertFromImage(*imageBuffer->image());
        if(scaleLarge && (pix.width() > w || pix.height() > h)){
            int newW = pix.width();
            int newH = pix.height();
            while(newW > w || newH > h){
                if(newW > w){
                    float percent = ((float)w)/newW;
                    newW = (int)(newW*percent);
                    newH = (int)(newH*percent);
                    qWarning("Scaling width");
                }
                if(newH > h){
                    float percent = ((float)h)/newH;
                    newW = (int)(newW*percent);
                    newH = (int)(newH*percent);
                    qWarning("Scaling height");
                }
            }
            QImage img = pix.convertToImage();
            img = img.smoothScale(newW, newH);
            // might as well dither here while we have a QImage
            if(dither && (img.numColors() == 0)){
                qWarning("Dithering");
                img = img.convertDepth(8, AvoidDither | DiffuseDither);
            }
            pix.convertFromImage(img);
        }
        else if(dither && pix.depth() > 8 || pix.depth() == 0){
            QImage img = pix.convertToImage();
            img = img.convertDepth(8, AvoidDither | DiffuseDither);
            pix.convertFromImage(img);
        }
        QPainter p(&printer);
        p.drawPixmap(x, y, pix);
    }
}

void UIManager::slotCopy()
{
    mainWidget->slotCopy();
}

void UIManager::slotPaste()
{
    mainWidget->slotPaste();
}

void UIManager::slotPageLayout()
{
    KPrinter p;
    KIFPrintDialog dlg(p, this);
    dlg.exec();
}


void UIManager::slotSlideShow()
{
    KIFSlideDialog dlg(this);
    if(dlg.exec() == QDialog::Accepted){
        QStringList list;
        if(dlg.useFileList()){
            if(fileList->count() == 0){
                KMessageBox::sorry(this,
                             i18n("The FileList is empty!\nTry the folder option."),
                             i18n("Pixie Slideshow Error"));
                return;
            }
            QListBoxItem *item = fileList->firstItem();
            while(item){
                list.append(item->text());
                item = item->next();
            }
        }
        else{
             CustomIconViewItem *i = mainWidget->firstItem();
             QString fn;
             bool isImage;
             while (i){
                 fn = pathStr + "/" + i->text();
                 KURL url("file:" + fn);
                 isImage =  KMimeType::findByURL(url, true, true)->name().left(6) ==
                     "image/";
                 if(isImage)
                     list.append(fn);
                 i = i->nextItem();
             }
             if(list.isEmpty()){
               KMessageBox::sorry(this,
                             i18n("No images in the current folder!"),
                             i18n("Pixie SlideShow Error"));
                return;
             }
        }
        (void)new KIFSlideShow(list, dlg.delay(), dlg.useMaxpect(),
                               dlg.loop());
    }
}

void UIManager::slotSlideShowTimer()
{
    /*
    unsigned int i = fileList->currentItem();
    if(i == fileList->count()-1)
        i = 0;
    else
        ++i;
    fileList->setCurrentItem(i);
    imageBuffer->slotSetFile(((KIFImageListItem *)fileList->
                              item(i))->fileName());
    fileTb->setItemEnabled(3, i->prev() != 0);
    fileTb->setItemEnabled(4, i->next() != 0);*/
}

void UIManager::slotReload()
{
    slotUpdateView();
}

void UIManager::slotGrab()
{
    (void)new KIFScreenGrab(this);
}

/*
void UIManager::slotDropFinished()
{
    slotURLSelected(fileList->item(fileList->count()-1));
    fileList->setCurrentItem(fileList->count()-1);
    }*/

void UIManager::slotLoadEditor()
{
    CustomIconViewItem *i = mainWidget->currentItem();
    if(!i){
        KMessageBox::sorry(this,
                             i18n("No image selected!"),
                             i18n("Pixie Editor Error"));
        return;
    }
    KURL url("file:" + pathStr + "/" + i->text());
    if(KMimeType::findByURL(url, true, true)->name().left(6) != "image/"){
        KMessageBox::sorry(this,
                           i18n("Selected file is not an image!"),
                           i18n("Pixie Editor Error"));
        return;
    }

    QString fileStr = pathStr + "/" + i->text();
    kifapp()->loadEditor(fileStr);
}

void UIManager::slotLoadEditorFile(const QString &str)
{
    kifapp()->loadEditor(str);
}

void UIManager::slotOpenFileList()
{
    QString fn = KFileDialog::getOpenFileName(pathStr, "*.flst", this,
                                              i18n("Open Pixie FileList"));
    if(fn.isNull())
        return;
    QFile f(fn);
    if(f.open(IO_ReadOnly)){
        fileList->clear();
        QTextStream t(&f);
        if(t.readLine() != "PixieList"){
            KMessageBox::sorry(this,  i18n("Invalid FileList file!"),
                               i18n("Pixie FileList Error!"));
            f.close();
            return;
        }
        while (!t.eof()){
            fileList->slotAppend(t.readLine());
        }
        f.close();
        fileListName = fn;
        fileMnu->setItemEnabled(100, true);
    }
    else
        KMessageBox::sorry(this,  i18n("Unable to open file!"),
                           i18n("Pixie FileList Error!"));

}

void UIManager::slotSaveFileList()
{
    if(fileListName.isNull())
        slotSaveFileListAs();
    else{
        QFile f(fileListName);
        if(f.open(IO_WriteOnly)){
            QTextStream t(&f);
            t << "PixieList\n";
            unsigned int i;
            for(i=0; i < fileList->count(); ++i)
                t << fileList->text(i) << '\n';
            f.close();
        }
        else
            KMessageBox::sorry(this, i18n("Unable to open file!"),
                               i18n("Pixie FileList Error!"));
    }
}

void UIManager::slotSaveFileListAs()
{
    QString fn = KFileDialog::getSaveFileName(pathStr, "*.flst", this,
                                              i18n("Open Pixie FileList"));
    if(fn.isNull())
        return;
    QFile f(fn);
    if(f.open(IO_WriteOnly)){
        QTextStream t(&f);
        t << "PixieList\n";
        unsigned int i;
        for(i=0; i < fileList->count(); ++i)
            t << fileList->text(i) << '\n';
        f.close();
        fileListName = fn;
        fileMnu->setItemEnabled(100, true);
    }
    else
        KMessageBox::information(this,  i18n("Unable to open file!"),
                                 i18n("Pixie FileList Error!"));
}


void UIManager::createMenu(KMainWindow *parent)
{
    KMenuBar *mbar = parent->menuBar();

    fileMnu = new QPopupMenu(parent);
    fileMnu->insertItem(BarIcon("window_new", 16), i18n("New Window"), this, SLOT(slotNewWindow()));
    fileMnu->insertSeparator();
    fileMnu->insertItem(i18n("Take a screenshot..."), this, SLOT(slotGrab()));
    fileMnu->insertItem(i18n("Slide Show..."), this, SLOT(slotSlideShow()));
    fileMnu->insertItem(i18n("Make Web Image Gallery..."), this, SLOT(slotExportHTML()));
    fileMnu->insertItem(i18n("Find Similiar Images..."), this, SLOT(slotCompareImages()));
    fileMnu->insertSeparator();
    fileMnu->insertItem(i18n("Create/Edit file catagories..."), this,
                        SLOT(slotEditCatagories()));
    fileMnu->insertSeparator();
    fileMnu->insertItem(i18n("Convert PixiePlus previews to Konqueror format"),
                        this, SLOT(slotConvertThumbsToKonq()));
    fileMnu->insertItem(i18n("Convert Konqueror previews to PixiePlus format"),
                        this, SLOT(slotConvertThumbsToPixie()));
    fileMnu->insertSeparator();
    fileMnu->insertItem(BarIcon("fileopen", 16), i18n("Open..."), this, SLOT(slotOpen()));
    fileMnu->insertItem(BarIcon("folder", 16), i18n("Create a new folder..."), this, SLOT(slotNewFolder()));
    fileMnu->insertItem(i18n("Page Layout..."), this, SLOT(slotPageLayout()));
    fileMnu->insertItem(BarIcon("fileprint", 16), i18n("Print"), this, SLOT(slotPrint()));
    fileMnu->insertSeparator();
    fileMnu->insertItem(BarIcon("fileopen", 16), i18n("Open FileList..."), this, SLOT(slotOpenFileList()));
    fileMnu->insertItem(BarIcon("filesave", 16), i18n("Save FileList"), this, SLOT(slotSaveFileList()),
                        0, 100);
    fileMnu->setItemEnabled(100, false);
    fileMnu->insertItem(BarIcon("filesave", 16), i18n("Save FileList As..."), this, SLOT(slotSaveFileListAs()));
    fileMnu->insertSeparator();
    fileMnu->insertItem(BarIcon("exit", 16), i18n("&Quit"), kifapp(), SLOT(quit()));
    mbar->insertItem(i18n("&File"), fileMnu);


    editMnu = new QPopupMenu(parent);
    editImageMnuId = editMnu->insertItem(i18n("Edit Image..."), this,
                                         SLOT(slotLoadEditor()));
    editMnu->insertSeparator();
    editMnu->insertItem(BarIcon("editcopy", 16), i18n("Copy"), this, SLOT(slotCopy()));
    editMnu->insertItem(BarIcon("editpaste", 16), i18n("Paste"), this, SLOT(slotPaste()));
    editMnu->insertItem(i18n("Select All"), mainWidget,
                        SLOT(slotSelectAll()));
    editMnu->insertItem(i18n("Unselect All"), mainWidget,
                        SLOT(slotUnselectAll()));
    editMnu->insertSeparator();
    mbar->insertItem(i18n("&Edit"), editMnu);

    viewMenu = new QPopupMenu(parent);

    sortMenu = new QPopupMenu(viewMenu);
    sortMenu->insertItem(i18n("Ascending by name"), 0);
    sortMenu->insertItem(i18n("Descending by name"), 1);
    sortMenu->insertItem(i18n("Ascending by date"), 2);
    sortMenu->insertItem(i18n("Descending by date"), 3);
    sortMenu->insertItem(i18n("Ascending by size"), 4);
    sortMenu->insertItem(i18n("Descending by size"), 5);
    sortMenu->insertItem(i18n("Ascending size/same sizes first"), 6);
    sortMenu->setItemChecked(sorting, true);
    sortMenu->insertSeparator();
    sortMenu->insertItem(i18n("Images on top"), 201);
    sortMenu->insertItem(i18n("Catagorized items on top"), 200);

    sortMenu->setItemChecked(200, catagoriesFirst);
    sortMenu->setItemChecked(201, imagesOnTop);
    connect(sortMenu, SIGNAL(activated(int)), this, SLOT(slotSortMenu(int)));
    viewMenu->insertItem(i18n("Sorting"), sortMenu, 1);


    sizeMenu = new QPopupMenu(viewMenu);
    sizeMenu->insertItem(i18n("Small"), (int)KIFThumbView::Small);
    sizeMenu->insertItem(i18n("Medium"), (int)KIFThumbView::Medium);
    sizeMenu->insertItem(i18n("Large"), (int)KIFThumbView::Large);
    sizeMenu->insertItem(i18n("Huge"), (int)KIFThumbView::Huge);
    sizeMenu->setItemChecked(iconSize, true);
    connect(sizeMenu, SIGNAL(activated(int)), this, SLOT(slotSizeMenu(int)));
    viewMenu->insertItem(i18n("Thumbnail Size"), sizeMenu, 2);

    previewMenu = new QPopupMenu(viewMenu);
    previewMenu->insertItem(i18n("Fast image and text thumbnails (Pixie only)"), 0);
    previewMenu->insertItem(i18n("Preview text files"), 1);
    previewMenu->insertItem(i18n("Preview HTML files"), 2);
    previewMenu->insertItem(i18n("Preview PS/PDF files"), 3);
    previewMenu->insertItem(i18n("Preview sound files"), 4);
    previewMenu->setItemChecked(0, internalImagePreview);
    previewMenu->setItemChecked(1, previewText);
    previewMenu->setItemChecked(2, previewHTML);
    previewMenu->setItemChecked(3, previewPS);
    previewMenu->setItemChecked(4, previewSound);

    connect(previewMenu, SIGNAL(activated(int)), this, SLOT(slotPreviewMenu(int)));
    viewMenu->insertItem(i18n("Previews"), previewMenu, 3);

    imageMenu = new QPopupMenu(viewMenu);
    imageMenu->insertItem(i18n("New window, scaled to desktop size"), 0);
    imageMenu->insertItem(i18n("New scrollable window"), 1);
    imageMenu->insertItem(i18n("Fullscreen"), 2);
    imageMenu->insertItem(i18n("Fullscreen/Maxpect"), 3);
    imageMenu->insertSeparator();
    imageMenu->insertItem(i18n("Fullscreen background color..."), 5);

    imageMenu->setItemChecked(viewMode, true);
    connect(imageMenu, SIGNAL(activated(int)), this, SLOT(slotImageMenu(int)));
    viewMenu->insertItem(i18n("View images as..."), imageMenu, 4);

    viewMenu->insertItem(i18n("Images and Folders only"), this,
                         SLOT(slotImageOnly()), 0, 5);
    viewMenu->setItemChecked(5, imageOnly);

    mbar->insertItem(i18n("&View"), viewMenu);

    QPopupMenu *windowMenu = dockHideShowMenu();
    mbar->insertItem(i18n("&Windows"), windowMenu);

    QPopupMenu *helpMenu = new QPopupMenu(parent);
    helpMenu->insertItem(SmallIcon("pixie", 16), i18n("About Pixie..."),
                         this, SLOT(slotAbout()));
    helpMenu->insertItem(BarIcon("contents", 16), i18n("User's Guide"),
                         this, SLOT(slotDocumentation()));


    mbar->insertSeparator();
    mbar->insertItem(i18n("&Help"), helpMenu);
}                   

void UIManager::createToolbar(KMainWindow *parent)
{
    KToolBar *tBar = parent->toolBar();

    tBar->insertButton("fileopen", 0, SIGNAL(clicked()), this,
                       SLOT(slotOpen()), true,
                       KStdAccel::description(KStdAccel::Open));
    tBar->insertButton("fileprint", 1, SIGNAL(clicked()), this,
                       SLOT(slotPrint()), true,
                       KStdAccel::description(KStdAccel::Print));
    tBar->insertButton("editcopy", 2, SIGNAL(clicked()), this,
                       SLOT(slotCopy()), true,
                       KStdAccel::description(KStdAccel::Copy));
    tBar->insertButton("editpaste", 3, SIGNAL(clicked()), this,
                       SLOT(slotPaste()), true,
                       KStdAccel::description(KStdAccel::Paste));
    tBar->insertButton("window_new", 4, SIGNAL(clicked()), this,
                       SLOT(slotNewWindow()), true, i18n("New Window"));
    fileTb = tBar;

    // Thumbnail viewer stuff
    tBar = parent->toolBar("thumbToolBar");
    tBar->insertButton("up", 0, SIGNAL(clicked()), this,
                       SLOT(slotUpDir()), true,
                       KStdAccel::description(KStdAccel::Up));
    tBar->insertButton("back", 1, SIGNAL(clicked()), this,
                       SLOT(slotBackDir()), true,
                       KStdAccel::description(KStdAccel::Back));
    tBar->insertButton("forward", 2, SIGNAL(clicked()), this,
                       SLOT(slotForwardDir()), true,
                       KStdAccel::description(KStdAccel::Forward));
    tBar->insertButton("gohome", 3, SIGNAL(clicked()), this,
                       SLOT(slotHomeClicked()), true,
                       KStdAccel::description(KStdAccel::Home));
    tBar->insertButton("reload", 4, SIGNAL(clicked()), this,
                       SLOT(slotReload()), true,
                       KStdAccel::description(KStdAccel::Reload));
    tBar->insertButton("thumb", 5, SIGNAL(clicked()), this,
                       SLOT(slotUpdateClicked()), true,
                       i18n("Create Thumbnails"));
    tBar->insertButton("stop", 6, SIGNAL(clicked()), this,
                       SLOT(slotStopClicked()), true, i18n("Stop"));
    upDirBtn = tBar->getButton(0);
    backDirBtn = tBar->getButton(1);
    forwardDirBtn = tBar->getButton(2);
    stopBtn = tBar->getButton(6);

    tBar->addSeparator();
    pathEdit = new KHistoryCombo(tBar);
    pathEdit->setDuplicatesEnabled(false);
    fileComp = new KURLCompletion();
    fileComp->setDir("/");
    pathEdit->setCompletionObject(fileComp);
    pathEdit->setEditText(QDir::current().absPath());
    pathEdit->addToHistory(QDir::current().absPath());
    connect(pathEdit, SIGNAL(returnPressed(const QString &)), this,
            SLOT(slotPathEdit(const QString &)));
    connect(pathEdit, SIGNAL(completion(const QString &)), this,
            SLOT(slotPathTextChanged(const QString &)));
    pathEdit->setDuplicatesEnabled(false);
    pathEdit->setMinimumWidth(200);

    //tBar->insertWidget(6, pathEdit->sizeHint().width(), pathEdit);
    //tBar->setItemAutoSized(6);
    tBar->setStretchableWidget(pathEdit);
    //tBar->setHorizontalStretchable(true);

    thumbTb = tBar;
}

void UIManager::slotPathEdit(const QString &str)
{
    mainWidget->viewport()->erase();
    QFileInfo fi(str);
    fileComp->addItem(str);
    pathEdit->addToHistory(str);
    if(fi.exists() && fi.isDir()){
        pathStr = str;
        dirTree->setPath(pathStr);
        mainWidget->update(pathStr, iconSize, sorting, false, dupFirst,
                           autoPreview, catagoriesFirst);
    }
    else{
        pathStr = fi.dirPath();
        dirTree->setPath(pathStr);
        mainWidget->update(pathStr, iconSize, sorting, false, dupFirst,
                           autoPreview, catagoriesFirst, fi.fileName());
    }
}

void UIManager::slotPathTextChanged(const QString &str)
{
    if(str.find('*') != -1)
        return;
    QFileInfo fi(str);
    QDir d(fi.dirPath(true));
    d.setFilter(QDir::Dirs);
    QString fileName = fi.fileName();
    fileName += "*";
    d.setNameFilter(fileName);
    const QStringList list = d.entryList();
    fileComp->clear();
    /*
    for (QStringList::ConstIterator it = list.begin(); it != list.end(); ++it)
    {
        qWarning("Adding path", (fi.dirPath()).latin1());
        fileComp->addItem(fi.dirPath()+(*it));
    } */
    fileComp->setItems(list);
    /*
    if(d.entryList().count() == 1){
        if(fi.dirPath() == "/")
            pathEdit->setEditText(fi.dirPath()+(*list.begin()) + "/");
        else
            pathEdit->setEditText(fi.dirPath()+"/"+(*list.begin()) + "/");
    }*/
}


void UIManager::slotUpDir()
{
    mainWidget->viewport()->erase();
    QDir dir(pathStr);
    if(dir.cdUp()){
        pathStr = dir.absPath();
        pathEdit->setEditText(pathStr);
        pathEdit->addToHistory(pathStr);
        dirTree->setPath(pathStr);
        mainWidget->update(pathStr, iconSize, sorting, false, dupFirst,
                           autoPreview, catagoriesFirst);
    }
}

void UIManager::slotBackDir()
{
    QStrList *list = mainWidget->history();
    const char *item = list->next();
    if(item == NULL){
        qWarning("History error: next item is null!");
        return;
    }
    mainWidget->viewport()->erase();
    QFileInfo fi(item);
    pathEdit->setEditText(item);
    if(fi.exists() && fi.isDir()){
        pathStr = item;
        dirTree->setPath(pathStr);
        mainWidget->update(pathStr, iconSize, sorting, false, dupFirst,
                           autoPreview, catagoriesFirst, QString::null, false);
    }
    else{
        pathStr = fi.dirPath();
        dirTree->setPath(pathStr);
        mainWidget->update(pathStr, iconSize, sorting, false, dupFirst,
                           autoPreview, catagoriesFirst, fi.fileName(), false);
    }
    if(item == list->getLast())
        slotEnableBackDir(false);
    slotEnableForwardDir(true);
}

void UIManager::slotForwardDir()
{
    QStrList *list = mainWidget->history();
    const char *item = list->prev();
    if(item == NULL){
        qWarning("History error: previous item is null!");
        return;
    }
    mainWidget->viewport()->erase();
    QFileInfo fi(item);
    pathEdit->setEditText(item);
    if(fi.exists() && fi.isDir()){
        pathStr = item;
        dirTree->setPath(pathStr);
        mainWidget->update(pathStr, iconSize, sorting, false, dupFirst,
                           autoPreview, catagoriesFirst, QString::null, false);
    }
    else{
        pathStr = fi.dirPath();
        dirTree->setPath(pathStr);
        mainWidget->update(pathStr, iconSize, sorting, false, dupFirst,
                           autoPreview, catagoriesFirst, fi.fileName(), false);
    }
    if(item == list->getFirst())
        slotEnableForwardDir(false);
    slotEnableBackDir(true);
}

void UIManager::slotEnableUpDir(bool en)
{
    upDirBtn->setEnabled(en);
}

void UIManager::slotEnableBackDir(bool en)
{
    backDirBtn->setEnabled(en);
}

void UIManager::slotEnableForwardDir(bool en)
{
    forwardDirBtn->setEnabled(en);
}


void UIManager::slotHomeClicked()
{
    mainWidget->viewport()->erase();
    pathStr = QDir::homeDirPath();
    pathEdit->setEditText(pathStr);
    pathEdit->addToHistory(pathStr);
    dirTree->setPath(pathStr);
    mainWidget->update(pathStr, iconSize, sorting, false, dupFirst,
                       autoPreview, catagoriesFirst);

}

void UIManager::slotUpdateView()
{
    mainWidget->viewport()->erase();
    mainWidget->update(pathStr, iconSize, sorting, false, dupFirst,
                      autoPreview, catagoriesFirst);
}

void UIManager::slotUpdateClicked()
{
    qWarning("In slotUpdateClicked, previewText: %d", previewText);
    mainWidget->slotGenerateThumbnails();
}

void UIManager::slotStopClicked()
{
    mainWidget->slotStopClicked();
}

void UIManager::slotThumbClicked(CustomIconViewItem *i)
{
    if(!imagePreview->isVisible()){
        slotSetStatus(i18n("Double click to view fullsize"));
        return;
    }
    if(!i)
        return;
    KIFThumbViewItem *clickedItem = (KIFThumbViewItem *)i;
    QString iStr = clickedItem->text();
    if(clickedItem->parentDir())
        iStr = "../";
    QFileInfo fi(pathStr + "/" + iStr);
    if(fi.absFilePath() == imagePreview->filename())
        return;
    if(fi.isDir()){
        slotSetStatus(i18n("Double click to enter folder"));
    }
    else{
        if(isImageExtension(fi.extension(false).ascii())){
            QApplication::setOverrideCursor(kifapp()->interactiveBusyCursor1());
            imagePreview->slotSetFile(fi.absFilePath());
            QApplication::restoreOverrideCursor();
            setCaption(fi.fileName());
            slotSetStatus(i18n("Double click to view fullsize"));
        }
        else
            slotSetStatus(i18n("Double click to run"));
    }
}

void UIManager::slotThumbDblClicked(CustomIconViewItem *i)
{
    if(!i)
        return;
    KIFThumbViewItem *clickedItem = (KIFThumbViewItem *)i;
    QString iStr = clickedItem->text();
    if(clickedItem->parentDir())
        iStr = "../";
    QFileInfo fi(pathStr + "/" + iStr);
    if(fi.isDir()){
        mainWidget->viewport()->erase();
        QDir d(pathStr + "/" + iStr);
        pathStr = d.absPath(); // get rid of crap in the URL
        pathEdit->setEditText(pathStr);
        pathEdit->addToHistory(pathStr);
        dirTree->setPath(pathStr);
        mainWidget->update(pathStr, iconSize, sorting, false, dupFirst,
                           autoPreview, catagoriesFirst);
        slotSetStatus(i18n("Entered new folder"));
    }
    else{
        // if it's an image, display and add to filelist - else exec
        // default app
        if(isImageExtension(fi.extension(false).ascii())){
            slotAddAndSetURL(pathStr+"/"+iStr);
            slotSetStatus(i18n("Image Loaded"));
        }
        else{
            slotSetStatus(i18n("Running application for ") + iStr);
            new KRun(KURL("file:"+pathStr+"/"+iStr));
        }
    }
}

void UIManager::slotNextInDir()
{
    KIFThumbViewItem *item = (KIFThumbViewItem *)mainWidget->currentItem();
    bool isImage = false;
    while(item && !isImage){
        item = (KIFThumbViewItem *)item->nextItem();
        if(item){
            KURL url("file:"+item->path());
            isImage =  KMimeType::findByURL(url, true, true)->name().left(6) == "image/";
        }
        else{
            item = (KIFThumbViewItem *)mainWidget->firstItem();
            KURL url("file:"+item->path());
            isImage =  KMimeType::findByURL(url, true, true)->name().left(6) == "image/";
        }
    }
    if(item){
        mainWidget->setCurrentItem(item);
        slotAddAndSetURL(pathStr+"/"+item->text());
        slotSetStatus(i18n("Image Loaded"));
    }
}

void UIManager::slotPrevInDir()
{
    KIFThumbViewItem *item = (KIFThumbViewItem *)mainWidget->currentItem();
    bool isImage = false;
    while(item && !isImage){
        item = (KIFThumbViewItem *)item->prevItem();
        if(item){
            KURL url("file:"+item->path());
            isImage =  KMimeType::findByURL(url, true, true)->name().left(6) == "image/";
        }
        else{
            item = (KIFThumbViewItem *)mainWidget->lastItem();
            KURL url("file:"+item->path());
            isImage =  KMimeType::findByURL(url, true, true)->name().left(6) == "image/";
        }
    }
    if(item){
        mainWidget->setCurrentItem(item);
        slotAddAndSetURL(pathStr+"/"+item->text());
        slotSetStatus(i18n("Image Loaded"));
    }
}


void UIManager::slotEnableStopBtn(bool on)
{
    stopBtn->setEnabled(on);
}

void UIManager::slotSetStatus(const QString &str)
{
    statusLbl->setText(str);
}

void UIManager::slotSortMenu(int i)
{
    mainWidget->viewport()->erase();
    KConfig *config = KGlobal::config();
    QString oldGrp = config->group();
    config->setGroup("UISettings");
    if(i == 200){
        catagoriesFirst = !catagoriesFirst;
        sortMenu->setItemChecked(200, catagoriesFirst);
        config->writeEntry("CatagoryFirst", catagoriesFirst);
    }
    else if(i == 201){
        imagesOnTop = !imagesOnTop;
        sortMenu->setItemChecked(201, imagesOnTop);
        config->writeEntry("ImagesOnTop", imagesOnTop);
    }
    else{
        int tmp;
        for(tmp=0; tmp <= 6; ++tmp)
            sortMenu->setItemChecked(tmp, false);
        sortMenu->setItemChecked(i, true);

        if(i == 6)
            dupFirst = true;
        else
            dupFirst = false;
        sorting = i;

        config->writeEntry("Sorting", sorting);
    }
    config->sync();
    config->setGroup(oldGrp);
    mainWidget->update(pathStr, iconSize, sorting, false, dupFirst,
                      autoPreview, catagoriesFirst);
}

void UIManager::slotPreviewMenu(int id)
{
    mainWidget->viewport()->erase();
    KConfig *config = KGlobal::config();
    QString oldGrp = config->group();
    config->setGroup("UISettings");

    if(id == 0){
        internalImagePreview = !internalImagePreview;
        previewMenu->setItemChecked(0, internalImagePreview);
        config->writeEntry("InternalPreview", internalImagePreview);
    }
    else if(id == 1){
        previewText = !previewText;
        qWarning("in previewText: %d", previewText);
        previewMenu->setItemChecked(1, previewText);
        config->writeEntry("PreviewText", previewText);
    }
    else if(id == 2){
        previewHTML = !previewHTML;
        previewMenu->setItemChecked(2, previewHTML);
        config->writeEntry("PreviewHTML", previewHTML);
    }
    else if(id == 3){
        previewPS = !previewPS;
        previewMenu->setItemChecked(3, previewPS);
        config->writeEntry("PreviewPS", previewPS);
    }
    else if(id == 4){
        previewSound = !previewSound;
        previewMenu->setItemChecked(4, previewSound);
        config->writeEntry("PreviewSound", previewSound);
    }
    config->sync();
    config->setGroup(oldGrp);

    mainWidget->update(pathStr, iconSize, sorting, false, dupFirst,
                       autoPreview, catagoriesFirst);
}


void UIManager::slotSizeMenu(int i)
{
    mainWidget->viewport()->erase();
    KConfig *config = KGlobal::config();
    config->setGroup("UISettings");
    sizeMenu->setItemChecked((int)KIFThumbView::Small, false);
    sizeMenu->setItemChecked((int)KIFThumbView::Medium, false);
    sizeMenu->setItemChecked((int)KIFThumbView::Large, false);
    sizeMenu->setItemChecked((int)KIFThumbView::Huge, false);
    sizeMenu->setItemChecked(i, true);
    config->writeEntry("IconSize", i);
    iconSize = (KIFThumbView::IconSize)i;
    config->sync();
    mainWidget->update(pathStr, iconSize, sorting, false, dupFirst,
                       autoPreview, catagoriesFirst);
}

void UIManager::slotImageOnly()
{
    mainWidget->viewport()->erase();
    KConfig *config = KGlobal::config();
 
    imageOnly = !viewMenu->isItemChecked(5);
    viewMenu->setItemChecked(5, imageOnly);
    config->setGroup("UISettings");
    config->writeEntry("ImageOnly", imageOnly);
    config->sync();
    mainWidget->setIconOnly(imageOnly);
    mainWidget->update(pathStr, iconSize, sorting, false, dupFirst,
                       autoPreview, catagoriesFirst);
}

void UIManager::slotImageMenu(int id)
{
    KConfig *config = KGlobal::config();
    config->setGroup("UISettings");

    if(id == 5){
        QColor c(config->readColorEntry("FullScreenColor", &Qt::white));
        if(KColorDialog::getColor(c, this) == QDialog::Accepted){
            if(fullScreenTopLevel)
                fullScreenTopLevel->setColor(c);
            config->writeEntry("FullScreenColor", c);
            config->sync();
        }
        return;
    }
    viewMode = id;
    imageMenu->setItemChecked(0, false);
    imageMenu->setItemChecked(1, false);
    imageMenu->setItemChecked(2, false);
    imageMenu->setItemChecked(3, false);
    imageMenu->setItemChecked(id, true);
    if(scaledTopLevel){
        delete scaledTopLevel;
        scaledTopLevel = NULL;
    }
    if(scrollTopLevel){
        delete scrollTopLevel;
        scrollTopLevel = NULL;
    }
    if(fullScreenTopLevel){
        delete fullScreenTopLevel;
        fullScreenTopLevel = NULL;
    }

    if(viewMode == 0){
        scaledTopLevel = new KIFScaledTopLevel(imageBuffer, NULL);
        connect(scaledTopLevel, SIGNAL(prevList()), fileList,
                SLOT(slotPrevInList()));
        connect(scaledTopLevel, SIGNAL(nextList()), fileList,
                SLOT(slotNextInList()));

        connect(scaledTopLevel, SIGNAL(prevImage()), this,
                SLOT(slotPrevInDir()));
        connect(scaledTopLevel, SIGNAL(nextImage()), this,
                SLOT(slotNextInDir()));
    }
    else if(viewMode == 1){
        scrollTopLevel = new KIFScrollTopLevel(imageBuffer, NULL);
        connect(scrollTopLevel, SIGNAL(prevList()), fileList,
                SLOT(slotPrevInList()));
        connect(scrollTopLevel, SIGNAL(nextList()), fileList,
                SLOT(slotNextInList()));

        connect(scrollTopLevel, SIGNAL(prevImage()), this,
                SLOT(slotPrevInDir()));
        connect(scrollTopLevel, SIGNAL(nextImage()), this,
                SLOT(slotNextInDir()));
    }
    else if(viewMode == 2){
        fullScreenTopLevel = new KIFFullScreen(imageBuffer, false, NULL);
        connect(fullScreenTopLevel, SIGNAL(prevList()), fileList,
                SLOT(slotPrevInList()));
        connect(fullScreenTopLevel, SIGNAL(nextList()), fileList,
                SLOT(slotNextInList()));

        connect(fullScreenTopLevel, SIGNAL(prevImage()), this,
                SLOT(slotPrevInDir()));
        connect(fullScreenTopLevel, SIGNAL(nextImage()), this,
                SLOT(slotNextInDir()));
    }
    else if(viewMode == 3){
        fullScreenTopLevel = new KIFFullScreen(imageBuffer, true, NULL);
        connect(fullScreenTopLevel, SIGNAL(prevList()), fileList,
                SLOT(slotPrevInList()));
        connect(fullScreenTopLevel, SIGNAL(nextList()), fileList,
                SLOT(slotNextInList()));

        connect(fullScreenTopLevel, SIGNAL(prevImage()), this,
                SLOT(slotPrevInDir()));
        connect(fullScreenTopLevel, SIGNAL(nextImage()), this,
                SLOT(slotNextInDir()));
    }

    config->writeEntry("ViewMode", id);
    config->sync();
}

void UIManager::slotHideAll()
{
    visible = isVisible();
    if(visible)
        hide();
}

void UIManager::slotShowAll()
{
    if(visible)
        show();
}


void UIManager::slotNewWindow()
{
    kifapp()->loadBrowser(NULL);
}

void UIManager::slotAbout()
{
    KIFAboutDialog dlg;
    dlg.exec();
}

void UIManager::slotDocumentation()
{
    KURL url(KGlobal::dirs()->findResource("appdata", "doc/en/index.html"));
    new KRun(url, 0, true);
}

void UIManager::slotExportHTML()
{
    QStringList fileList;
    HTMLExportWizard wiz(fileList, this);
    if(wiz.exec() != QDialog::Rejected){
        QStringList list;
        CustomIconViewItem *i = mainWidget->firstItem();
        QString fn;
        bool isImage;
        while (i){
            if(i->text().left(4) != "thb_"){
                fn = pathStr + "/" + i->text();

                KURL url("file:" + fn);
                isImage =  KMimeType::findByURL(url, true, true)->name().left(6) ==
                    "image/";
                if(isImage)
                    list.append(fn);
            }
            i = i->nextItem();
        }
        if(list.isEmpty()){
            KMessageBox::sorry(this,
                               i18n("No images in the current folder!"),
                               i18n("Pixie HTML Gallery Error"));
            return;
        }
        HTMLExportBase exporter;
        connect(&exporter, SIGNAL(updateProgress(int)), progress,
                SLOT(setValue(int)));
        connect(&exporter, SIGNAL(setStatusBarText(const QString &)), this,
                SLOT(slotSetStatus(const QString &)));

        exporter.run(pathStr, list);
    }
}

void UIManager::slotEditCatagories()
{
    CatagoryDialog dlg(this);
    if(dlg.exec() == QDialog::Accepted)
        dlg.sync();
}

void UIManager::slotCompareImages()
{
    int iSize;
    switch(iconSize){
    case KIFThumbView::Huge:
        iSize = 120;
        break;
    case KIFThumbView::Large:
        iSize = 90;
        break;
    case KIFThumbView::Medium:
        iSize = 64;
        break;
    case KIFThumbView::Small:
        iSize = 48;
        break;
    default:
        iSize = 90;
    }
    // This is goofy >:) A second window is created, but you need to delete
    // the compare object before it's used because KIFCompare is modal.
    KIFCompare *compare = new KIFCompare(pathStr, iSize, this);
    KIFCompareView *view = compare->compareView();
    delete compare;
    if(view)
        view->makeThumbnails();
    else
        qWarning("Null compare view");
}

void UIManager::slotNewFolder()
{
    bool okay;
    QString folderStr = QInputDialog::getText(i18n("Folder Name"),
                                              i18n("Please enter a name for the new folder."),
                                              QLineEdit::Normal, QString::null,
                                              &okay,
                                              this);
    if(okay){
        if(mkdir((pathStr + "/" + folderStr).ascii(), 0777) == -1){
            KMessageBox::sorry(this, i18n("Could not create folder!"));
        }
    }
}

void UIManager::slotConvertThumbsToKonq()
{
    mainWidget->convertToKonqThumbs();
}

void UIManager::slotConvertThumbsToPixie()
{
    mainWidget->convertToPixieThumbs();
}


