#ifndef __KIF_UIMANAGER_H
#define __KIF_UIMANAGER_H

#include <qwidget.h>
#include <qlistbox.h>
#include <qpopupmenu.h>
#include <kmainwindow.h>
#include <qtoolbar.h>
#include <qlabel.h>
#include <kcombobox.h>
#include <kdockwidget.h>
#include "dirtree.h"
#include "thumbiconview.h"
#include <kurlcompletion.h>
#include <ktoolbarbutton.h>

//class KIFImageWindow;
class KIFImage;
class KIFImagePreview;
class KIFScaledTopLevel;
class KIFScrollTopLevel;
class KIFFullScreen;
class KIFDndListBox;
class QTabWidget;
class KProgress;
class KIFFileList;


class UIManager : public KDockMainWindow
{
    Q_OBJECT
    friend class KIFBatchEffect;
    friend class KIFThumbView;
    friend class KIFThumbViewItem;

public:
    UIManager(KIFFileList *list, const char *name=0);
    ~UIManager();
    QString currentPath(){return(pathStr);}
    void resetImageBuffer();
    bool isBackDirEnabled(){return(backDirBtn->isEnabled());}
    bool isForwardDirEnabled(){return(forwardDirBtn->isEnabled());}
    bool isUpDirEnabled(){return(upDirBtn->isEnabled());}
public slots:
    void slotHotListClicked(const QString &path);
    void slotAddAndSetURL(const QString &path);
    void slotAddURL(const QString &path);
    void slotURLSelected(QListBoxItem *i);
    void slotSetTopItem();
    void slotNewFolder();
    void slotOpen();
    void slotPrint();
    void slotCopy();
    void slotPaste();
    void slotReload();
    void slotUpDir();
    void slotBackDir();
    void slotForwardDir();
    void slotGrab();
    void slotSlideShow();
    void slotSlideShowTimer();
    void slotPageLayout();
    void slotExportHTML();
    void slotEditCatagories();
    void slotUpdateView();
protected slots:
    void slotTreeClicked(QListViewItem *i);
    void slotTreeOpened(QListViewItem *i);
    int findURLIndex(const QString &url);
    void slotSizeMenu(int);
    void slotSortMenu(int);
    void slotPreviewMenu(int);
    void slotImageMenu(int);
    void slotImageOnly();
    void slotNewWindow();
    // thumbnail slots
    void slotPathEdit(const QString &);
    void slotPathTextChanged(const QString &);
    void slotHomeClicked();
    void slotUpdateClicked();
    void slotStopClicked();
    void slotThumbClicked(CustomIconViewItem *i);
    void slotThumbDblClicked(CustomIconViewItem *i);
    void slotEnableStopBtn(bool en);
    void slotSetStatus(const QString &);

    void slotEnableBackDir(bool);
    void slotEnableForwardDir(bool);
    void slotEnableUpDir(bool);

    void slotOpenFileList();
    void slotSaveFileList();
    void slotSaveFileListAs();

    void slotNextInDir();
    void slotPrevInDir();

    void slotLoadEditor();
    void slotLoadEditorFile(const QString &fileName);

    void slotHideAll();
    void slotShowAll();

    void slotAbout();
    void slotDocumentation();

    void slotCompareImages();

    void slotConvertThumbsToKonq();
    void slotConvertThumbsToPixie();
protected:
    bool imageShown();
    void createMenu(KMainWindow *parent);
    void createToolbar(KMainWindow *parent);
    void loadAllSettings();

    PixieDirTree *dirTree;
    //KIFImageWindow *imageWindow;
    KIFImage *imageBuffer;
    KIFImagePreview *imagePreview;
    KIFScaledTopLevel *scaledTopLevel;
    KIFScrollTopLevel *scrollTopLevel;
    KIFFullScreen *fullScreenTopLevel;

    QTabWidget *treeTabWidget;
    KIFFileList *fileList;
    QPopupMenu *fileMnu, *editMnu;

    QString fileListName;

    // thumbnail stuff
    KIFThumbView *mainWidget;
    //KComboBox *pathEdit;
    KHistoryCombo *pathEdit;
    KURLCompletion *fileComp;
    QString pathStr;
    int sorting;
    bool dupFirst;
    bool catagoriesFirst;

    bool imageOnly, autoPreview, imagesOnTop;

    bool internalImagePreview, previewText, previewHTML, previewPS,
        previewSound;

    int viewMode;
    KIFThumbView::IconSize iconSize;
    KToolBar *thumbTb, *fileTb;
    KToolBarButton *stopBtn, *upDirBtn, *backDirBtn, *forwardDirBtn;
    KProgress *progress;
    int editImageMnuId;
    QPopupMenu *sortMenu, *sizeMenu, *previewMenu, *viewMenu, *imageMenu;
    QLabel *statusLbl;

    bool visible;
};


#endif
