#include "catagorymgr.h"
#include <qfile.h>
#include <qdir.h>
#include <qdatastream.h>
#include <qstringlist.h>


CatagoryManager::CatagoryManager(QObject *parent, const char *name)
    : QObject(parent, name)
{
    int i;
    for(i=0; i < 256; ++i)
        catarray[i] = NULL;
    numItems = 0;
    readDb();
}

CatagoryManager::~CatagoryManager()
{
    clear();
}

bool CatagoryManager::readDb()
{
    qWarning("Loading catagory database");
    clear();
    QString fileStr(QDir::homeDirPath() + "/.pixiecatagories");
    if(!QFile::exists(fileStr)){
        qWarning("No Pixie catagory database");
        return(true); // only return false on error
    }
    QFile f(fileStr);
    if(!f.open(IO_ReadOnly)){
        // TODO: error dialog
        qWarning("Unable to open catagory database.");
        return(false);
    }
    QDataStream stream;
    stream.setDevice(&f);

    unsigned int id;
    unsigned char tmp;
    while(!stream.atEnd()){
        // gcc doesn't like unsigned char array indexes
        stream >> tmp;
        id = tmp;
        catarray[id] = new QString;
        stream >> *catarray[id];
        nameList.append(*catarray[id]);
        ++numItems;
    }
    f.close();
    qWarning("Catagory database loaded");
    return(true);
}

bool CatagoryManager::writeDb()
{
    QString fileStr(QDir::homeDirPath() + "/.pixiecatagories");
    QFile f(fileStr);
    if(!f.open(IO_WriteOnly)){
        // TODO: error dialog
        qWarning("Unable to write catagory database.");
        return(false);
    }
    QDataStream stream;
    stream.setDevice(&f);
    int i;
    for(i=1; i < 256; ++i){
        if(catarray[i] != NULL){
            stream << (unsigned char)i;
            stream << *catarray[i];
        }
    }
    f.close();
    return(true);
}

void CatagoryManager::clear()
{
    if(!numItems)
        return;
    int i;
    for(i=1; i < 256; ++i){
        if(catarray[i])
            delete catarray[i];
        catarray[i] = NULL;
    }
    numItems = 0;
    nameList.clear();
}

void CatagoryManager::insertStringList(const QStringList &list)
{
    // this method doesn't really insert a new string list but checks
    // for new items and removed items in the list and handles it
    // appropriately, leaving all other items alone

    QStringList::ConstIterator it = list.begin();
    int i;
    bool hasItem;
    nameList.clear();
    while(it != list.end()){
        hasItem = false;
        for(i=1; i < 256 && !hasItem; ++i){
            if(catarray[i] != NULL){
                if((*it) == *catarray[i]){
                    hasItem = true;
                }
            }
        }
        if(!hasItem){
            qWarning("Adding new item: %s", (*it).latin1());
            for(i=1; i < 256 && catarray[i]; ++i)
                ;
            if(catarray[i] == NULL){
                catarray[i] = new QString;
                *catarray[i] = (*it);
                ++numItems;
            }
            else{
                // TODO Messagebox
                qWarning("Unable to add item %s!", (*it).latin1());
            }
        }
        ++it;
    }
    // check for removed items
    for(i=1; i < 256; ++i){
        if(catarray[i] != NULL){
            if(list.findIndex(*catarray[i]) == -1){
                qWarning("Removing item %s", catarray[i]->latin1());
                delete catarray[i];
                catarray[i] = NULL;
                --numItems;
            }
        }
    }
    nameList = list;
}

bool CatagoryManager::sync()
{
    if(writeDb()){
        emit changed();
        return(true);
    }
    else
        return(false);
}

int CatagoryManager::id(const QString &catagory)
{
    int i;
    for(i=1; i < 256; ++i){
        if(catarray[i] != NULL){
            if(*catarray[i] == catagory)
                return(i);
        }
    }
    return(0);
}













