#include "ifapp.h"
#include "scaledtoplevel.h"
#include "scrolltoplevel.h"
#include "fullscreen.h"
#include "magickhandler.h"
#include "gif.h"
#include "imagelistitem.h"
#include "catagorymgr.h"
#include <qdir.h>
#include <qpixmap.h>
#include <qimage.h>
#include <kglobal.h>
#include <kconfig.h>
#include <kstddirs.h>
#include <kconfig.h>
#include <klocale.h>
#include <kimageio.h>
#include "filelist.h"
#include <kmessagebox.h>


extern "C"{
    #include <magick.h>
};

static const char * busy1_xpm[] = {
"32 32 3 1",
" 	c None",
".	c #000000",
"+	c #FFFFFF",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"            .......             ",
"          ..+++++++..           ",
"         .+++++++++++.          ",
"        .+++++++++++++.         ",
"       ...+++++++++++...        ",
"      .....+++++++++.....       ",
"      ......+++++++......       ",
"     ........+++++........      ",
"     .........+++.........      ",
"     ..........+..........      ",
"     .....................      ",
"     ..........+..........      ",
"     .........+++.........      ",
"     ........+++++........      ",
"      ......+++++++......       ",
"      .....+++++++++.....       ",
"       ...+++++++++++...        ",
"        .+++++++++++++.         ",
"         .+++++++++++.          ",
"          ..+++++++..           ",
"            .......             ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                "};

static const char * busy2_xpm[] = {
"32 32 3 1",
" 	c None",
".	c #000000",
"+	c #FFFFFF",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"            .......             ",
"          ...........           ",
"         .............          ",
"        ...............         ",
"       .+.............+.        ",
"      .+++...........+++.       ",
"      .++++.........++++.       ",
"     .++++++.......++++++.      ",
"     .+++++++.....+++++++.      ",
"     .++++++++...++++++++.      ",
"     .+++++++++.+++++++++.      ",
"     .++++++++...++++++++.      ",
"     .+++++++.....+++++++.      ",
"     .++++++.......++++++.      ",
"      .++++.........++++.       ",
"      .+++...........+++.       ",
"       .+.............+.        ",
"        ...............         ",
"         .............          ",
"          ...........           ",
"            .......             ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                "};

static const char * busy_interactive1_xpm[] = {
"32 32 3 1",
" 	c None",
".	c #000000",
"+	c #FFFFFF",
"...........                     ",
".++++++++.                      ",
".+++++++.                       ",
".++++++.                        ",
".+++++.  .....                  ",
".++++. .........                ",
".+++. ...........               ",
".++. .............              ",
".+. .++.........++.             ",
"..  .+++.......+++.             ",
".  .+++++.....+++++.            ",
"   .++++++...++++++.            ",
"   .+++++++.+++++++.            ",
"   .++++++...++++++.            ",
"   .+++++.....+++++.            ",
"    .+++.......+++.             ",
"    .++.........++.             ",
"     .............              ",
"      ...........               ",
"       .........                ",
"         .....                  ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                "};

static const char * busy_interactive2_xpm[] = {
"32 32 3 1",
" 	c None",
".	c #000000",
"+	c #FFFFFF",
"...........                     ",
".++++++++.                      ",
".+++++++.                       ",
".++++++.                        ",
".+++++.  .....                  ",
".++++. ..+++++..                ",
".+++. .+++++++++.               ",
".++. ..+++++++++..              ",
".+. ....+++++++....             ",
"..  .....+++++.....             ",
".  .......+++.......            ",
"   ........+........            ",
"   .................            ",
"   ........+........            ",
"   .......+++.......            ",
"    .....+++++.....             ",
"    ....+++++++....             ",
"     ..+++++++++..              ",
"      .+++++++++.               ",
"       ..+++++..                ",
"         .....                  ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                ",
"                                "};

static KIFApplication* appPtr = NULL;

KIFApplication* kifapp(){return(appPtr);}

KIFApplication::KIFApplication(int argc, char **argv)
    : KApplication(argc, argv)
{
    list = NULL;
    catMgr = NULL;
    dlregistered = false;

    connect(this, SIGNAL(lastWindowClosed()), SLOT(quit()));
    appPtr = this;
    QPixmap busy1Pix(busy1_xpm);
    QPixmap busy2Pix(busy2_xpm);
    QPixmap interactiveBusy1Pix(busy_interactive1_xpm);
    QPixmap interactiveBusy2Pix(busy_interactive2_xpm);

    busy1Cursor = QCursor(busy1Pix);
    busy2Cursor = QCursor(busy2Pix);
    interactiveBusy1Cursor = QCursor(interactiveBusy1Pix, 1, 1);
    interactiveBusy2Cursor = QCursor(interactiveBusy2Pix, 1, 1);
    inBusy = false;

    browserAfterList = true;

    // Register built-in image formats
    // TODO: TGA
    QImageIO::defineIOHandler("XCF",
                              "gimp xcf",
                              0,
                              qimageio_readXCF,
                              NULL);
    QImageIO::defineIOHandler("GIF",
                              "^GIF[0-9][0-9][a-z]",
                              0,
                              NULL,
                              write_gif_image);
    QImageIO::defineIOHandler("PCX",
                              "^\12[\2\5]",
                              0,
                              qimageio_magickreadimage,
                              qimageio_magickwriteimage);
    /*
    QImageIO::defineIOHandler("XWD",
                              "^.\1\1..\1\1",
                              0,
                              qimageio_magickreadimage,
                              qimageio_magickwriteimage);*/
    /*QImageIO::defineIOHandler("PCD",
                              "^*PCD*",
                              0,
                              qimageio_magickreadimage,
                              NULL); */
    KImageIO::registerFormats();
    browserhandle = editorhandle = NULL;

    mimeImageDict.setAutoDelete(true);
}

KIFApplication::~KIFApplication()
{
    if(catMgr)
        delete catMgr;
/*
    if(browserhandle)
        lt_dlclose(browserhandle);
    if(editorhandle)
        lt_dlclose(editorhandle);
        */
}

CatagoryManager* KIFApplication::catagoryManager()
{
    return(catMgr);
}

void KIFApplication::startBusyCursor()
{
    if(!inBusy){
        inBusy = true;
        cursorState = false;
        setOverrideCursor(busy1Cursor);
    }
}

void KIFApplication::incBusyCursor()
{
    if(inBusy){
        cursorState = !cursorState;
        if(cursorState)
            setOverrideCursor(busy2Cursor, true);
        else
            setOverrideCursor(busy1Cursor, true);
    }
}

void KIFApplication::stopBusyCursor()
{
    inBusy = false;
    restoreOverrideCursor();
}

void KIFApplication::startInteractiveBusyCursor()
{
    if(!inBusy){
        inBusy = true;
        cursorState = false;
        setOverrideCursor(interactiveBusy1Cursor);
    }
}

void KIFApplication::incInteractiveBusyCursor()
{
    if(inBusy){
        cursorState = !cursorState;
        if(cursorState)
            setOverrideCursor(interactiveBusy2Cursor, true);
        else
            setOverrideCursor(interactiveBusy1Cursor, true);
    }
}

void KIFApplication::stopInteractiveBusyCursor()
{
    inBusy = false;
    restoreOverrideCursor();
}

void KIFApplication::hideWindows()
{
    emit hideAllWindows();
}

void KIFApplication::showWindows()
{
    emit showAllWindows();
}

void KIFApplication::runInitialFileList(KIFFileList *fileList)
{
    list = fileList;
    imageBuffer = new KIFImage(this);
    connect(fileList, SIGNAL(selected(QListBoxItem *)), this,
            SLOT(slotFileListSelected(QListBoxItem *)));

    KConfig *config = KGlobal::config();
    config->setGroup("UISettings");
    int viewMode = config->readNumEntry("ViewMode", 0);
    if(viewMode == 0){
        KIFScaledTopLevel *scaledTopLevel = new KIFScaledTopLevel(imageBuffer, NULL);
        connect(scaledTopLevel, SIGNAL(selectionChanged(const QRect &)),
                imageBuffer, SLOT(slotSelectionChanged(const QRect &)));
        connect(scaledTopLevel, SIGNAL(prevList()), fileList,
                SLOT(slotPrevInList()));
        connect(scaledTopLevel, SIGNAL(nextList()), fileList,
                SLOT(slotNextInList()));

        connect(scaledTopLevel, SIGNAL(finished()), this,
                SLOT(slotFileListFinished()));
        viewer = scaledTopLevel;
    }
    else if(viewMode == 1){
        KIFScrollTopLevel *scrollTopLevel = new KIFScrollTopLevel(imageBuffer, NULL);
        connect(scrollTopLevel, SIGNAL(prevList()), fileList,
                SLOT(slotPrevInList()));
        connect(scrollTopLevel, SIGNAL(nextList()), fileList,
                SLOT(slotNextInList()));

        connect(scrollTopLevel, SIGNAL(finished()), this,
                SLOT(slotFileListFinished()));
        viewer = scrollTopLevel;
    }
    else if(viewMode == 2){
        KIFFullScreen *fullScreenTopLevel = new KIFFullScreen(imageBuffer, false, NULL,
                                                              NULL, false);
        connect(fullScreenTopLevel, SIGNAL(prevList()), fileList,
                SLOT(slotPrevInList()));
        connect(fullScreenTopLevel, SIGNAL(nextList()), fileList,
                SLOT(slotNextInList()));
        connect(fullScreenTopLevel, SIGNAL(finished()), this,
                SLOT(slotFileListFinished()));
        viewer = fullScreenTopLevel;
    }
    else if(viewMode == 3){
        KIFFullScreen *fullScreenTopLevel = new KIFFullScreen(imageBuffer, true, NULL,
                                                             NULL, false);
        connect(fullScreenTopLevel, SIGNAL(prevList()), fileList,
                SLOT(slotPrevInList()));
        connect(fullScreenTopLevel, SIGNAL(nextList()), fileList,
                SLOT(slotNextInList()));
        connect(fullScreenTopLevel, SIGNAL(finished()), this,
                SLOT(slotFileListFinished()));
        viewer = fullScreenTopLevel;
    }
    viewer->show();
    QListBoxItem *i = fileList->item(0);
    fileList->setCurrentItem(i);
    fileList->setSelected(i, true);
    imageBuffer->slotSetFile(((KIFImageListItem *)i)->fileName());
}

void KIFApplication::slotFileListFinished()
{
    delete viewer;
    delete imageBuffer;
    if(browserAfterList){
        loadBrowser(list);
        delete list;
    }
    else
        quit();
}

void KIFApplication::slotFileListSelected(QListBoxItem *i)
{
    imageBuffer->slotSetFile(((KIFImageListItem *)i)->fileName());
}

void KIFApplication::loadEditor(const QString &fileStr)
{
    static QWidget *(*init_ptr)(const QString &) = NULL;

    if(!init_ptr){
        if(!dlregistered){
            dlregistered = true;
            lt_dlinit();
        }
        QString libPath = KGlobal::dirs()->findResource("lib",
                                                        "libpixie_editor.la");
        editorhandle = lt_dlopen(libPath.latin1());
        if(!editorhandle){
            KMessageBox::sorry(NULL,
                               i18n("Cannot dynamic load the editor!"),
                               i18n("Pixie Plugin Error"));
            return;
        }
        lt_ptr init_func = lt_dlsym(editorhandle, "init");
        init_ptr = (QWidget *(*)(const QString &))init_func;
        if(!init_ptr){
            KMessageBox::sorry(NULL,
                               i18n("Invalid editor plugin!"),
                               i18n("Pixie Plugin Error"));
            lt_dlclose(editorhandle);
        }
        else{
            init_ptr(fileStr);
        }

    }
    else{
        qWarning("Editor plugin already loaded.");
        init_ptr(fileStr);
    }
}

void KIFApplication::loadBrowser(KIFFileList *fileList)
{
    if(!catMgr)
        catMgr = new CatagoryManager;
    static QWidget *(*init_ptr)(KIFFileList *) = NULL;
    if(!init_ptr){
        if(!dlregistered){
            dlregistered = true;
            lt_dlinit();
        }

        QString libPath = KGlobal::dirs()->findResource("lib",
                                                        "libpixie_browser.la");
        browserhandle = lt_dlopen(libPath.latin1());
        if(!browserhandle){
            KMessageBox::error(NULL,
                               i18n("Cannot dynamic load the browser!"),
                               i18n("Pixie Plugin Error"));
            exit(1);
        }
        lt_ptr init_func = lt_dlsym(browserhandle, "init");
        init_ptr = (QWidget *(*)(KIFFileList *))init_func;
        if(!init_ptr){
            KMessageBox::error(NULL,
                               i18n("Invalid browser plugin!"),
                               i18n("Pixie Plugin Error"));

            lt_dlclose(browserhandle);
            browserhandle = 0;
            exit(2);
        }
        else{
            qWarning("Running browser init pointer");
            init_ptr(fileList);
            qWarning("Browser running");
        }
    }
    else{
        qWarning("Browser plugin already loaded");
        init_ptr(NULL);
    }
}

// Nasty image handling hacks. Qt only allows images to be identified by the
// first 14 bytes of the image header! This obviously doesn't work for TGA,
// PCD, etc.... it's not long enough. You also can't identify images by
// extension. So we include replacement functions for image read, write, and
// some utilities.
//
// XWD is in here because it's header conflicts with ICO, so we need to go
// by extension
bool kifSaveImage(QImage &img, const QString &fileName,
                  const char *format, int quality)
{
    // TODO: psudeocolor versions
    Image *magickImg;
    ImageInfo info;
    bool result;

    GetImageInfo(&info);
    strcpy(info.filename, fileName.latin1());
    if(img.isNull())
        return(false);

    if(qstrcmp(format, "PCD") == 0){
        if(img.depth() < 32)
            img = img.convertDepth(32);
        qWarning("In PCD save for file: %s", info.filename);
        magickImg = CreateImage(img.width(), img.height(), "BGRA",
                                (StorageType)0, img.bits());
        if(magickImg == NULL)
            return(false);
        strcpy(magickImg->filename, fileName.latin1());
        result = WritePCDImage(&info, magickImg);
        DestroyImage(magickImg);
        return(result);
    }
    else if(qstrcmp(format, "TGA") == 0){
        if(img.depth() < 32)
            img = img.convertDepth(32);
        qWarning("In TGA save for file: %s", info.filename);
        magickImg = CreateImage(img.width(), img.height(), "BGRA",
                                (StorageType)0, img.bits());
        if(magickImg == NULL)
            return(false);
        strcpy(magickImg->filename, fileName.latin1());
        result = WriteTGAImage(&info, magickImg);
        DestroyImage(magickImg);
        return(result);
    }
    else if(qstrcmp(format, "MIFF") == 0){
        if(img.depth() < 32)
            img = img.convertDepth(32);
        qWarning("In MIFF save for file: %s", info.filename);
        magickImg = CreateImage(img.width(), img.height(), "BGRA",
                                (StorageType)0, img.bits());
        if(magickImg == NULL)
            return(false);
        strcpy(magickImg->filename, fileName.latin1());
        result = WriteMIFFImage(&info, magickImg);
        DestroyImage(magickImg);
        return(result);
    }
    else if(qstrcmp(format, "XWD") == 0){
        if(img.depth() < 32)
            img = img.convertDepth(32);
        qWarning("In XWD save for file: %s", info.filename);
        magickImg = CreateImage(img.width(), img.height(), "BGRA",
                                (StorageType)0, img.bits());
        if(magickImg == NULL)
            return(false);
        strcpy(magickImg->filename, fileName.latin1());
        result = WriteXWDImage(&info, magickImg);
        DestroyImage(magickImg);
        return(result);
    }
    else if(qstrcmp(format, "PICT") == 0){
        if(img.depth() < 32)
            img = img.convertDepth(32);
        qWarning("In PICT save for file: %s", info.filename);
        magickImg = CreateImage(img.width(), img.height(), "BGRA",
                                (StorageType)0, img.bits());
        if(magickImg == NULL)
            return(false);
        strcpy(magickImg->filename, fileName.latin1());
        // let minimagick decide on the handler
        result = WritePICTImage(&info, magickImg);
        DestroyImage(magickImg);
        return(result);
    }
    else
        return(img.save(fileName, format, quality));
}

bool kifLoadImage(QImage &img, const QString &fileName, const char *format)
{
    QString type(format == NULL ? kifImageType(fileName) : QString(format));
    if(type == "TGA" || type == "XWD" || type == "PCD" || type == "MIFF" ||
       type == "PICT"){
        img.reset();
        Image *magickImg;
        ImageInfo info;

        GetImageInfo(&info);
        strcpy(info.filename, fileName.latin1());
        qWarning("In kifloadimage for file: %s", info.filename);
        magickImg = ReadImage(&info);
        if(magickImg == NULL){
            return(false);
        }
        qWarning("Image read");
        img.create(magickImg->columns, magickImg->rows, 32);
        GetPixels(magickImg, 0, 0, img.width(), img.height(),
                  "BGRA", (StorageType)0, img.bits());
        DestroyImage(magickImg);
        return(true);
    }
    else
        return(img.load(fileName, format));
}

bool kifLoadPixmap(QPixmap &pix, const QString &fileName, const char *format,
                   QPixmap::ColorMode mode)
{
    QString type(format == NULL ? kifImageType(fileName) : QString(format));
    if(type == "TGA" || type == "XWD" || type == "PCD" || type == "MIFF" ||
       type == "PICT"){
        QImage img;
        if(!kifLoadImage(img, fileName, type.ascii()))
            return(false);
        pix.convertFromImage(img, mode);
        return(true);
    }
    else
        return(pix.load(fileName, format, mode));
}


QString kifImageType(const QString &filename)
{
    QFileInfo fi(filename);
    QString extStr = fi.extension(false).lower();
    if(extStr == "xcf")
        return("XCF");
    else if(extStr == "pcx")
        return("PCX");
    else if(extStr == "xwd")
        return("XWD");
    else if(extStr == "pcd")
        return("PCD");
    else if(extStr == "tga")
        return("TGA");
    else if(extStr == "miff")
        return("MIFF");
    else if(extStr == "pict")
        return("PICT");
    else
        return(KImageIO::type(filename));
}

QString kifImagePattern(bool write)
{
    QString pattern = KImageIO::pattern(write ? KImageIO::Writing :
                                        KImageIO::Reading);
    pattern += '\n';
    if(write){
        pattern += i18n("*.gif *.GIF|CompuServe GIF Pictures\n");
        pattern += i18n("*.tga *.TGA|Truevision Targa Pictures\n");
        pattern += i18n("*.pcx *.PCX|ZSoft Paintbrush Pictures\n");
        pattern += i18n("*.pcd *.PCD|PhotoCD Pictures\n");
        pattern += i18n("*.pict *.PICT|Apple QuickDraw Pictures\n");
        pattern += i18n("*.miff *.MIFF|ImageMagick Pictures\n");
        pattern += i18n("*.xwd *.XWD|X Windows Dump Pictures");
    }
    else{
        pattern += i18n("*.gif *.GIF|CompuServe GIF Pictures\n");
        pattern += i18n("*.tga *.TGA|Truevisions Targa Pictures\n");
        pattern += i18n("*.xcf *.XCF|GNU Image Manipulation Pictures\n");
        pattern += i18n("*.pcx *.PCX|ZSoft Paintbrush Pictures\n");
        pattern += i18n("*.pcd *.PCD|PhotoCD Pictures\n");
        pattern += i18n("*.pict *.PICT|Apple QuickDraw Pictures\n");
        pattern += i18n("*.miff *.MIFF|ImageMagick Pictures\n");
        pattern += i18n("*.xwd *.XWD|X Windows Dump Pictures");
    }
    return(pattern);
}

QString kifImageTypeToExtension(const char *type)
{
    QString str = QString(type).lower();
    if(str == "tiff")
        str = "tif";
    else if(str == "jpeg")
        str = "jpg";
    return(str);
}

QStrList kifImageOutputFormats( )
{
    QStrList iotypes = QImageIO::outputFormats();
    iotypes.append("TGA");
    iotypes.append("PCD");
    iotypes.append("MIFF");
    iotypes.append("XWD");
    iotypes.append("PICT");
    return(iotypes);
}













