#ifndef __KIF_APPLICATION_H
#define __KIF_APPLICATION_H

#include <kapplication.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qcursor.h>
#include <qasciidict.h>
#include <ltdl.h>

class KIFFileList;
class KIFImage;
class QListBoxItem;
class CatagoryManager;

void qimageio_readXCF(QImageIO *io);

QString kifImageType(const QString &filename);
QString kifImagePattern(bool write);
QString kifImageTypeToExtension(const char *type);
bool kifSaveImage(QImage &img, const QString &fileName, const char *format,
                  int quality=-1);
bool kifLoadImage(QImage &img, const QString &fileName, const char *format=0);
bool kifLoadPixmap(QPixmap &pix, const QString &fileName, const char *format=0,
                   QPixmap::ColorMode mode = QPixmap::Auto);
QStrList kifImageOutputFormats();

class KIFApplication : public KApplication
{
    Q_OBJECT
public:
    KIFApplication(int argc, char **argv);
    ~KIFApplication();
    CatagoryManager* catagoryManager();
    void startBusyCursor();
    void incBusyCursor();
    void stopBusyCursor();
    void startInteractiveBusyCursor();
    void incInteractiveBusyCursor();
    void stopInteractiveBusyCursor();
    QCursor& busyCursor1(){return(busy1Cursor);}
    QCursor& busyCursor2(){return(busy2Cursor);}
    QCursor& interactiveBusyCursor1(){return(interactiveBusy1Cursor);}
    QCursor& interactiveBusyCursor2(){return(interactiveBusy2Cursor);}
    void hideWindows();
    void showWindows();
    void runInitialFileList(KIFFileList *fileList);
    void setBrowserAfterList(bool en){browserAfterList = en;}
    void loadBrowser(KIFFileList *fileList); // run only once!
    void loadEditor(const QString &fileStr);
    QAsciiDict<QImage>* mimetypeImageDict(){return(&mimeImageDict);}
signals:
    void hideAllWindows();
    void showAllWindows();
protected slots:
    void slotFileListFinished();
    void slotFileListSelected(QListBoxItem *i);
protected:
    bool cursorState, inBusy;
    QCursor busy1Cursor, busy2Cursor;
    QCursor interactiveBusy1Cursor, interactiveBusy2Cursor;

    KIFFileList *list;
    KIFImage *imageBuffer;
    QWidget *viewer;
    CatagoryManager *catMgr;
    bool browserAfterList;
    lt_dlhandle browserhandle, editorhandle;
    bool dlregistered;

    QAsciiDict<QImage> mimeImageDict;
};

KIFApplication* kifapp();

#endif

