#include "ifapp.h"
#include "filelist.h"
#include <qfileinfo.h>
#include <qdir.h>
#include <qtextstream.h>
#include <kmessagebox.h>
#include <kurl.h>
#include <kmimetype.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <klocale.h>

static const char *description =
	I18N_NOOP("Mosfet's Pixie Image and Photo manager.");

static KCmdLineOptions options[] =
{
    { "a", 0, 0},
    { "all", I18N_NOOP("Load all images in the current directory"), 0},
    { "d", 0, 0},
    { "dir <file>", I18N_NOOP("Load all images in a given directory"), 0},
    { "m", 0, 0},
    { "mini", I18N_NOOP("Exit after viewing cmdline images (don't load browser plugin)"), 0},
    { "f", 0, 0},
    { "filelist <file>", I18N_NOOP("Load the given FileList file"), 0},
    { "+[file(s)]", I18N_NOOP("File(s) or URL(s) to open"), 0 },
    { 0, 0, 0 }
};

int main( int argc, char **argv )
{
    KAboutData aboutData( "pixie", I18N_NOOP("Pixie"),
    "1.0", description, KAboutData::License_BSD,
    "(c) 2000 Daniel Duley <mosfet@kde.org>");

    KCmdLineArgs::init( argc, argv, &aboutData );
    KCmdLineArgs::addCmdLineOptions(options);

    KIFApplication app(argc, argv);

    KIFFileList *fileList = new KIFFileList;
    fileList->hide();

    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
    if(args->isSet("mini")){
        app.setBrowserAfterList(false);
    }
    if(args->isSet("filelist")){
        QFile f(args->getOption("filelist"));
        if(f.open(IO_ReadOnly)){
            fileList->clear();
            QTextStream t(&f);
            if(t.readLine() != "PixieList"){
                KMessageBox::error(NULL, i18n("Invalid FileList file!"),
                                   i18n("Pixie FileList Error"));
                f.close();
                return(1);
            }
            while (!t.eof()){
                fileList->slotAppend(t.readLine());
            }
            f.close();
        }
        else{
            KMessageBox::error(NULL, i18n("Cannot open FileList file!"),
                               i18n("Pixie FileList Error"));
            return(1);
        }
    }
    else{
        for( int i = 0; i < args->count(); i++ ){
            QFileInfo fi(args->arg(i));
            if(fi.isDir())
                QDir::setCurrent(fi.absFilePath());
            else
                fileList->slotAppend(fi.absFilePath());
        }
        if(args->isSet("all") || args->isSet("dir")){
            QDir d(args->isSet("all") ? args->cwd() :
                   args->getOption("dir"));
            if(!d.exists())
                qWarning("Invalid directory specified!");
            else{
                d.setFilter(QDir::Files);
                const QFileInfoList *list = d.entryInfoList();
                QFileInfoListIterator it(*list);
                QFileInfo *fi;
                KURL url;
                while((fi=it.current())){
                    url.setPath(fi->absFilePath());
                    if(KMimeType::findByURL(url, true, true)->name().left(6) ==
                       "image/"){
                        fileList->slotAppend(fi->absFilePath());
                    }
                    ++it;
                }
            }
        }
    }
    if(fileList->count() == 0){
        app.loadBrowser(fileList);
    }
    else
        app.runInitialFileList(fileList);

    /*
    if(args->isSet("thumbnail"))
        w->slotThumbnails(); */
    return(app.exec());
}
